<?php
namespace App\Http\Controllers;

use App\Http\ResponseHelper;
use Config;
use DB;
use Illuminate\Http\Request;
use Log;
use App\Http\Controllers\ShopifyMultiPass;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;

class CommonController extends Controller
{
    private $helper;
    private $shopify_api_version;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Function for creating a customer in Shopify
     * This function is not for a quiz user
     * params : email, password
     */
    public function createShopifyCustomer(Request $request)
    {

        $options = new Options();
        $storeDomain = Config::get('shopify.shopDomainQuiz');
        $accessToken = Config::get('shopify.passwordQuiz');
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $api = new BasicShopifyAPI($options);
        $api->setSession(new Session($storeDomain, $accessToken));

        $email = !empty($request->input('email')) ? trim($request->input('email')) : '';
        $customerPassword = !empty($request->input('password')) ? trim($request->input('password')) : '';

        Log::channel('commonControllerLog')->info('Email:'.$email.', Password:'.$customerPassword);

        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($customerPassword)) {
            $returnData = [
                'error' => 'Parameter missing - password',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if (strlen($customerPassword) < 5) {
            $returnData = [
                'error' => 'Password is too short (minimum is 5 characters)',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $userEmailDetails = DB::table('users')
            ->select('*')
            ->where('users.user_email', '=', $email)
            ->get();

        if (isset($userEmailDetails[0]->user_email)) {

            $returnData = [
                'error' => 'The email has already been taken',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        } else {

            $tags = [];
            $tags[] = "passwordSet";

            $params = array(
                'customer' => array(
                    'email' => $email,
                    "send_email_welcome" => "false",
                    "password" => $customerPassword,
                    "password_confirmation" => $customerPassword,
                    "tags" => $tags,
                    // "accepts_marketing" => "true"
                    "email_marketing_consent" => array(
                        "state" => "subscribed",
                        "opt_in_level" => "single_opt_in"
                    )
                ),
            );

            Log::channel('commonControllerLog')->info('---------customer creation endpoint---------');
            Log::channel('commonControllerLog')->info('Request: '.json_encode($params));

            $retry = false;
            $try = 1;
            $result = '';
            do {
                Log::channel('commonControllerLog')->info($try . ' try-create shopify customer ');
                $result = $api->rest('POST', '/admin/api/' . $this->shopify_api_version . '/customers.json', $params);

                if (!isset($result['body']['customer']['id'])) {
                    $retry = true;
                    $try++;
                }

            } while ($retry && $try <= 3);
            Log::channel('commonControllerLog')->info('Response: '.json_encode($result));

            if (isset($result['body']['customer']['id'])) {
                
                /* code changes for multipass login */
                $customer_login_data = array(
                    "email" => $email
                );
                $multiPassToken = env('MULTI_PASS_TOKEN', true);
                $multipass = new ShopifyMultipass($multiPassToken);
                $token = $multipass->generate_token($customer_login_data);
    
                /* code changes for multipass login */
                
                $returnData = [
                    'success' => 'Customer successfully created.',
                    'multi_pass_token' => ($token != '')?$token:"",
                    "customer_id" => $result['body']['customer']['id']
                ];
                
                Log::channel('commonControllerLog')->info('Customer successfully created.');
                return $this->helper->returnData(200, "success", $returnData);
            } else {

                // echo(json_encode($result));die;
                if (isset($result['errors']['body']['errors'])) {
                    $returnData = [
                        'error' => 'Failed to create the customer in Shopify',
                    ];
                    Log::channel('commonControllerLog')->info('Failed to create the customer in Shopify');

                } else {
                    $returnData = [
                        'error' => "Invalid email or password",
                    ];
                    Log::channel('commonControllerLog')->info('Invalid email or password');
                }
                return $this->helper->returnData(600, "error", $returnData);

            }
        }
    }

}
