<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Log;
use Illuminate\Http\Request;
// use App\Models\Shopify\ExtentedShopifyAPI;
// use OhMyBrew\BasicShopifyAPI;
use Config;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use App\Models\Shopify\Customer;
use App\Models\Shopify\Order;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;

class CustomerController extends Controller {

    private $customerObj;
    private $orderObj;
    private $iterableObj;
    private $rechargeObj;
    private $helper;
    private $api;
    private $shopify_api_version;

    function __construct() {

        $this->customerObj = new Customer();
        $this->orderObj = new Order();
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
        $options = new Options();
        $storeDomain = Config::get('shopify.shopDomainQuiz');
        $accessToken = Config::get('shopify.passwordQuiz');
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session($storeDomain, $accessToken));
    }

    /**
     * Get customer activation status
     */
    public function getCustomerActivationStatus(Request $request) {
        
        $email = !empty($request->input('email')) ? trim($request->input('email')) : '';
        if(empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email'
            ];

            Log::channel('customerControllerLog')->info("getCustomerActivationStatus: Parameter missing - email");
            $this->sendResponse(600, "error", $returnData );
        }
       
        $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

            $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('customerControllerLog')->info("getCustomerActivationStatus: Failed to get the 
            customer details from Shopify(emailAddress:".$email."), Error: (".$error.")");
            $this->sendResponse(600, "error", $returnData);
        }

        // $params = [];
        // $customerDetails = $this->api->rest('GET', '/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$email, $params);by login 
        
        if(!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            
            $returnData = [
                "customer_available" => false,
                "activation_status" => null
            ];

            Log::channel('customerControllerLog')->info("getCustomerActivationStatus: The Shopify API 
            response is empty(Email:".$email);

            $this->sendResponse(200, "success", $returnData );
        } else {
            $customer = [];
            foreach ($customerDetails['customers'] as $cust) {
                if (trim(strtolower($cust['email'])) == strtolower($email)) {
                    $customer = $cust;
                    break;
                }
            }

            $status = !empty($customer['state']) ? trim($customer['state']) : '';

            $returnData = [
                "customer_available" => true,
                "activation_status" => $status
            ];

            Log::channel('customerControllerLog')->info("getCustomerActivationStatus: Successfully returned 
            the customer activation status");

            $this->sendResponse(200, "success", $returnData);
        }
    }


    /**
     * Send endpoint response
     */
    public function sendResponse($status,$message, $body) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "body" => $body
        );
        echo json_encode($response);
        die();
    }

    /**
     * Get activation URL
     */
    public function getActivationUrl(Request $request) {
       
        $email = !empty($request->input('email')) ? trim($request->input('email')) : '';
        
        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email'
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: Parameter missing - email");
            $this->sendResponse(600, "error", $returnData);
        }
       
        $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: Failed to get the 
            customer details from Shopify(emailAddress:".$email."), Error: (".$error.")");
            $this->sendResponse(600, "error", $returnData);
        }


        // $params = NULL;
        // $result = $this->api->rest('GET', 
        //         htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$email), $params); 

        // print_r($customerDetails); die;
                
        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            
            $returnData = [
                'error' => 'email not found'
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: The Shopify API response is empty(Email:".$email);

            $this->sendResponse(600, "error", $returnData);
        }

        $customer = '';
        foreach($customerDetails['customers'] as $cust) {
            if(strtolower(trim($cust['email'])) == strtolower($email)) {
                $customer = $cust;
                break;
            }
        }

        if (empty($customer)) {

           $returnData = [
                'error' => 'email not found'
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: Not found the searched email in the Shopify API 
            response(Email:".$email);
            $this->sendResponse(600, "error", $returnData);
        }

        if (isset($customer['state']) && strtolower(trim($customer['state'])) == "enabled") {
            $returnData = [
                "activation_status" => "enabled"
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: Already enabled(Email:".$email);
             $this->sendResponse(200, "success", $returnData);
        }

        $params = NULL;
        $result = $this->api->rest('POST', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/'.$customer['id'].'/account_activation_url.json'), $params);
        
        
        if (!isset($result['body']['account_activation_url']) || empty($result['body']['account_activation_url'])) {

            $returnData = [
                'error' => 'Failed to create activation URL'
            ];

            Log::channel('customerControllerLog')->info("getActivationUrl: Failed to create activation URL(Email:".$email);
            $this->sendResponse(600, "error", $returnData);
        } else {
            
            $activationURL = $result['body']['account_activation_url'];
            
            $iterableMailRequest = array(

                "campaignId" => intval(env('ITERABLE_CUSTOMER_ACTIVATION_CAMPAIGN_ID',true)),
                "recipientEmail" => $email,
                "dataFields" => array(
                    "customUrl" => $activationURL,
                )
            );

            $iterableResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($iterableResponse->code)) == 'success') {           
                
                Log::channel('customerControllerLog')->info("getActivationUrl: Successfully triggered the mail(Email:".$email.")");
    
            } else {

                $errorMessage = !empty($iterableResponse->code) ? $iterableResponse->code : '';
                Log::channel('customerControllerLog')->info("getActivationUrl: Failed to send the mail(Email:".$email.", Error:".$errorMessage.")");

                $returnData = [
                    "activation_status" => "disabled",
                    "mail_status" => "Not send"
                ];
   
               $this->sendResponse(600, "error", $returnData);
            }

             $returnData = [
                 "activation_status" => "disabled",
                 "mail_status" => "sent"
             ];

            $this->sendResponse(200, "success", $returnData);
        }
    }
    
    /**
     * Endpoint response
     */
    public function returnResponse($status,$message, $body, $callback) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "body" => $body
        );
        echo $callback . '(' . json_encode($response) . ')';
        die();
    }
    
    /**
     * Get recharge hash
     */
    public function getRechargeHash(Request $request) {
        
        if (empty($request->input('data'))) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];

            Log::channel('customerControllerLog')->info("getRechargeHash: Endpoint parameters are missing");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }

        $receivedData = $request->input('data');
        $receivedData = json_decode($receivedData);
        
        $rechargeHash = null;

        $email = !empty($receivedData->email) ? trim($receivedData->email) : '';
        $customerId = !empty($receivedData->customerId) ? trim($receivedData->customerId) : '';
        //print_r($receivedData);die;
        
        if (!isset($email) || empty($email)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];

            Log::channel('customerControllerLog')->info("getRechargeHash: Parameter missing - email");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }

         if (!isset($customerId) || empty($customerId)) {
            
            $returnData = [
                'message' => 'Parameter missing - customerId'
            ];

            Log::channel('customerControllerLog')->info("getRechargeHash: Parameter missing - customerId");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }
       
        $params = NULL;
        $result = $this->api->rest('GET', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json'), $params); 

        
        if(!isset($result['body']['customer']) || empty($result['body']['customer'])) {
            
            $returnData = [                
                "message" => "Invalid Customer details."            
            ]; 

            Log::channel('customerControllerLog')->info("getRechargeHash: Failed to get the customer details from 
            Shopify(ShCustomerId:".$customerId);

            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        } 

        $customer = $result['body']['customer'];

        if (trim(strtolower($customer['email'])) == strtolower($email) && trim($customer['id']) == $customerId) {

            $rechargeResults = $this->rechargeObj->getCustomerDetailsByShopifyCustomerId($customerId);
            
            if (isset($rechargeResults->error) || isset($rechargeResults->errors) || empty($rechargeResults->customers)) {
        
                $error = "Unknown error";
                if (!empty($rechargeResults->error)) {

                    $error = json_encode($rechargeResults->error);
                } elseif (!empty($rechargeResults->errors)) {
                    
                    $error = json_encode($rechargeResults->errors);
                }
                if (empty($rechargeResults->customers)) {

                    $error = "Not find the customer details in recharge(CustomerId - " . $customerId . ")";
                }

                $returnData = [                
                    "message" => "Failed to get the customer details from recharge."            
                ];         

                Log::channel('customerControllerLog')->info("getRechargeHash:Failed to get the customer details from recharge
                (ShCustomerId:".$customerId.", Error:".$error);

                $this->returnResponse(600, "error", $returnData, $request->input('callback'));
            } 

            foreach($rechargeResults->customers as $rechargeCustomer) {

                if(trim($rechargeCustomer->shopify_customer_id) == $customerId) {

                    $rechargeHash = $rechargeCustomer->hash;
                    break;
                }
            }
        }

        if($rechargeHash != null){

            $returnData = [
                "customerEmail" => $email,
                "rechargeHash"  => $rechargeHash            
            ];

            Log::channel('customerControllerLog')->info("getRechargeHash: Successfully returned the recharge hash
            (ShCustomerId:".$customerId);

            $this->returnResponse(200, "success", $returnData, $request->input('callback'));
        } else {

            $returnData = [                
                "message" => "Invalid customer details"            
            ];         

            Log::channel('customerControllerLog')->info("getRechargeHash: Failed to return the recharge hash
            (ShCustomerId:".$customerId);
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }            
    }
    
    /**
     * Get customer standing details
     */
    public function getCustomerStanding(Request $request) {
        
        if (empty($request->input('data'))) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Endpoint parameters are missing");

            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }
        
        $receivedData = $request->input('data');
        $receivedData = json_decode($receivedData);

        $requestEmail = !empty($receivedData->email) ? trim($receivedData->email) : '';
        $currentOrderId = !empty($receivedData->orderId) ? trim($receivedData->orderId) : '';
        $customerAvailable = false;
        
        // print_r($receivedData);die;
        
        if (!isset($requestEmail) || empty($requestEmail)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Parameter missing - email");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }

         if (!isset($currentOrderId) || empty($currentOrderId)) {
            
            $returnData = [
                'message' => 'Parameter missing - orderId'
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Parameter missing - orderId");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }
        
        // $requestEmail = trim($receivedData->email);
        // $currentOrderId = trim($receivedData->orderId);
        // $customerAvailable = false;
        
        $customerDetails = $this->customerObj->getCustomerDetailBySearch($requestEmail);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $requestEmail . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Failed to get the 
            customer details from Shopify(emailAddress:".$requestEmail."), Error: (".$error.")");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }

        // $customerDetails = $this->api->rest('GET',
        //             htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$requestEmail), null);

        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            $returnData = [
                'message' => 'No such customer found in Shopify',
                "customerStanding" => null 
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Customer details from Shopify are empty(Email:".$requestEmail.")");
            $this->returnResponse(200, "success", $returnData, $request->input('callback'));
        }

        $customers = $customerDetails['customers'];            
        foreach ($customers as $customer) {

            if (trim(strtolower($customer['email'])) == strtolower($requestEmail)) {

                $customerAvailable = true;
            }
        }
        if (!$customerAvailable) {
            $returnData = [
                'message' => 'No such customer found in Shopify',
                "customerStanding" => null 
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Searched customer not found in Shopify response(Email:".$requestEmail.")");
            $this->returnResponse(200, "success", $returnData, $request->input('callback'));
        }
       
       
        $customerOrderDetails = $this->orderObj->getOrderDetailsByEmail($requestEmail);

        if (!empty($customerOrderDetails['errors']) || !empty($customerOrderDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerOrderDetails['error'])) {

                $error = json_encode($customerOrderDetails['error']);
            } elseif (!empty($customerOrderDetails['errors'])) {
                
                $error = json_encode($customerOrderDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the order details from Shopify (EmailID: " . $requestEmail . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: Failed to get the order details from Shopify
            (emailAddress:".$requestEmail."), Error: (".$error.")");
            $this->returnResponse(600, "error", $returnData, $request->input('callback'));
        }


        // $params = NULL;
        // $result = $this->api->rest('GET', 
        //         htmlentities('/admin/api/'.$this->shopify_api_version.'/orders.json?status=any&query='.$requestEmail), $params); 

        //print_r($customerOrderDetails['orders']); die;

        
        if($customerAvailable && (!isset($customerOrderDetails['orders']) || empty($customerOrderDetails['orders']))) {
            
            $returnData = [
                "customerStanding" => 'New'                           
            ];

            Log::channel('customerControllerLog')->info("getCustomerStanding: No orders in Shopify(Email:".$requestEmail.")");
            $this->returnResponse(200, "success", $returnData, $request->input('callback'));;
        } else {

            $orders = $customerOrderDetails['orders'];  
            $orderDetails = [];
            foreach ( $orders as $order) {

                if (trim(strtolower($order['email'])) == strtolower($requestEmail)) {

                    $orderDetails[] = $order['id'];
                }
            }
            

            if (count($orderDetails) > 1) {
                $returnData = [
                    "customerStanding" => 'Returning'                           
                ];

                Log::channel('customerControllerLog')->info("getCustomerStanding: The customer has more than 1 order in Shopify(Email:".$requestEmail.")");
                $this->returnResponse(200, "success", $returnData, $request->input('callback'));
            }    
            
            if (count($orderDetails) == 1 && in_array($currentOrderId,$orderDetails)) {
                $returnData = [
                    "customerStanding" => 'New'                           
                ];

                Log::channel('customerControllerLog')->info("getCustomerStanding: No orders in Shopify(Email:".$requestEmail.")");
                $this->returnResponse(200, "success", $returnData, $request->input('callback'));
            } else {

                $returnData = [
                    "customerStanding" => 'New'                           
                ];

                Log::channel('customerControllerLog')->info("getCustomerStanding: No orders in Shopify(Email:".$requestEmail.")");
                $this->returnResponse(200, "success", $returnData, $request->input('callback'));
            }
            
        }

    }

    // public function executeCurl($url,$type,$data) {
	    
	//     $ch = curl_init();
	//     curl_setopt($ch, CURLOPT_URL, $url);
    //     curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    //     curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
    //     curl_setopt($ch, CURLOPT_TIMEOUT, 30);
	//     curl_setopt($ch, CURLOPT_HEADER, FALSE);
	//     curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	//             "Content-Type: application/json"
	//     ));
    //     curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE); // temporarily disable SSL verification
    //     curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE); // temporarily disable SSL verification

	//     if ($type == "POST"){
	//         curl_setopt($ch, CURLOPT_POST, TRUE);
    //         curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
	//     }

	//     $result = curl_exec($ch);

	//     if(curl_errno($ch)){
	//         echo 'Request Error:' . curl_error($ch);
	//         return false;
    //     }
        
    //     Log::channel('customerControllerLog')->info("curl result  => ".$result);
	//     return $result;
    // }
}
