<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use App\Models\Gorgias;
use App\Models\IterableApi;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Log;

class GorgiasTicketCreationCron extends Controller
{
    private $rechargeObj;
    private $iterableObj;
    private $gorgiasObj;
    private $logDirectory;
    private $logPath;
    private $logFiles;
    private $nurishTicketCreationReport;
    private $natureMadeTicketCreationReport;
    private $failedChargeCountToCreateTicket;
    private $nurishStoreDomain;
    private $natureMadeStoreDomain;
    private $ticketRecipientName;
    private $nurishTicketRecipientEmail;
    private $natureMadeTicketRecipientEmail;
    private $paymentDeclinedMessage;
    private $subscriptionIncCount = 0;
    private $nurishTicketCreationProcessedList = [];
    private $natureMadeTicketCreationProcessedList = [];

    public function __construct() {

        $this->rechargeObj = new RechargeApi('cronJobs');
        $this->iterableObj = new IterableApi();
        $this->gorgiasObj = new Gorgias();
        $this->failedChargeCountToCreateTicket = env("FAILED_CHARGE_COUNT_TO_CREATE_GORGIAS_TICKET", true);
        $this->nurishStoreDomain = env("NURISH_STORE_DOMAIN", true);
        $this->natureMadeStoreDomain = env("NATURE_MADE_STORE_DOMAIN", true);
        $this->ticketRecipientName = env("TICKET_RECIPIENT_NAME", true);
        $this->nurishTicketRecipientEmail = env("NURISH_RECIPIENT_EMAIL", true);
        $this->natureMadeTicketRecipientEmail = env("NATURE_MADE_RECIPIENT_EMAIL", true);
        $this->paymentDeclinedMessage = "Customer <customer_name> needs to update card";
    }

    /**
     * Function to process Gorgias ticket creation
     */
    public function processGorgiasTicketCreation() {

        $this->processSubscriptions('Nurish');
        $this->sendCronReport('Nurish');
        $this->processSubscriptions('Naturemade');
        $this->sendCronReport('Naturemade');
    }

    /**
     * Function to get processed subscription charges from recharge and create Gorgias ticket
     */
    public function processSubscriptions($type = null) {

        Log::channel('gorgiasCronLog')->info($type.": Processing started");

        $endDate = date("Y-m-d");
        $startDate = date('Y-m-d', strtotime('-2 day', strtotime($endDate)));
        // $endDate = "2021-03-26";
        // $startDate = "2021-03-24";
        $pageNo = 1;
        $limit  = 250;
        $nextPageLink = "";
        $chargeCount = 0;
        do {
            try {
                Log::channel('gorgiasCronLog')->info("Nurish: Read recharge charge records page -". $pageNo);
                $processedCharges = $this->rechargeObj->getChargesByCursorPagination($limit, $nextPageLink, $startDate, $endDate, $type);
            }
            catch (Exception $e) {
                $errorText = "An error occurred when fetching the charges from the recharge";
                Log::channel('gorgiasCronLog')->info($type.": ". $errorText);
                die();
            }
            //sleep(1);
            $subscriptionCharges = [];
            $processedChargeData = [];
            if (!empty($processedCharges['charges']) && isset($processedCharges['charges']) && !empty($processedCharges)) {

                $processedChargeData = $processedCharges;
                $chargeCount = 0;
                $subscriptionId = '';

                foreach ($processedChargeData['charges'] as $processedChargeValue) {

                    $subscriptionId = !empty($processedChargeValue['line_items'][0]['subscription_id']) ? 
                    trim($processedChargeValue['line_items'][0]['subscription_id']) : '';
                    
                    $this->subscriptionIncCount++;

                    if (!empty($subscriptionId)) {

                        Log::channel('gorgiasCronLog')->info($type.": ".$this->subscriptionIncCount.") Processing Email:".
                        $processedChargeValue['email']." subscriptionID:".$subscriptionId);

                        sleep(1);
                        $subscriptionCharges = $this->rechargeObj->getChargesBySubscriptionIdInDesc($subscriptionId, $type);

                        if (!empty($subscriptionCharges->error) || !empty($subscriptionCharges->errors) ||
                            empty($subscriptionCharges) || empty($subscriptionCharges->charges)) {

                            $error  = "Unknown error";
                            if (!empty($subscriptionCharges->error)) {
                                $error = json_encode($subscriptionCharges->error);
                            } else if (!empty($subscriptionCharges->errors)) {
                                $error = json_encode($subscriptionCharges->errors);
                            }
                            Log::channel('gorgiasCronLog')->info($type.": ".$this->subscriptionIncCount.") Failed to get the 
                            charges of the subscription(subscriptionID: ". $subscriptionId);
                            continue;
                        }

                        $failedChargeCount = 0;
                        $failedChargeCountStatus = 0;
                        $chargeIncCount = 0;
                        $firstSubChargeDetails = [];
                        foreach ($subscriptionCharges->charges as $chargeKey => $chargeValue) {

                            $chargeStatus = !empty($chargeValue->status) ? trim(strtolower($chargeValue->status)) : "";
                            if (($chargeStatus == "queued" && $failedChargeCount != 0) || ($chargeStatus == "success")) {

                                if ($failedChargeCount < $this->failedChargeCountToCreateTicket) {

                                    $failedChargeCountStatus = 0;
                                    break;
                                }
                            }
                            if ($failedChargeCount >= $this->failedChargeCountToCreateTicket) {
                                $failedChargeCountStatus = 1;
                                break;
                            }

                            if ($chargeIncCount == 0) {
                                $firstSubChargeDetails = $chargeValue;
                            }

                            if ($chargeStatus == "error") {
                                $failedChargeCount++;
                            }
                            $chargeIncCount++;
                        }

                        if ($failedChargeCountStatus == 0) {
                            Log::channel('gorgiasCronLog')->info($type.": ".$this->subscriptionIncCount.") Skipped: Not found ".$this->failedChargeCountToCreateTicket.
                            " failed charges in the subscription(failedChargeCount: ".$failedChargeCount.", subscriptionID: ". $subscriptionId .")");
                        }

                        $processedDataArray = [];
                        if (($failedChargeCountStatus == 1) && ($failedChargeCount >= $this->failedChargeCountToCreateTicket)) {

                            $gorgiasTicketCreatedResponse = $this->createTicketInGorgias($firstSubChargeDetails, $type);

                            if (!empty($gorgiasTicketCreatedResponse->error) ||  isset($gorgiasTicketCreatedResponse->error) || 
                            empty($gorgiasTicketCreatedResponse) || empty($gorgiasTicketCreatedResponse->id)) {
            
                                $error  = "Unknown error";
                                if (!empty($gorgiasTicketCreatedResponse->error)) {
                                    $error = json_encode($gorgiasTicketCreatedResponse->error);
                                } else if (!empty($gorgiasTicketCreatedResponse->errors)) {
                                    $error = json_encode($gorgiasTicketCreatedResponse->errors);
                                }
                                $processedDataArray = [
                                    "customerName" => $processedChargeValue['first_name']." ".$processedChargeValue['last_name'],
                                    "email" => $processedChargeValue['email'],
                                    "rechargeCustomerId" => $processedChargeValue['customer_id'],
                                    "subscriptionId" => $subscriptionId,
                                    "status" => "Failed",
                                ];
                                Log::channel('gorgiasCronLog')->info($type.": ".$this->subscriptionIncCount.") Failed to create the ticket in Gorgias
                                (subscriptionID: ". $subscriptionId ."), Error: ". $error);
                            } else {

                                $processedDataArray = [
                                    "customerName" => $processedChargeValue['first_name']." ".$processedChargeValue['last_name'],
                                    "email" => $processedChargeValue['email'],
                                    "rechargeCustomerId" => $processedChargeValue['customer_id'],
                                    "subscriptionId" => $subscriptionId,
                                    "status" => "Success",
                                ];
                                Log::channel('gorgiasCronLog')->info($type.": ".$this->subscriptionIncCount.") Successfully created the ticket in Gorgias
                                (subscriptionID: ". $subscriptionId);
                            }
                            if (strtolower($type) == "nurish") {
                                $this->nurishTicketCreationProcessedList[] = $processedDataArray;
                            } else if (strtolower($type) == "naturemade") {
                                $this->natureMadeTicketCreationProcessedList[] = $processedDataArray;
                            }
                        }
                    }
                }
                $chargeCount = count($processedChargeData['charges']);
                $nextPageLink = !empty($processedCharges['next']) ? trim($processedCharges['next']) : "";
                if (empty($nextPageLink)) {
                    break;
                }
            }

        $pageNo++;

        } while($chargeCount == $limit);

        Log::channel('gorgiasCronLog')->info($type.": Process completed");
    }

    /**
     * Create Gorgias ticket
     */
    public function createTicketInGorgias($chargeDetails, $processingProject = null) {

        $storeDomain = '';
        $ticketSubject = '';
        $projectName = '';
        $recipientEmail = '';
        $firstName = !empty($chargeDetails->first_name) ? trim($chargeDetails->first_name) : "";
        $lastName = !empty($chargeDetails->last_name) ? trim($chargeDetails->last_name) : "";
        $customerName = $firstName." ".$lastName;
        $this->paymentDeclinedMessage = str_replace("<customer_name>", "(".$customerName.")", $this->paymentDeclinedMessage);
        $ticketSubject = $processingProject." - ".$this->paymentDeclinedMessage;
        $projectName = $processingProject;

        if (strtolower($processingProject) == "nurish") {
            $storeDomain = $this->nurishStoreDomain;
            $recipientEmail = $this->nurishTicketRecipientEmail;

        } else if (strtolower($processingProject) == "naturemade") {
            $storeDomain = $this->natureMadeStoreDomain;
            $recipientEmail = $this->natureMadeTicketRecipientEmail;
        }
        $message = null;
        $ticketCreatedResponse = [];

        $totalChargeAmount = !empty($chargeDetails->total_price) ? trim($chargeDetails->total_price) : "0.00";
        $defaultErrorMessages = ["Customer needs to update credit card"];
        $messagesToTicket = ["Customer needs to update their credit card"];
        $errorMessage = !empty($chargeDetails->error) ? $chargeDetails->error : null;
        $rechargeUrl = "https://".$storeDomain."/tools/recurring/portal/";
        $customerHash = !empty($chargeDetails->customer_hash) ? trim($chargeDetails->customer_hash) : "";
        $customerUrl = !empty($customerHash) ? $rechargeUrl.$customerHash."/customer" : "";
        $subscriptionUrl = !empty($customerHash) ? $rechargeUrl.$customerHash."/subscriptions" : "";
        $creditCardInputUrl = !empty($customerHash) ? $rechargeUrl.$customerHash."/payment_source" : "";
        $customerEmail = !empty($chargeDetails->email) ? trim($chargeDetails->email) : "";
        $recipientName = $this->ticketRecipientName;

        if (!empty($errorMessage)) {
            foreach ($defaultErrorMessages as $errorKey => $errorValue) {
                if (strpos($errorMessage, $errorValue) !== false) {
                    $errorMessage = $messagesToTicket[$errorKey];
                    break;
                }
            }
        }

        $message .= "Hello Team,<br><br>";
        $message .= $customerName."'s account on ".$projectName." has experienced a billing error. ".$errorMessage."<br><br>";
        $message .= "Charge Failure Reason: ".$errorMessage."<br>";
        $message .= "Total Charge amount: $".$totalChargeAmount."<br><br>";

        $message .= "Customer URL: ".$customerUrl."<br>";
        $message .= "Subscription URL: ".$subscriptionUrl."<br>";
        $message .= "Credit Card Input URL: ".$creditCardInputUrl."<br>";

        $gorgiasBody = [
            "customer" => [
                    "email" => $customerEmail
            ],
            "messages" => [
                    [
                    "receiver" => [
                        "email" => $recipientEmail
                    ],
                    "sender" => [
                        "email" => $customerEmail
                    ],
                    "source" => [
                        "from" => [
                            "name" => $firstName." ".$lastName,
                            "address" => $customerEmail
                        ],
                        "to" => [
                            [
                                "name" => $recipientName,
                                "address" => $recipientEmail
                            ]
                        ],
                        "type" => "email"
                    ],
                    "via" => "email",
                    "body_html" => $message,
                    "channel" => "email",
                    "from_agent" => false,
                    "subject" => $ticketSubject
                    ]
            ],
            "channel" => "email",
            "via" => "email"
        ];

        $ticketCreatedResponse = $this->gorgiasObj->createGorgiasTicket(json_encode($gorgiasBody));
        return $ticketCreatedResponse;
    }

    /**
     * Create and send cron working report of Nurish and Nature-made
     */
    public function sendCronReport($type) {

        $iterableMailRequest = '';
        $commonContent = '';
        $iterableCampaignId = '';
        $reportCsvName = '';
        $projectType = strtolower($type);
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $csvFilePath = env("CRON_LOG_CSV_PATH", true);

        $subscriptionTotalProcessedCount = $this->subscriptionIncCount;
        $this->subscriptionIncCount = 0;
        $subscriptionTicketCreationProcessedCount = 0;
        $processedSubscriptionList = [];
        if ($projectType == "nurish") {
            $subscriptionTicketCreationProcessedCount = count($this->nurishTicketCreationProcessedList);
            $processedSubscriptionList = $this->nurishTicketCreationProcessedList;
            $iterableCampaignId = env('NURISH_GORGIAS_ITERABLE_CAMPAIGN_ID', true);
        } else if ($projectType == "naturemade") {
            $subscriptionTicketCreationProcessedCount = count($this->natureMadeTicketCreationProcessedList);
            $processedSubscriptionList = $this->natureMadeTicketCreationProcessedList;
            $iterableCampaignId = env('NATUREMADE_GORGIAS_ITERABLE_CAMPAIGN_ID', true);
        }

        if ($subscriptionTicketCreationProcessedCount > 30) {
            $this->setPaths();
            $this->getLogFilesList();
            $this->setWithLogFiles();
            $filePointerLog = '';
            if ($projectType == "nurish") {
                $reportCsvName = $this->nurishTicketCreationReport;
                $gorgiasTicketCreatedLog = $this->logFiles['nurishTicketProcessedReport']['filename'];
                $filePointerLog = fopen($gorgiasTicketCreatedLog, "a");
            } else if ($projectType == "naturemade") {
                $reportCsvName = $this->natureMadeTicketCreationReport;
                $gorgiasTicketCreatedLog = $this->logFiles['natureMadeTicketProcessedReport']['filename'];
                $filePointerLog = fopen($gorgiasTicketCreatedLog, "a");
            }
            foreach ($processedSubscriptionList as $processedSubKey => $processedSubValue) {
                fputcsv($filePointerLog, array(
                    $processedSubValue['customerName'],
                    $processedSubValue['email'],
                    $processedSubValue['rechargeCustomerId'],
                    $processedSubValue['subscriptionId'],
                    $processedSubValue['status']
                ));
            }

        }

        $recipientEmails = 'testing@p80w.com';

        //Processing subscription Gorgias ticket created report
        $subject = $type."-Dev Gorgias Ticket Creation Report - ".$reportSendingTime;
        
        if ($subscriptionTicketCreationProcessedCount == 0) {
            Log::channel('gorgiasCronLog')->info($type.": No tickets were created in Gorgias during this execution");
            $commonContent = '<p>No tickets were created in Gorgias during this execution</p><br>';

            if ($subscriptionTotalProcessedCount == 1) {
                $commonContent .= $subscriptionTotalProcessedCount.' record processed';
            } else {
                $commonContent .= $subscriptionTotalProcessedCount.' records processed';
            }

            $iterableMailRequest = array(
                "campaignId" => intval($iterableCampaignId),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => '',
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } else if ($subscriptionTicketCreationProcessedCount > 30) {

            $fileUrl = $csvFilePath.'/GorgiasTicketCreatedReport/'.$this->reportFileDirectory.'/'.$reportCsvName;
            if ($subscriptionTicketCreationProcessedCount == 1) {
                $commonContent = '<p>'.$subscriptionTicketCreationProcessedCount.' Subscription have had processed successfully to create a ticket in Gorgias</p>';
            } else {
                $commonContent = '<p>'.$subscriptionTicketCreationProcessedCount.' Subscriptions have had processed successfully to create tickets in Gorgias</p>';
            }
            $recordDetails = 'Please <a href="'.$fileUrl.'">click here</a> to download the file.';

            if ($subscriptionTotalProcessedCount == 1) {
                $recordDetails .= '<p>'.$subscriptionTotalProcessedCount.' record processed</p>';
            } else {
                $recordDetails .= '<p>'.$subscriptionTotalProcessedCount.' records processed</p>';
            }

            $iterableMailRequest = array(
                "campaignId" => intval($iterableCampaignId),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
            
        } else if ($subscriptionTicketCreationProcessedCount > 0 && $subscriptionTicketCreationProcessedCount <= 30) {
            if ($subscriptionTicketCreationProcessedCount == 1) {
                $commonContent = '<p>'.$subscriptionTicketCreationProcessedCount.' Subscription have had processed successfully to create a ticket in Gorgias</p>';
            } else {
                $commonContent = '<p>'.$subscriptionTicketCreationProcessedCount.' Subscriptions have had processed successfully to create tickets in Gorgias</p>';
            }
            
            $recordDetails = '';
            if (!empty($processedSubscriptionList)) {           
                $recordDetails .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>CustomerName</td>
                        <td>Email</td>
                        <td>RechargeCustomerID</td> 
                        <td>SubscriptionID</td>
                        <td>Status</td>                       
                    </tr>            
                ";

                $i = 1;
                foreach ($processedSubscriptionList as $record) {
                            
                    $recordDetails .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record['customerName']."</td> 
                        <td>".$record['email']."</td>  
                        <td>".$record['rechargeCustomerId']."</td>
                        <td>".$record['subscriptionId']."</td>
                        <td>".$record['status']."</td>
                    </tr>
                    ";
                    $i++;            
                }
                $recordDetails .= '</table>';

                if ($subscriptionTotalProcessedCount == 1) {
                    $recordDetails .= '<p>'.$subscriptionTotalProcessedCount.' record processed</p>';
                } else {
                    $recordDetails .= '<p>'.$subscriptionTotalProcessedCount.' records processed</p>';
                }
            }
            
            $iterableMailRequest = array(
                "campaignId" => intval($iterableCampaignId),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        }
        
        if (!empty($iterableMailRequest)) {

            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest), $type);

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                Log::channel('gorgiasCronLog')->info($type.": Gorgias ticket creation report mail send successfully");
            } else {
                Log::channel('gorgiasCronLog')->info($type.": Failed to sent the Gorgias ticket creation report mail");
                Log::channel('gorgiasCronLog')->info($type.": Response => ". json_encode($mailTriggerResponse));                 
            }
        }
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/GorgiasTicketCreatedReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);
        }
        
        $this->nurishTicketCreationReport = "nurish-gorgias-ticket-creation-processed-list-".date("Y_m_d_H_i").".csv";
        $this->natureMadeTicketCreationReport = "nature-made-gorgias-ticket-creation-processed-list-".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'nurishTicketProcessedReport' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->nurishTicketCreationReport,
                "mode"     => "",
                "columns"  => array(
                    "CustomerName",
                    "Email",
                    "RechargeCustomerID",
                    "SubscriptionID",
                    "Status"
                )
            ),
            'natureMadeTicketProcessedReport' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->natureMadeTicketCreationReport,
                "mode"     => "",
                "columns"  => array(
                    "CustomerName",
                    "Email",
                    "RechargeCustomerID",
                    "SubscriptionID",
                    "Status"
                )
            )
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }

}
