<?php

namespace App\Http\Controllers;
header("Access-Control-Allow-Origin: *");

use App\Http\Requests\OrderValidation;
use Illuminate\Http\Request;
use App\Models\Shopify\Product;
use App\Models\Shopify\Customer;
use App\Models\IterableModel;
//use OhMyBrew\BasicShopifyAPI;
use Config;
use stdClass;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Log;

class OrderController extends Controller {

    private $productObj;
    private $customerObj;
    private $shopifyApi;
    private $iterableObj;
    private $shProductDetails = [];
    private $shopify_api_version;
    private $api;

     /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->productObj = new Product();
        $this->customerObj = new Customer();
        $this->iterableObj = new IterableModel();
        //$this->shopifyApi = new BasicShopifyAPI();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);

        $options = new Options();
        $password   = Config::get('shopify.passwordOrder');
        $shopDomain = Config::get('shopify.shopDomainOrder');
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $this->shopifyApi = new BasicShopifyAPI($options);
        $this->shopifyApi->setSession(new Session($shopDomain, $password));

        // $this->shopifyApi->setShop($shopDomain);
        // $this->shopifyApi->setAccessToken($password);
    }

    // To store the order in Shopify
    public function saveOrder(Request $request) {
        
        $message = "Endpoint parameters are missing";
        $status = "Failed";
        $receivedRequest = !empty($request->input('data')) ? $request->input('data') : '';

        if (empty($receivedRequest)) {

            $response = array(
                'status' => $status,
                'errorMessage' => $message,
            );

            Log::channel('orderControllerLog')->info("Endpoint parameters are missing");
            echo json_encode($response);
            die();
        } else {

            $data = json_decode($receivedRequest,true);
            $lineItemStatus = 0;
            $shippingStatus = 0;
            if (empty($data['order']['email']) || empty($data['order']['product_id']) ||
            empty($data['order']['variant_id'])) {
                $response = array(
                    'status' => $status,
                    'errorMessage' => $message,
                );
                Log::channel('orderControllerLog')->info("Endpoint parameters are missing(Request: ".json_encode($receivedRequest).")");
                echo $request->input('callback') . '(' . json_encode($response) . ')';
                die();
            } else {

                if (isset($data['order']['line_items'])){
                    foreach ($data['order']['line_items'] as $lineItems){
                        if ((empty($lineItems['variant_id']) || empty($lineItems['product_id']) ||
                        empty($lineItems['name']) || empty($lineItems['value']))
                        && empty($lineItems['utm'])
                        && empty($lineItems['source'])
                        && empty($lineItems['campaign'])
                        && empty($lineItems['medium'])
                        && empty($lineItems['content'])
                        && empty($lineItems['term'])
                        && empty($lineItems['quiz_id'])
                        ) {

                            $lineItemStatus = 1;
                        }
                    }
                } else {
                    $lineItemStatus = 1;
                }

                if (empty($data['order']['line_items']) || $lineItemStatus == 1) {
                    $response = array(
                        'status' => 'Failed',
                        'errorMessage' => 'Missing required parameters in the line item properties',
                    );
                    Log::channel('orderControllerLog')->info("Missing required parameters in the line item properties(Request: ".
                    json_encode($receivedRequest).")");
                    echo $request->input('callback') . '(' . json_encode($response) . ')';
                    die();
                }

                if (!empty($data['order']['shipping_address'])) {
                    $shippingaddress = $data['order']['shipping_address'];
                    if(empty($shippingaddress['first_name']) || empty($shippingaddress['last_name']) ||
                    empty($shippingaddress['address1']) || empty($shippingaddress['city']) ||
                    empty($shippingaddress['country']) || empty($shippingaddress['state'])){
                        $shippingStatus = 1;
                    }
                } else {
                    $shippingStatus = 1;
                }

                if ($shippingStatus == 1) {
                    $response = array(
                        'status' => 'Failed',
                        'errorMessage' => 'Missing required parameters in the Shipping Address',
                    );
                    Log::channel('orderControllerLog')->info("Missing required parameters in the line item properties(Request: ".
                    json_encode($receivedRequest).")");
                    echo $request->input('callback') . '(' . json_encode($response) . ')';
                    die();
                }

            }

        }

        $requestData = json_decode($receivedRequest,true);
        $setOrderRequest = $this->setOrderRequest($requestData);
        $params['order'] = $setOrderRequest;

        $result = $this->shopifyApi->rest('POST', htmlentities('/admin/api/'.$this->shopify_api_version.'/orders.json'), $params);

        if (!empty($result['errors'])) {

            $response = array(
                'status' => $status,
                'errorMessage' => json_encode($result['errors']),
            );
            Log::channel('orderControllerLog')->info("Failed to create the order in Shopify, Error: ".json_encode($result['errors']));
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        } else {

            if (!empty($requestData['order']['newsletter']) && !empty($requestData['order']['email'])) {

                $emailAddress = trim($requestData['order']['email']);
                $checkOutStatus = trim($requestData['order']['newsletter']);
                $preferenceIds = env('INFLUENCER_PREFERENCE_IDS',true);
                $preferenceIdsExplode = explode(",",$preferenceIds);

                $channelIds = env('INFLUENCER_CHANNEL_IDS',true);
                $channelIdsExplode = explode(",",$channelIds);

                $iterableSubscriptionDetails = $this->iterableObj->getIterableSubscription($emailAddress);


                $iterableSubscriptionDetails = (array)$iterableSubscriptionDetails;
                if (isset($iterableSubscriptionDetails['user'])) {

                    $unsubscribedChannelIds = !empty($iterableSubscriptionDetails['user']
                    ->dataFields->unsubscribedChannelIds) ? $iterableSubscriptionDetails['user']
                    ->dataFields->unsubscribedChannelIds : '';

                    $unsubscribedMessageTypeIds = !empty($iterableSubscriptionDetails['user']
                    ->dataFields->unsubscribedMessageTypeIds) ? $iterableSubscriptionDetails['user']
                    ->dataFields->unsubscribedMessageTypeIds : '';

                    $subscribedMessageTypeIds =  !empty($iterableSubscriptionDetails['user']
                    ->dataFields->subscribedMessageTypeIds) ? $iterableSubscriptionDetails['user']
                    ->dataFields->subscribedMessageTypeIds : '';

                    $newUnsubscribedChannelIds = [];
                    $newUnsubscribedMessageTypeId = [];
                    $newSubscribedMessageTypeIds = [];

                    if ($checkOutStatus == 'true') {

                        if (!empty($unsubscribedChannelIds)) {
                            $newUnsubscribedChannelIds = array_diff($unsubscribedChannelIds, (is_array($channelIdsExplode) ?
                            $channelIdsExplode : array($channelIdsExplode)));
                        } else {
                            $newUnsubscribedChannelIds = [];
                        }

                        if (!empty($unsubscribedMessageTypeIds)) {
                            $newUnsubscribedMessageTypeIds = array_diff($unsubscribedMessageTypeIds, (is_array($preferenceIdsExplode) ?
                            $preferenceIdsExplode : array($preferenceIdsExplode)));
                        } else {
                            $newUnsubscribedMessageTypeIds = [];
                        }

                        if(!empty($subscribedMessageTypeIds)) {
                            $newSubscribedMessageTypeIds = $subscribedMessageTypeIds;
                        }
                        for ($i=0; $i < count($preferenceIdsExplode) ; $i++) {

                            $newSubscribedMessageTypeIds[] = $preferenceIdsExplode[$i];
                        }
                        $newSubscribedMessageTypeIds = array_unique($newSubscribedMessageTypeIds);

                    } elseif ($checkOutStatus == 'false') {

                        $newUnsubscribedChannelIds = !empty($unsubscribedChannelIds) ? $unsubscribedChannelIds : [];


                        if (!empty($unsubscribedMessageTypeIds)) {

                            $newUnsubscribedMessageTypeIds = $unsubscribedMessageTypeIds;
                        }
                        for ($i=0; $i < count($preferenceIdsExplode); $i++) {

                            $newUnsubscribedMessageTypeIds[] = $preferenceIdsExplode[$i];
                        }
                        $newUnsubscribedMessageTypeIds = array_unique($newUnsubscribedMessageTypeIds);


                        if (!empty($subscribedMessageTypeIds)) {
                            $newSubscribedMessageTypeIds = array_diff($subscribedMessageTypeIds, (is_array($preferenceIdsExplode) ?
                            $preferenceIdsExplode : array($preferenceIdsExplode)));
                        } else {
                            $newSubscribedMessageTypeIds = [];
                        }

                    }

                    $updateRequest = array(
                        "email" => $emailAddress,
                        "unsubscribedChannelIds" => array_map('intval', $newUnsubscribedChannelIds),
                        "unsubscribedMessageTypeIds" => array_map('intval', $newUnsubscribedMessageTypeIds),
                        "subscribedMessageTypeIds" => array_map('intval', $newSubscribedMessageTypeIds)
                    );

                    $preferenceUpdatedResponse = $this->iterableObj->updateUserIterableSubscription(
                        json_encode($updateRequest));
                    if (isset($preferenceUpdatedResponse->code) && $preferenceUpdatedResponse->code == 'Success') {

                        Log::channel('orderControllerLog')->info("Successfully updated the user iterable subscription, Email: ".$emailAddress);
                    } else {
                        Log::channel('orderControllerLog')->info("Failed to update the user iterable subscription, Email: ".$emailAddress);
                    }

                } else {

                    Log::channel('orderControllerLog')->info("Failed to get the user iterable subscription details from iterable, Email: ".$emailAddress);
                }
            }
            $order_details = $result['body'];
            $response = array(
                'status' => 'success',
                'successMessage' => array('order_id' => $order_details['order']['id'],
                'message' => 'Order created successfully'),
                'errorMessage' => "",
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }

    // To set the request for order and draft order
    public function setOrderRequest($reqObj) {

        $customerEmail = $reqObj['order']['email'];
        $request = [];
        $lineProperties = [];
        $prod  = $this->setProductDetails();
        // print_r($prod); die;
        if (isset($reqObj['order']['line_items'])) {
            // Handling duplicate lines in properties
            $rearrangeLineItems = array();
            $i = 0;
            $j= 1;
            foreach ($reqObj['order']['line_items'] as $key => $itemValue){
                if (!isset($itemValue['utm']) && !isset($itemValue['source']) && !isset($itemValue['campaign'])
                 && !isset($itemValue['medium'])
                 && !isset($itemValue['content'])
                 && !isset($itemValue['term'])
                 && !isset($itemValue['quiz_id'])
                 ){
                    if (isset($rearrangeLineItems[$itemValue['variant_id']])){
                        $rearrangeLineItems[$itemValue['variant_id']]['quantity'] +=
                        $itemValue['quantity'];
                    } else {
                        $itemValue['name'] = 'Vitamin'.($j<10? '0'.$j++:$j++);
                        $rearrangeLineItems[$itemValue['variant_id']] = $itemValue;
                    }
                } else {
                    if (isset($itemValue['utm'])) {
                        $rearrangeLineItems['utm'] = array('utm' => $itemValue['utm']);
                    } else if(isset($itemValue['source'])) {
                        $rearrangeLineItems['source'] = array('source' => $itemValue['source']);
                    } else if(isset($itemValue['campaign'])) {
                        $rearrangeLineItems['campaign'] = array('campaign' => $itemValue['campaign']);
                    } else if(isset($itemValue['medium'])) {
                        $rearrangeLineItems['medium'] = array('medium' => $itemValue['medium']);
                    } else if(isset($itemValue['content'])) {
                        $rearrangeLineItems['content'] = array('content' => $itemValue['content']);
                    } else if(isset($itemValue['term'])) {
                        $rearrangeLineItems['term'] = array('term' => $itemValue['term']);
                    } else if(isset($itemValue['quiz_id'])) {
                        $rearrangeLineItems['quiz_id'] = array('quiz_id' => $itemValue['quiz_id']);
                    }
                }
                $i++;
            }

            if(isset($reqObj['order']['Pack'])){
                $rearrangeLineItems['packs'] = array('packs' => $reqObj['order']['Pack']);
            }
            // print_r($rearrangeLineItems); die;
            unset($reqObj['order']['line_items']);
            $reqObj['order']['line_items'] = array_values($rearrangeLineItems);

            $totalPrice = 0;
            foreach ($reqObj['order']['line_items'] as $key => $itemValue) {
                $propertyValue = '';
                if (!isset($itemValue['utm']) && !isset($itemValue['packs']) && !isset($itemValue['source'])
                && !isset($itemValue['campaign'])
                && !isset($itemValue['medium'])
                && !isset($itemValue['content'])
                && !isset($itemValue['term'])
                && !isset($itemValue['quiz_id'])
                ) {

                    $productId = $itemValue['product_id'];
                    $variantId = $itemValue['variant_id'];
                    if (isset($prod['products'][$productId]) && isset($prod['products']
                    [$productId]['variants'][$variantId])){
                        $variantData = $prod['products'][$productId]['variants'][$variantId];
                        $lineItemDosage = !empty($itemValue['dosage']) ? $itemValue['dosage'] : 'default title';
                        $propertyValue .= $itemValue['value'].' | '.$lineItemDosage.' | '
                        .$itemValue['quantity'].' | '.$itemValue['variant_id'].' | '.$variantData['sku'];
                        $lineProperties[] = ['name' => $itemValue['name'],'value' => $propertyValue];
                        $lineItemQuantity = !empty($itemValue['quantity']) ? $itemValue['quantity'] : '1';
                        $totalPrice += intval($lineItemQuantity) * $variantData['price'];
                    }
                } else if (isset($itemValue['utm'])) {
                    $lineProperties[] = ['name'=> 'utm','value'=> $itemValue['utm']];
                } else if (isset($itemValue['source'])){
                    $lineProperties[] = ['name'=> 'source','value'=> $itemValue['source']];
                } else if (isset($itemValue['campaign'])){
                    $lineProperties[] = ['name'=> 'campaign','value'=> $itemValue['campaign']];
                } else if (isset($itemValue['medium'])){
                    $lineProperties[] = ['name'=> 'medium','value'=> $itemValue['medium']];
                } else if (isset($itemValue['content'])){
                    $lineProperties[] = ['name'=> 'content','value'=> $itemValue['content']];
                } else if (isset($itemValue['term'])){
                    $lineProperties[] = ['name'=> 'term','value'=> $itemValue['term']];
                } else if (isset($itemValue['quiz_id'])){
                    $lineProperties[] = ['name'=> 'quiz_id','value'=> $itemValue['quiz_id']];
                } else if (isset($itemValue['packs'])){
                    $lineProperties[] = ['name'=> 'Packs','value'=> $itemValue['packs']];
                }
            }
        }

        $customerDetails = $this->customerObj->getCustomerDetailBySearch($customerEmail);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('orderControllerLog')->info("Failed to get the customer details from Shopify(emailAddress:".$customerEmail."), 
            Error: (".$error.")");
            $this->sendResponse(600, "error", $returnData);
        }

        // $result_data = $this->shopifyApi->rest('GET',
        // '/admin/api/'.$this->shopify_api_version.'/customers/search.json',["query" =>"email:".$customerEmail, "fields" =>
        // "id,email,first_name,tags"]);
        $customerTag = '';
        $tag = !empty($customerDetails['customers'][0]['tags']) ? trim($customerDetails['customers'][0]['tags']) : "";

        if (!empty($tag)) {

            $customerTag = $tag.' ,VIP';
        } else {
            $customerTag = 'VIP';
        }

        $shippingAddress = $reqObj['order']['shipping_address'];
        $request['customer']['first_name'] = $shippingAddress['first_name'];
        $request['customer']['last_name'] = $shippingAddress['last_name'];
        $request['customer']['email'] = $reqObj['order']['email'];
        $request['customer']['tags'] = $customerTag;
        $request['email'] = $reqObj['order']['email'];
        $request['tags'] = 'VIP';
        $request['line_items'] = array(
            array(
                'title' => "Your Monthly Subscription (Ships every 7 Days)",
                'product_id' => $reqObj['order']['product_id'],
                'variant_id' => $reqObj['order']['variant_id'],
                'quantity'   => !empty($reqObj['order']['quantity']) ?
                $reqObj['order']['quantity'] : '1',
                'price'      => $totalPrice,
                'properties' => $lineProperties
            )
        );

        $request['shipping_address'] = array(
            "address1"  => $shippingAddress['address1'],
            "address2"  => $shippingAddress['address2'],
            "city"      => $shippingAddress['city'],
            "country"   => $shippingAddress['country'],
            "first_name"=> $shippingAddress['first_name'],
            "last_name"  => $shippingAddress['last_name'],
            "province"  => $shippingAddress['state'],
            "zip"       => $shippingAddress['zip'],
            "phone"     => $shippingAddress['phone'],
        );
        $request['send_receipt'] = true;
        return $request;
    }

    /**
     * To set the shopify product details
     */
     public function setProductDetails() {

        $shopifyVariants = [];
        $shopifyProducts['products'] = [];
        $limit = 50;
        $nextLink = '';
        $previousLink = '';

        do {

            try{

                if (empty($previousLink) && empty($nextLink)) {

                    $productDetails = $this->shopifyApi->rest('GET',
                    '/admin/api/'.$this->shopify_api_version.'/products.json',["published_status" => "published", "fields" =>
                    "id,title,variants", "limit" => $limit]);
                    $previousLink = !empty($productDetails['link']['previous']) ?
                    $productDetails['link']['previous'] : '';
                    $nextLink = !empty($productDetails['link']['next']) ?
                    $productDetails['link']['next'] : '';

                } elseif(!empty($nextLink)) {

                    $params = ["fields" => "id,title,variants", "limit" => $limit, 'page_info' => $nextLink];
                    $productDetails = $this->shopifyApi->rest('GET', '/admin/api/'.$this->shopify_api_version.'/products.json',$params);
                    $previousLink = !empty($productDetails['link']['previous']) ?
                    $productDetails['link']['previous'] : '';
                    $nextLink = !empty($productDetails['link']['next']) ?
                    $productDetails['link']['next'] : '';
                }
            } catch(Exception $e) {
                echo PHP_EOL."Error when fetching the products from Shopify and the error is -  "
				 .$e->getMessage();
			 	die;
            }

            $productDetails = $productDetails['body'];

            if (!empty($productDetails)) {

                $subscription_product = array('1809168695367','4359186350162');

                foreach ($productDetails['products'] as $products) {
                    if (in_array($products['id'], $subscription_product)) {
                        continue;
                    }

                    foreach ($products['variants'] as $variants) {

                        $shopifyVariants[$variants['id']] = [
                            'id' => $variants['id'] ,
                            'product_id' => $variants['product_id'],
                            'title' => $variants['title'],
                            'sku' => $variants['sku'],
                            'price' => $variants['price']
                        ];
                    }
                    $shopifyProducts['products'][$products['id']] = ['id'=> $products['id'],
                    'title'=> $products['title'],'variants'=> $shopifyVariants];
                    $shopifyVariants = [];

                }

            }
            $count = count($productDetails['products']);
            if(empty($nextLink)) {
                break;
            }
        } while($limit == $count);

        return $shopifyProducts;
    }
}

?>
