<?php

namespace App\Http\Controllers;

use DB;
use Log;
use App\Users;
use Illuminate\Http\Request;
use App\Http\ResponseHelper;
use App\Jobs\OrderTaggingJob;
use Config;
use OhMyBrew\BasicShopifyAPI;

class OrderTaggingWebhookController extends Controller
{

    private $helper;
    private $shopifyProductDao;
    private $sharedSecret;
    private $randomString;
    private $shopify_api_version;
    private $shopifyWebhookSecret;

    public function __construct() {

        $this->helper = new ResponseHelper();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
        $this->shopifyWebhookSecret = env('SHOPIFY_WEBHOOK_SECRET', true);
    }


    /**
     * Update tag to the Shopify order
     */
    public function processWebhook() {

        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_shopify_webhook($requestData, $hmac_header);
        $webhookStatus = var_export($verified, true);

        if ($webhookStatus) {

            $receivedData = json_decode($requestData);

            if (empty($receivedData->id)) {

                Log::channel('orderTag')->info('Shopify order create 
                webhook response is not correct, WebhookResponse:'.$requestData);
            } else {
    
                $queuedJobResponse = dispatch(new OrderTaggingJob($requestData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            Log::channel('orderTag')->info('Shopify order create webhook validation 
            failed, WebhookResponse:'.$requestData);
        }
    }

    /**
     * Shopify webhook verification
     */
    public function verify_shopify_webhook($data, $hmac_header) {

        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shopifyWebhookSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    } 

}
