<?php

namespace App\Http\Controllers;

use App\Http\ResponseHelper;
use App\Jobs\CreateShopifyCustomerJob;
use App\Jobs\recommendationToIterableJob;
use App\Jobs\TagUserJob;
use Carbon\Carbon;
use Config;
use DB;
use Illuminate\Http\Request;
use JsonSerializable;
use Log;
use Illuminate\Support\Facades\Crypt;
use App\Models\IterableApi;
use App\Models\DatabaseModel;
use App\Models\Shopify\Customer;
use App\Http\Controllers\ShopifyMultiPass;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;

class QuizController extends Controller
{

    private $helper;
    private $shopify_api_version;
    public $iterableObj;
    public $customerObj;
    public $dbObj;
    private $api;

    public function __construct() {

        $this->iterableUrl = env('ITERABLE_URL', true);
        $this->iterableApiKey = env('ITERABLE_API_KEY', true);
        $this->helper = new ResponseHelper();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
        $this->iterableObj = new IterableApi();
        $this->dbObj = new DatabaseModel();
        $this->customerObj = new Customer();

        $password = Config::get('shopify.passwordQuiz');
        $shopDomain = Config::get('shopify.shopDomainQuiz');
        $options = new Options();
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session($shopDomain, $password));
    }

    /**
     * Store quiz section
     */
    public function storeQuiz(Request $request) {

        $page = $request->input('page');
        $sessionID = $request->input('sessionID');
        $question = $request->input('question');
        $questionID = $request->input('questionID');
        $questionNo = $request->input('questionNo');
        $responses = $request->input('responses');
        $responseIds = $request->input('responseIDs');
        $responseKeys = $request->input('responseKeys');

        $questionTextArray = array('31', '49');

        if (empty($sessionID)) {

            // error
            $returnData = [
                'error' => 'Invalid Session',
            ];

            Log::channel('quizControllerLog')->info("Invalid Session for the quiz");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if ($page == 'close') {
            Log::channel('quizControllerLog')->info('Inside close page - ' . $sessionID);
            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            $other = DB::table('question_responses')->select('id')->where([
                ['qn_id', '=', '12'],
                ['response', '=', 'Other'],
            ])->value('id');

            $userResponse = DB::select('
                select
                    qm.id,
                    qm.question,
                    qm.question_no,
                    qr.response,
                    qr.response_key,
                    ur.response_text
                from
                    user_responses as ur
                join
                    question_master as qm
                on
                    qm.id = ur.qn_id
                left join
                    question_responses as qr
                on
                    ur.response_id = qr.id
                where
                    session_id = ?', [$sessionID]);

            $dataResult = [];
            $details = [];

            if (!empty($userResponse)) {

                foreach ($userResponse as $response) {
                    $questionId = $response->id;
                    $questionNo = $response->question_no;
                    $questionText = $response->question;
                    $answerText = $response->response;
                    $answerKey = $response->response_key;
                    $responseText = $response->response_text;

                    if ($questionNo == '11' && $responseText != null) {
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                        $details[$questionNo]['AnswerText'][] = $other;
                        continue;
                    }

                    if (!isset($details[$questionNo])) {
                        $details[$questionNo] = [];
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                    }

                    if (in_array($questionNo, $questionTextArray)) {
                        $details[$questionNo]['AnswerText'][] = $answerText;
                    } else {
                        $details[$questionNo]['AnswerText'][] = empty($answerKey) ? $responseText : strval($answerKey);
                    }
                }
                $questionDetails = array_values($details);
                $dataResult['SurveyAnswers']['SessionId'] = $sessionID;
                $dataResult['SurveyAnswers']['Questions'] = $questionDetails;
            }

            $returnData = $dataResult;

            $jsonData = json_encode($returnData);
            Log::channel('quizControllerLog')->info($jsonData);

            $returnASCII = $this->encode_items($returnData);

            $jsonDataASCII = json_encode($returnASCII);
            Log::channel('quizControllerLog')->info($jsonDataASCII);

            // CALL to Recommendation API
            Log::channel('quizControllerLog')->info('Before recommendation called - ' . $sessionID);

            Log::channel('quizControllerLog')->info($returnASCII);
            $recommendation = $this->callRecommendation($returnASCII);
            Log::channel('quizControllerLog')->info($recommendation);
            Log::channel('quizControllerLog')->info('Recommendation result for - ' . $sessionID);

            $recommendationObj = json_decode($recommendation);
            if (isset($recommendationObj->Result)) {
                $recommendationObj = $recommendationObj->Result;
            }

            if (isset($recommendationObj->Recommendation) && !empty($recommendationObj->Recommendation)) {
                Log::channel('quizControllerLog')->info('The recommendation is not empty - ' . $sessionID);

                $updateData = $updateQuizUser = DB::table('quiz_user')->where(['session_id' => $sessionID])
                    ->update(['recommended_at' => date('Y-m-d H:i:s')]);

                $lastInsertedId = DB::table('recommendations')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'email' => isset($recommendationObj->Email) ? $recommendationObj->Email : ' ',
                        'first_name' => isset($recommendationObj->Firstname) ? $recommendationObj->Firstname : ' ',
                        'sex_profile' => isset($recommendationObj->SexProfile) ? $recommendationObj->SexProfile : ' ',
                        'sex' => isset($recommendationObj->Sex) ? $recommendationObj->Sex : ' ',
                        'age' => isset($recommendationObj->Age) ? $recommendationObj->Age : ' ',
                        'zipcode' => isset($recommendationObj->ZipCode) ? $recommendationObj->ZipCode : ' ',
                        'latitude' => isset($recommendationObj->Latitude) ? $recommendationObj->Latitude : ' ',
                        'longitude' => isset($recommendationObj->Longitude) ? $recommendationObj->Longitude : ' ',
                        'city' => isset($recommendationObj->City) ? $recommendationObj->City : ' ',
                        'state' => isset($recommendationObj->State) ? $recommendationObj->State : ' ',
                        'allergens' => isset($recommendationObj->Allergens) ? json_encode($recommendationObj->Allergens) : ' ',
                        'deficiencies' => isset($recommendationObj->Deficiencies) ? json_encode($recommendationObj->Deficiencies) : ' ',
                        'recommendation' => isset($recommendationObj->Recommendation) ? json_encode($recommendationObj->Recommendation) : ' ',
                        'buzz_words' => isset($recommendationObj->BuzzWordsTips) ? json_encode($recommendationObj->BuzzWordsTips) : ' ',
                        'north37' => isset($recommendationObj->North37) ? $recommendationObj->North37 : ' ',
                        'prostate_interest' => isset($recommendationObj->ProstateInterest) ? $recommendationObj->ProstateInterest : ' ',
                        'prenatal_interest' => isset($recommendationObj->PrenatalInterest) ? $recommendationObj->PrenatalInterest : ' ',
                        'postnatal_interest' => isset($recommendationObj->PostnatalInterest) ? $recommendationObj->PostnatalInterest : ' ',
                        'healthy_interest' => isset($recommendationObj->HealthyMentrualCycleInterest) ? $recommendationObj->HealthyMentrualCycleInterest : ' ',
                        'perimenopause_interest' => isset($recommendationObj->PerimenopauseInterest) ? $recommendationObj->PerimenopauseInterest : ' ',
                        'menopause_interest' => isset($recommendationObj->MenopauseInterest) ? $recommendationObj->MenopauseInterest : ' ',
                        'planning_pregnancy' => isset($recommendationObj->PlanningPregnancy) ? $recommendationObj->PlanningPregnancy : ' ',
                        'pregnant' => isset($recommendationObj->Pregnant) ? $recommendationObj->Pregnant : ' ',
                        'breastfeeding' => isset($recommendationObj->Breastfeeding) ? $recommendationObj->Breastfeeding : ' ',
                        'taking_medication' => isset($recommendationObj->TakingMedication) ? $recommendationObj->TakingMedication : ' ',
                        'blood_clotting_issue' => isset($recommendationObj->BloodClottingIssue) ? $recommendationObj->BloodClottingIssue : ' ',
                        'response_time' => isset($recommendationObj->ResponseTime) ? $recommendationObj->ResponseTime : ' ',
                    ]
                );

                if (empty($lastInsertedId)) {
                    $returnData = [
                        'error' => 'Insertion Failed',
                    ];

                    Log::channel('quizControllerLog')->info("Recommendation insertion failed for the session - ".$sessionID);
                    return $this->helper->returnData(600, "error", $returnData);
                }

                // For the tagging purpose
                $questionResponse = DB::select('
                    select
                        qm.id,
                        qm.question,
                        qr.response
                    from
                        user_responses as ur
                    join
                        question_master as qm
                    on
                        qm.id = ur.qn_id
                    left join
                        question_responses as qr
                    on
                        ur.response_id = qr.id
                    where
                        session_id = ? and ur.qn_id in (14,32)', [$sessionID]);

                $tagQuestion = array();
                if (!empty($questionResponse)) {

                    foreach ($questionResponse as $qResponse) {

                        if (!isset($tagQuestion[$qResponse->id])) {

                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        } else {

                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        }
                    }
                }
                // Tag User
                $tagUser = dispatch(new TagUserJob($recommendation, $tagQuestion, $sessionID));

                // Iterable
                $recToIterableResponse = dispatch(new recommendationToIterableJob($sessionID));

                $recProducts = $recommendationObj->Recommendation;

                if (!empty($recProducts)) {

                    foreach ($recProducts as $recProd) {

                        $reason = array();
                        $reasonProduct = $recProd->ReasonsForProduct;

                        if (!empty($reasonProduct)) {
                            foreach ($reasonProduct as $value) {
                                $stripValue = preg_replace('/[^a-zA-Z0-9-;.,+"()%\/(&deg;) ]/', '', $value);
                                array_push($reason, $stripValue);
                            }
                            $recProd->ReasonsForProduct = $reason;
                        }
                    }
                }

                $recommendationObj = json_decode($recommendation);
                if (isset($recommendationObj->Result)) {

                    $recommendationObj = $recommendationObj->Result;
                }

                $recommendationObj->Recommendation = $recProducts;

                if (!empty($recommendationObj->Email)) {

                    /* End trigger email */
                    $eventRequest = array(
                        "email" => $recommendationObj->Email,
                        "eventName" => "Quiz Completion",
                        // "campaignId" => 882346,
                        "dataFields" => array("userName" => $recommendationObj->Firstname),
                    );
                    // Trigger Event
                    $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

                    if (isset($eventResponse->code) && trim(strtolower($eventResponse->code)) == 'success') {

                        Log::channel('quizControllerLog')->info('Event-triggered successfully Please check mail : ' . $recommendationObj->Email);
                    } else {

                        Log::channel('quizControllerLog')->info('Event not triggered');
                        Log::channel('quizControllerLog')->info('Error: '.json_encode($eventResponse));
                    }

                }
                return $this->helper->returnData(200, "success", json_decode($recommendation));

            } else {
                Log::channel('quizControllerLog')->info('The recommendation - ' . $sessionID);
                $returnData = [
                    'error' => 'Empty Recommendation',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

        }

        if (empty($questionID)) {
            
            // error
            $returnData = [
                'error' => 'Quiz not found',
            ];

            Log::channel('quizControllerLog')->info('Quiz not found (empty question_id) - ' . $sessionID);
            return $this->helper->returnData(600, "error", $returnData);
        }

        // To delete the already existing question and update the new record
        $deleteQuizDetails = DB::table('user_responses')
            ->where([
                ['session_id', '=', $sessionID],
                ['qn_no', '>=', $questionNo],
            ])->delete();

        if (!empty($responseIds)) {
            $lastInsertedId = null;
            foreach ($responseIds as $responseId) {
                $lastInsertedId = DB::table('user_responses')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'qn_id' => $questionID,
                        'qn_no' => $questionNo,
                        'response_id' => $responseId,
                        // 'response_text' => null,
                        'deleted' => 0,
                        'status' => 0,
                    ]
                );
                
                Log::channel('quizControllerLog')->info('Question - '.$questionNo.',  Response_id - '.$responseId.', sessionID - ' . $sessionID);

                if (empty($lastInsertedId)) {
                    $returnData = [
                        'error' => 'Insertion Failed',
                    ];
                    
                    Log::channel('quizControllerLog')->info('Quiz response insertion failed - ' . $sessionID);
                    Log::channel('quizControllerLog')->info('Question_no: ' . $questionNo.",  Response_id : ".$responseId);

    
                    return $this->helper->returnData(600, "error", $returnData);
                }
            }

            $returnData = [
                'stored' => true,
                'message' => 'success'
            ];

        } else {

            $lastInsertedId = DB::table('user_responses')->insertGetId(
                [
                    'session_id' => $sessionID,
                    'qn_id' => $questionID,
                    'qn_no' => $questionNo,
                    'response_id' => null,
                    'response_text' => $responses[0],
                    'deleted' => 0,
                    'status' => 0,
                ]
            );
            
            Log::channel('quizControllerLog')->info('Question - '.$questionNo.',  response_text - '.$responses[0].', sessionID - ' . $sessionID);

            if (empty($lastInsertedId)) {
                $returnData = [
                    'error' => 'Insertion Failed',
                ];

                Log::channel('quizControllerLog')->info('Quiz response insertion failed - ' . $sessionID);
                Log::channel('quizControllerLog')->info('Question_no: ' . $questionNo.",  Response_id : ".$responses[0]);

                return $this->helper->returnData(600, "error", $returnData);
            }

            $returnData = [
                'stored' => true,
                'message' => 'success',
            ];

        }

        if ($page == 'email') {

            Log::channel('quizControllerLog')->info('Inside email page - ' . $sessionID);
            Log::channel('quizControllerLog')->info('Email response_text - '.print_r($responses,1));
            
            $email = isset($responses[0]) ? $responses[0] : '';
            $registrationAuth = '';
            $authMatchFlag = '';

            if (empty($email)) {
                $returnData = [
                    'error' => 'Email is empty',
                ];

                Log::channel('quizControllerLog')->info('Empty email for the session - ' . $sessionID);

                return $this->helper->returnData(600, "error", $returnData);
            }

            $user = DB::table('users')->where('user_email', '=', $email)->get();

            if (!isset($user[0]->user_email)) {

                // create
                Log::channel('quizControllerLog')->info("create a user in the user's table and Shopify ");
                $userName = '';
                $registrationAuth = $this->generateRandomPasswordToken(32);
                $authMatchFlag = 'true';

                $userId = DB::table('users')->insertGetId([
                    'user_email' => $email,
                    'registration_auth' => $registrationAuth,
                    'status' => 1,
                ]);

                if (empty($userId)) {
                    Log::channel('quizControllerLog')->info("User not saved in DB - ".$email. " with session id = ".$sessionID );
                }

                $quizId = DB::table('quiz_user')->select('id')->where([
                    'session_id' => $sessionID,
                ])->value('id');

                if ($quizId) {

                    $updateQuizUser = DB::table('quiz_user')->where(['id' => $quizId])
                        ->update(['user_id' => $userId]);

                } else {
                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $userId,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);

                    if (empty($quizId)) {
                        Log::channel('quizControllerLog')->info("UserID not saved in quiz_user table - ".$userId. " with session id = ".$sessionID);
                    }
                }

                $userNameResponse = DB::table('user_responses')->where([
                    ['session_id', '=', $sessionID],
                    ['qn_id', '=', '2'],
                ])->get();

                if ($userNameResponse[0]) {
                    $userName = $userNameResponse[0]->response_text;
                }

                $customerCreationDetails = [];
                $customerCreationDetails['email'] = $email;
                $customerCreationDetails['userName'] = $userName;
                $customerCreationDetails['sessionId'] = $sessionID;
                // for setting the partner value

                if (!empty($request->input('partnerValue'))) {
                    $customerCreationDetails['partnerValue'] = $request->input('partnerValue');
                }

                // Create customer in Shopify
                $result = dispatch(new CreateShopifyCustomerJob($customerCreationDetails));

            } else {

                $userId = $user[0]->id;
                $userEmail = $user[0]->user_email;

                $customerDetails = $this->customerObj->getCustomerDetailBySearch($userEmail);

                if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                    $error = "Unknown error";
                    if (!empty($customerDetails['error'])) {

                        $error = json_encode($customerDetails['error']);
                    } elseif (!empty($customerDetails['errors'])) {
                        
                        $error = json_encode($customerDetails['errors']);
                    }
                    if (empty($error)) {

                        $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $userEmail . ")";
                    }
                    $returnData = [
                        "error" => $error
                    ];

                    Log::channel('quizControllerLog')->info("Failed to get the customer details from Shopify(emailAddress:".$userEmail."), 
                    Error: (".$error.")");
                    return $this->helper->returnData(600, "error", $returnData);
                }


                if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {

                    $returnData = [
                        'error' => 'No such customer found in Shopify'
                    ];

                    Log::channel('quizControllerLog')->info("No such customer found in Shopify (search) - ".$sessionID." with userEmail - ".$userEmail);
                    
                    return $this->helper->returnData(600, "error", $returnData);
                }

                $customer = [];
                foreach ($customerDetails['customers'] as $cust) {
                    if (trim(strtolower($cust['email'])) == trim(strtolower($email))) {
                        $customer = $cust;
                        break;
                    }
                }

                $passwordSet = '';
                if( !empty($customer)) {
                    $customerId = $customer['id'];
                    $customerTag = $customer['tags'];

                    if (strpos($customerTag, 'passwordSet') !== false) {

                        $passwordSet = "true";
                    } else {

                        $passwordSet = "false";
                    }
                }
                
                $userStatus = $this->getActivationStatus($email);
                $quizId = DB::table('quiz_user')->select('id')->where([
                    'session_id' => $sessionID,
                ])->value('id');

                if ($quizId) {

                    $updateQuizUser = DB::table('quiz_user')->where(['id' => $quizId])
                        ->update(['user_id' => $userId]);

                } else {
                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $userId,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);

                    if(empty($quizId)) {
                        Log::channel('quizControllerLog')->info("UserID not saved in quiz_user table - ".$userId. " with session id = ".$sessionID );
                    }
                }

                //update session id to iterable
                $this->updateRegistrationAuthInIterable($email, $sessionID);

                //password reset email send
                $recoverMailResponse = '';
                $passwordRecoverEmailStatus = '';
                $registrationAuthCookie     = $request->input('registration_auth');
                if(trim($registrationAuthCookie) == ''){
                    //$recoverMailResponse = $this->sendPasswordRecoveryEmail($email);
                }else{
                    $userTokenDetails = DB::table('users')
                    ->where('user_email', '=', $email)
                    ->where('registration_auth','=',$registrationAuthCookie)
                    ->get();
                    if(isset($userTokenDetails[0]->user_email)){
                        $authMatchFlag = 'true';
                    }else{
                        $authMatchFlag = 'false';
                        //$recoverMailResponse = $this->sendPasswordRecoveryEmail($email);
                    }
                }

            }

            $returnData = [
                'stored' => true,
                'message' => 'success',
                'registrationAuth' => isset($registrationAuth) ? $registrationAuth : '',
                'session_id' => $sessionID,
                'emailSendStatus' => isset($passwordRecoverEmailStatus)?$passwordRecoverEmailStatus:'',
                'authMatch' => isset($authMatchFlag)?$authMatchFlag:'',
                'userStatus' => isset($userStatus) ? $userStatus : 'enabled',
                'passwordSet' => isset($passwordSet) ? $passwordSet : "false"
            ];

        }
        return $this->helper->returnData(200, "success", $returnData);

    }


    /**
     * Update registration auth in iterable
     */
    public function updateRegistrationAuthInIterable($email, $sessionID) {

        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "sessionId" => $sessionID
            ),
        );
        $updateIterableData = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
        Log::channel('quizControllerLog')->info(json_encode($updateIterableData));
    }


    /**
     * Generate random password token
     */
    public function generateRandomPasswordToken($length) {

        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    /**
     * Delete quiz
     */
    public function deleteQuiz(Request $request) {

        $quizId = $request->input('quizId');
        $email = $request->input('email');
        $userID = '';

        if (empty($quizId)) {
            $returnData = [
                'error' => 'Invalid Quiz Id',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $userData = DB::table('users')->where('user_email', '=', $email)->get();
        if (isset($userData[0]->id)) {
            $userID = $userData[0]->id;
        } else {
            $returnData = [
                'error' => 'Invalid Email Id',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }
        if (!empty($userID)) {
            $quizIds = array();
            $quizzes = DB::table('quiz_user')->where('user_id', '=', $userID)->get();

            if (!empty($quizzes)) {
                foreach ($quizzes as $quiz) {
                    array_push($quizIds, $quiz->id);
                }
            }

            if (!in_array($quizId, $quizIds)) {
                $returnData = [
                    'error' => 'The quiz does not exist in the server',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            $updateQuiz = DB::table('quiz_user')
                ->where('id', $quizId)
                ->update(['deleted' => 1]);

            if ($updateQuiz == 1) // success
            {
                $returnData = [
                    'deleted' => true,
                    'message' => 'successfully deleted',
                ];

                return $this->helper->returnData(200, "success", $returnData);
            } else {
                $returnData = [
                    'error' => 'The quiz does not exist',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }
        }
    }

    /**
     * Get in PST time zone
     */
    protected function getPST($DateTime, $userTimeZoneOffset) {

        /*$userTimezone = new \DateTimeZone('America/Los_Angeles');
        $gmtTimezone = new \DateTimeZone('UTC');
        $utc = new \DateTime($DateTime);
        $offset = $userTimezone->getOffset($utc);
        $pst = date('Y-m-d H:i:s', $utc->format('U') + $offset);
        return $pst;*/

        $utc = new \DateTime($DateTime);
        $pst = date('Y-m-d H:i:s', $utc->format('U') + $userTimeZoneOffset);
        return $pst;

    }

    /**
     * Get quizzes
     */
    public function getQuizzes(Request $request) {

        $userEmail = $request->input('email');

        $userTimeZoneOffset = $request->input('offset');

        $records = 6; //$request->input('records');
        $page = empty($request->input('page')) ? 1 : $request->input('page');

        $userData = DB::table('users')->where('user_email', '=', $userEmail)->get();

        if (isset($userData[0]->id)) {
            $userID = $userData[0]->id;
        } else {
            $returnData = [
                'error' => 'Invalid Email Id',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $userQuizCount = DB::table('quiz_user')
            ->select(
                'quiz_user.id'
            )
            ->leftJoin('recommendations', 'quiz_user.session_id', '=', 'recommendations.session_id')
            ->where([
                ['quiz_user.user_id', '=', $userID],
                ['quiz_user.deleted', '=', '0'],
                ['quiz_user.status', '=', '1'],
            ])
            ->whereNotNull('recommendations.recommendation')
            ->get()->count();

        $userQuizDetails = DB::table('quiz_user')
            ->select(
                'quiz_user.id',
                'quiz_user.session_id',
                'quiz_user.created_at',
                'quiz_user.updated_at',
                'recommendations.recommendation'
            )
            ->leftJoin('recommendations', 'quiz_user.session_id', '=', 'recommendations.session_id')
            ->where([
                ['quiz_user.user_id', '=', $userID],
                ['quiz_user.deleted', '=', '0'],
                ['quiz_user.status', '=', '1'],
            ])
            ->whereNotNull('recommendations.recommendation')
            ->orderBy('id', 'desc')
            ->limit($records)
            ->offset(($page - 1) * $records)
            ->get();

        $sessionIds = [];

        foreach ($userQuizDetails as $uDetails) {
            array_push($sessionIds, $uDetails->session_id);
        }

        $interests = [];
        if (!empty($sessionIds)) {
            $quizResponses = DB::table('user_responses')
                ->select(
                    'user_responses.session_id',
                    'user_responses.response_id',
                    'question_responses.response'
                )
                ->leftJoin('question_responses', 'user_responses.response_id', '=', 'question_responses.id')
                ->where('user_responses.qn_id', '=', '14')
                ->whereIn('session_id', $sessionIds)
                ->get();

            foreach ($quizResponses as $data) {
                $interests[$data->session_id][] = $data->response;
            }

        }

        $quizDetailsArray = [];

        $sessionIds = [];
        foreach ($userQuizDetails as $userQuizDetail) {
            $quizId = $userQuizDetail->id;
            $sessionID = $userQuizDetail->session_id;
            $recommendation = $userQuizDetail->recommendation;
            $q30Details = $this->getQ30Details($sessionID); // The bit to identify whether q30 must be shown or not
            $sexDetails = $this->getSexDetails($sessionID); // The bit to identify sex details
            $ageDetails = $this->getAgeDetails($sessionID); // The bit to identify age details
            // $createdAt  = $this->getPST($userQuizDetail->created_at,$userTimeZoneOffset);
            // $updatedAt  = $this->getPST($userQuizDetail->updated_at,$userTimeZoneOffset);

            $createdAt = new Carbon($userQuizDetail->created_at, 'America/Los_Angeles');
            $createdAt = $createdAt->tz('utc');

            $updatedAt = new Carbon($userQuizDetail->updated_at, 'America/Los_Angeles');
            $updatedAt = $updatedAt->tz('utc');

            $temp = [];
            $temp['quizID'] = $quizId;
            $temp['sessionID'] = $sessionID;
            $temp['startDate'] = $createdAt;
            $temp['endDate'] = $updatedAt;
            $temp['q30'] = $q30Details; // The bit to identify whether q30 must be shown or not
            $temp['sex'] = $sexDetails; // The bit to identify whether gender
            $temp['age'] = $ageDetails; // The bit to identify the age

            if (isset($interests[$sessionID])) {
                $temp['interests'] = $interests[$sessionID];
            }

            $quizDetailsArray[] = $temp;

        }

        if (($records * $page) < $userQuizCount) {
            $hasNext = true;
        } else {
            $hasNext = false;
        }

        $returnData = [
            'email' => $userEmail,
            'totalRecords' => $userQuizCount,
            'pageSize' => $records,
            'pageNumber' => $page,
            'hasNext' => $hasNext,
            'quizzes' => $quizDetailsArray,
        ];
        return $this->helper->returnData(200, "success", $returnData);

    }

    /**
     * getQ30Details
     */
    public function getQ30Details($sessionID) {

        $quizDetails = $this->getQuizData($sessionID);

        if (!empty($quizDetails)) {

            $questionDetails = $quizDetails['SurveyAnswers']['Questions'];

            if (isset($questionDetails['28']) && isset($questionDetails['29'])) {
                $q28 = $questionDetails['28']['AnswerID'][0];
                $q29 = $questionDetails['29']['AnswerID'][0];

                if ($q28 == '388' && $q29 == '392') {
                    return false;
                } else {
                    return true;
                }
            }
            return false;
        } else {
            return false;
        }
    }

    /**
     * Get sex details
     */
    public function getSexDetails($sessionID) {

        $quizDetails = $this->getQuizData($sessionID);
        if (!empty($quizDetails)) {

            $questionDetails = $quizDetails['SurveyAnswers']['Questions'];

            if (isset($questionDetails['2'])) { // Sex Details
                $sex = $questionDetails['2']['AnswerText'][0];
                return $sex;
            } else {
                return null;
            }
        }
        return null;
    }

    /**
     * Get age details
     */
    public function getAgeDetails($sessionID) {

        $quizDetails = $this->getQuizData($sessionID);
        if (!empty($quizDetails)) {

            $questionDetails = $quizDetails['SurveyAnswers']['Questions'];

            if (isset($questionDetails['3'])) { // Age Details
                $age = $questionDetails['3']['AnswerText'][0];
                return $age;
            } else {
                return null;
            }
        }
        return null;
    }

    /**
     * Get recommendation
     */
    public function getRecommendation(Request $request) {

        $sessionID = $request->input('sessionID');

        if (!empty($sessionID)) {
            $results = DB::select('
                select
                    recommendation
                from
                    recommendations
                where
                    session_id = ?', [$sessionID]);

            if (isset($results[0]->recommendation)) {
                $returnData = [
                    'recommendation' => $results[0]->recommendation,
                    'sessionID' => $sessionID,

                ];
                return $this->helper->returnData(200, "success", $returnData);
            } else {
                $returnData = [
                    'error' => "No recommendation for the given SessionID",

                ];
                return $this->helper->returnData(200, "error", $returnData);
            }
        } else {

            $returnData = [
                'error' => 'Invalid Session Id',
            ];
            return $this->helper->returnData(200, "error", $returnData);
        }

    }

    /**
     * Get quiz recommendation
     */
    public function getQuizRecommendation(Request $request) {

        $sessionID = $request->input('session_id');

        if (!empty($sessionID)) {
            $results = DB::select('
                select
                    recommendation
                from
                    recommendations
                where
                    session_id = ?', [$sessionID]);

            if (isset($results[0]->recommendation)) {
                $returnData = [
                    'recommendation' => $results[0]->recommendation,
                    'sessionID' => $sessionID,

                ];
                return $this->helper->returnData(200, "success", $returnData);
            } else {
                $returnData = [
                    'error' => "No recommendation for the given SessionID",

                ];
                return $this->helper->returnData(200, "error", $returnData);
            }
        } else {

            $returnData = [
                'error' => 'Invalid Session Id',
            ];
            return $this->helper->returnData(200, "error", $returnData);
        }
    }

    // public function getEncryptedDetails(){
    //     $encrypted = Crypt::encryptString('Hello world.');
    //     $decrypted = Crypt::decryptString($encrypted);
    //     echo $encrypted;
    //     echo '--------'.$decrypted;
    // }

    /**
     * Get quiz details
     */
    public function getQuizDetails(Request $request) {

        $sessionID = $request->input('sessionID');

        $results = DB::select('
        select
            qm.id,
            qm.question,
            IFNULL(qr.response,ur.response_text) as response,
            qm.page
        from
            user_responses as ur
        join
            question_master as qm
        on
            qm.id = ur.qn_id
        left join
            question_responses as qr
        on
            ur.response_id = qr.id
        where
            session_id = ?', [$sessionID]);

        $dataResult = [];
        $details = [];

        foreach ($results as $result) {
            $questionId = $result->id;
            $questionText = $result->question;
            $answerText = $result->response;
            $questionHandle = $result->page;

            $temp = [];

            if (!isset($details[$questionId])) {
                $details[$questionId] = [];
                $details[$questionId]['QuestionNo'] = $questionId;
                $details[$questionId]['QuestionText'] = $questionText;
                $details[$questionId]['QuestionHandle'] = $questionHandle;
            }

            $details[$questionId]['AnswerText'][] = $answerText;
        }

        $questionDetails['Questions'] = $details;
        $dataResult['SessionId'] = $sessionID;
        $dataResult['SurveyAnswers'] = $questionDetails;

        $returnData = $dataResult;

        return $this->helper->returnData(200, "success", $returnData);
    }

    /**
     * Get resume quiz details
     */
    public function getResumeQuizDetails(Request $request) {

        $sessionID = $request->input('sessionID');

        $emailDetails =  DB::select('
            SELECT u.user_email
                FROM quiz_user as qu
            JOIN users as u
                on u.id =  qu.user_id
            WHERE
                session_id = ? ', [$sessionID]);

        $emailID = isset($emailDetails[0]) ? $emailDetails[0]->user_email : '';
        $customerDetails = $this->getCustomerDetails($emailID);

        $results = DB::select('
        select
            qm.id,
            qm.question,
            IFNULL(qr.response,ur.response_text) as response,
            qm.page
        from
            user_responses as ur
        join
            question_master as qm
        on
            qm.id = ur.qn_id
        left join
            question_responses as qr
        on
            ur.response_id = qr.id
        where
            session_id = ?', [$sessionID]);

        $dataResult = [];
        $details = [];

        foreach ($results as $result) {
            $questionId = $result->id;
            $questionText = $result->question;
            $answerText = $result->response;
            $questionHandle = $result->page;

            $temp = [];

            if (!isset($details[$questionId])) {
                $details[$questionId] = [];
                $details[$questionId]['QuestionNo'] = $questionId;
                $details[$questionId]['QuestionText'] = $questionText;
                $details[$questionId]['QuestionHandle'] = $questionHandle;
            }

            $details[$questionId]['AnswerText'][] = $answerText;
        }

        $questionDetails['Questions'] = $details;
        $dataResult['SessionId'] = $sessionID;
        $dataResult['SurveyAnswers'] = $questionDetails;
        $dataResult['CustomerDetails'] = $customerDetails;

        $returnData = $dataResult;

        return $this->helper->returnData(200, "success", $returnData);
    }

    /**
     * Update password tag to the customer
     */
    public function updatePasswordTag(Request $request) {

        $customerId = $request->input('customerID');
        $customerEmail = $request->input('email');

        if (empty($customerId) && empty($customerEmail)) {
            $returnData = [
                'error' => 'Parameter missing - customerID or email'
            ];

            Log::channel('quizControllerLog')->info("updatePasswordTag: Parameter missing - customerID or email");

            return $this->helper->returnData(600, "error", $returnData);
        }

        if (!empty($customerId)) {
            $result = $this->api->rest('GET', htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json'), null);

            if (!isset($result['body']['customer']) || empty($result['body']['customer'])) {
                $returnData = [
                    'error' => 'No such customer found in Shopify'
                ];

                Log::channel('quizControllerLog')->info("updatePasswordTag: Customer not found in Shopify(CustomerId:".$customerId.")");

                return $this->helper->returnData(600, "error", $returnData);
            }

            $customerId = !empty($result['body']['customer']['id']) ? trim($result['body']['customer']['id']) : "";
            $customerTag = !empty($result['body']['customer']['tags']) ? trim($result['body']['customer']['tags']) : "";

            $params = array(
                'customer' => array(
                    'id' => $customerId,
                    'tags' => $customerTag.", passwordSet"
                )
            );

            $try = 1;
            $retry = false;
            do {
                $updatedResult = $this->api->rest('PUT', '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json', $params);
                if (!isset($updatedResult['body']['customer']['id'])) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while ($retry && $try <=3);

            //return error
            if (!isset($updatedResult['body']['customer']['id'])) {
                $returnData = [
                    'error' => 'No such customer'
                ];
                Log::channel('quizControllerLog')->info("updatePasswordTag: Failed to update the customer tag(CustomerId:".$customerId.")");
                return $this->helper->returnData(600, "error", $returnData);
            }

            //success
            $returnData = [
                'success' => 'Password Tag successfully updated'
            ];

            Log::channel('quizControllerLog')->info("updatePasswordTag: Password Tag successfully updated(CustomerId:".$customerId.")");

            return $this->helper->returnData(200, "success", $returnData);
        }

        if (!empty($customerEmail)) {

            $customerDetails = $this->customerObj->getCustomerDetailBySearch($customerEmail);

            if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                $error = "Unknown error";
                if (!empty($customerDetails['error'])) {

                    $error = json_encode($customerDetails['error']);
                } elseif (!empty($customerDetails['errors'])) {
                    
                    $error = json_encode($customerDetails['errors']);
                }
                if (empty($error)) {

                    $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $customerEmail . ")";
                }
                $returnData = [
                    "error" => $error
                ];

                Log::channel('quizControllerLog')->info("updatePasswordTag: Failed to get the customer details from Shopify(emailAddress:".$customerEmail."), 
                Error: (".$error.")");
            }

            if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
                $returnData = [
                    'error' => 'email not found',
                ];

                Log::channel('quizControllerLog')->info("updatePasswordTag: Not found any email address(Email:".$customerEmail.")");

                return $this->helper->returnData(600, "error", $returnData);
            }

            $customer = [];
            foreach ($customerDetails['customers'] as $cust) {
                if (trim(strtolower($cust['email'])) == trim(strtolower($customerEmail))) {
                    $customer = $cust;
                    break;
                }
            }

            if (empty($customer)) {

                Log::channel('quizControllerLog')->info("updatePasswordTag: Failed the found the user in Shopify(Email:".$customerEmail.")");

                $returnData = [
                    'error' => 'No such customer found in Shopify'
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            $customerId = $customer['id'];
            $customerTag = !empty($customer['tags']) ? trim($customer['tags']) : "";

            $params = array(
                'customer' => array(
                    'id' => $customerId,
                    'tags' => $customerTag.", passwordSet"
                )
            );

            $try = 1;
            $retry = false;
            do {
                $result = $this->api->rest('PUT', '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json', $params);
                if (!isset($result['body']['customer']['id'])) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while ($retry && $try <=3);

            //return error
            if (!isset($result['body']['customer']['id'])) {
                $returnData = [
                    'error' => 'No such customer'
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            //success
            $returnData = [
                'success' => 'Password Tag successfully updated'
            ];

            return $this->helper->returnData(200, "success", $returnData);
        }
    }

    /**
     * Get customer details
     */
    public function getCustomerDetails($email) {

        $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('quizControllerLog')->info("getCustomerDetails: Failed to get the customer details from Shopify(emailAddress:".$email."), 
            Error: (".$error.")");
        }

        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            $returnData = [
                'email' => $email,
                'userStatus' => "",
                'passwordSet' => "",
                'registrationAuth' => "",
                'authMatch' => ""
            ];
            return $returnData;
        }

        $customer = [];
        foreach ($customerDetails['customers'] as $cust) {
            if (trim(strtolower($cust['email'])) == trim(strtolower($email))) {
                $customer = $cust;
                break;
            }
        }
        $customerStatus = !empty($customer['state']) ? trim($customer['state']) : "";
        $customerTag = !empty($customer['tags']) ? trim($customer['tags']) : "";
        $customerEmail = $customer['email'];

        $registrationAuth = DB::table('users')->select('registration_auth')->where([
            'user_email' => $email,
        ])->value('registration_auth');


        if (strpos($customerTag, "passwordSet") !== false) {
            $passwordSet = "true";
        } else {
            $passwordSet = "false";
        }

        $returnData = [
            'email' => $email,
            'userStatus' => $customerStatus,
            'passwordSet' => $passwordSet,
            'registrationAuth' => !empty($registrationAuth) ? $registrationAuth : '',
            'authMatch' => !empty($registrationAuth) ? "true" : "false"
        ];

        return $returnData;
    }

    /**
     * Update customer password
     */
    public function updateResetPassword(Request $request) {

        $customerEmail = !empty($request->input('email')) ? trim($request->input('email')) : "";
        $customerAuthToken = !empty($request->input('registration_auth')) ? trim($request->input('registration_auth')) : "";
        $customerPassword = !empty($request->input('password')) ? strtolower($request->input('password')) : "";
        $customerConfPassword = !empty($request->input('confirm_password')) ? strtolower($request->input('confirm_password')) : "";
    
        $user = DB::table('users')->where('user_email', '=', $customerEmail)->get();  
    
        if (!isset($user[0]->user_email)) {  
            $returnData = [  
                'error' => 'No such customer found'  
            ];  

            Log::channel('quizControllerLog')->info("updateResetPassword: No such customer found");
    
            return $this->helper->returnData(600, "error", $returnData);  
        }  
    
        $registrationAuth = DB::table('users')->select('registration_auth')->where([  
            'user_email' => $customerEmail,  
        ])->value('registration_auth');  

        $registrationAuth = !empty($registrationAuth) ? trim($registrationAuth) : "";
    
        if ($customerAuthToken != $registrationAuth) {  
            $returnData = [  
                'error' => 'Invalid Token provided'  
            ];
            
            Log::channel('quizControllerLog')->info("updateResetPassword: Invalid Token provided(Email: ".$customerEmail.")");
    
            return $this->helper->returnData(600, "error", $returnData);  
        }  
    
        if ($customerPassword != $customerConfPassword) {  
            $returnData = [  
                'error' => 'Both passwords must be the same'  
            ];  

            Log::channel('quizControllerLog')->info("updateResetPassword: Both passwords must be the same(Email: ".$customerEmail.")");
    
            return $this->helper->returnData(600, "error", $returnData);  
        }  
    

        $customerDetails = $this->customerObj->getCustomerDetailBySearch($customerEmail);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $customerEmail . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('quizControllerLog')->info("updateResetPassword: Failed to get the customer details from Shopify(emailAddress:".$customerEmail."), 
            Error: (".$error.")");
        }


        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {

            $returnData = [  
                'error' => 'No such Customer found in Shopify'  
            ];  
            Log::channel('quizControllerLog')->info("updateResetPassword: No such Customer found in Shopify(Email: ".$customerEmail.")");
            return $this->helper->returnData(600, "error", $returnData);
        }

        $customer = [];
        foreach ($customerDetails['customers'] as $cust) {
            if (trim(strtolower($cust['email'])) == strtolower($customerEmail)) {
                $customer = $cust;
                break;
            }
        }
    
        $customerTag = !empty($customer['tags']) ? trim($customer['tags']) : "";  
        $customerId = !empty($customer['id']) ? trim($customer['id']) : "";  
        $customerStatus = !empty($customer['state']) ? trim($customer['state']) : "";  
    
        if ($customerStatus == "disabled") {  
            $returnData = [  
                'error' => 'Customer not enabled',  
            ];  
            Log::channel('quizControllerLog')->info("updateResetPassword: Customer not enabled(Email: ".$customerEmail.")");
            return $this->helper->returnData(600, "error", $returnData);  
        }  
    
        if (strpos($customerTag, "passwordSet") !== false) {  
            $passwordSet = "true";  
    
            $returnData = [  
                'error' => 'passwordSet',  
            ];  
            Log::channel('quizControllerLog')->info("updateResetPassword: Already reset the password(CustomerId: ".$customerId.")");
            return $this->helper->returnData(600, "error", $returnData);  
    
        } else {  
            $passwordSet = "false";  
    
            $params = array(  
                'customer' => array(  
                    'id' => $customerId,  
                    'password' => $customerPassword,  
                    'password_confirmation' => $customerConfPassword,  
                    'tags' => $customerTag.", passwordSet"  
                ),  
            );  
    
            //try a maximum of three times in case of an unexpected response  
            $try = 1;  
            $retry = false;  
            do {  
                $result = $this->api->rest('PUT', '/admin/api/' . $this->shopify_api_version . '/customers/' . $customerId . '.json', $params);  
                if (!isset($result->body->customer->id)) {
                    
                    sleep(1);
                    $retry = true;  
                    $try++;  
                }  
            } while ($retry && $try <= 3);  
    
            //return error  
            if (!isset($result['body']['customer']['id'])) {  
                $returnData = [  
                    'error' => 'No such customer',  
                ];  
                Log::channel('quizControllerLog')->info("updateResetPassword: Failed to update the customer(CustomerId: ".$customerId.")");
                return $this->helper->returnData(600, "error", $returnData);  
            }  
            
            /* code changes for multipass login */
            $customer_login_data = array(
                "email" => $customerEmail,
                "return_to" => env("MULTI_PASS_REDIRECT_TO", true),
            );
            $multiPassToken = env('MULTI_PASS_TOKEN', true);
            $multipass = new ShopifyMultipass($multiPassToken);
            $token = $multipass->generate_token($customer_login_data);
            /* code changes for multipass login */
    
            $returnData = [  
                'success' => 'Password successfully updated',  
                'multi_pass_token' => ($token != '')?$token:"",
            ];  
    
            return $this->helper->returnData(200, "success", $returnData);  
        }
    }

    /**
     * Get customer activation status
     */
    public function getCustomerActivationStatus(Request $request) {

        $customerEmail = !empty($request->input('email')) ? trim($request->input('email')) : "";

        if (empty($customerEmail)) {
            $returnData = [
                'error' => 'Parameter missing - email'
            ];
            Log::channel('quizControllerLog')->info("getCustomerActivationStatus: Parameter missing - email");
            return $this->helper->returnData(600, "error", $returnData);
        }

        $customerDetails = $this->customerObj->getCustomerDetailBySearch($customerEmail);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $customerEmail . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('quizControllerLog')->info("getCustomerActivationStatus: Failed to get the customer details from Shopify(emailAddress:".$customerEmail."), 
            Error: (".$error.")");
        }


        //$result = $this->api->rest('GET', htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$customerEmail), null);

        $returnData = [
            'activation_status' => null,
            'passwordSet' => null
        ];

        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
 
            Log::channel('quizControllerLog')->info("getCustomerActivationStatus: No such Customer found in Shopify(Email: ".$customerEmail.")");
            return $this->helper->returnData(200, "success", $returnData);
        }

        $customer = [];
        foreach ($customerDetails['customers'] as $cust) {
            if (trim(strtolower($cust['email'])) == strtolower($customerEmail)) {
                $customer = $cust;
                break;
            }
        }
    
        $customerTag = !empty($customer['tags']) ? trim($customer['tags']) : "";  
        $customerStatus = !empty($customer['state']) ? trim($customer['state']) : "";

        if (strpos($customerTag, "passwordSet") !== false) {
            $passwordSet = "true";
        } else {
            $passwordSet = "false";
        }
        $returnData = [
            'activation_status' => $customerStatus,
            'passwordSet' => $passwordSet
        ];

         return $this->helper->returnData(200, "success", $returnData);
    }


    /**
     * Call recommendation
     */
    public function callRecommendation($request) {

        if (!empty($request)) {
            $url = Config::get('recommendation.recommendationUrl');
            $type = 'POST';
            $api_key = null;
            $api_passwd = null;

            return $this->executeCurl($url, $type, $request, $api_key, $api_passwd);
        } else {

            return null;
        }
    }

    /**
     * Call get pack
     */
    public function callGetPack($request) {

        if (!empty($request)) {
            $url = Config::get('recommendation.packUrl');
            $type = 'POST';
            $api_key = null;
            $api_passwd = null;

            return $this->executeCurl($url, $type, $request, $api_key, $api_passwd);
        } else {
            return null;
        }
    }

    /**
     * CURL call to get the recommendation
     */
    public function executeCurl($url, $type, $data, $api_key, $api_passwd) {

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/json",
        ));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); // temporarily disable SSL verification
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // temporarily disable SSL verification

        if ($type == "POST") {
            curl_setopt($ch, CURLOPT_POST, true);
            $json = json_encode(new ArrayValue($data));
            $payload = '"' . addslashes($json) . '"';
            curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        }

        if ($type == "PUT") {
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
            $json = json_encode(new ArrayValue($data));
            $payload = '"' . addslashes($json) . '"';
            curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        }

        $result = curl_exec($ch);

        if (curl_errno($ch)) {
            echo 'Request Error:' . curl_error($ch);
            return false;
        }

        Log::channel('quizControllerLog')->info("curl result  => " . $result);
        return $result;
    }

    /**
     * Update customer password
     */
    public function updateCustomerPassword(Request $request) {

        Log::channel('quizControllerLog')->info("------updateCustomerPassword-------");

        $email = $request->input('email');
        $responses = $request->input('responses');
        $custPassword = isset($responses[0]) ? $responses[0] : '';
        $sessionID = $request->input('sessionID');
        $authToken  = $request->input('token');

        Log::channel('quizControllerLog')->info("Request: ".$request->input());

        if (empty($sessionID)) {
            $returnData = [
                'error' => 'Parameter missing - sessionID',
            ];
            Log::channel('quizControllerLog')->info("updateCustomerPassword: Parameter missing - sessionID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email',
            ];
            Log::channel('quizControllerLog')->info("updateCustomerPassword: Parameter missing - email");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($custPassword)) {
            $returnData = [
                'error' => 'Parameter missing - password',
            ];
            Log::channel('quizControllerLog')->info("updateCustomerPassword: Parameter missing - password");
            return $this->helper->returnData(600, "error", $returnData);
        }

        $userEmailDetails = DB::table('users')
                ->select('*')
                ->where('users.user_email','=', $email)
                ->where('users.registration_auth','=', $authToken)
                ->get();


        if (!isset($userEmailDetails[0]->user_email)) {

            $returnData = [
                'error' => 'email not matching with sessionID',
            ];
            Log::channel('quizControllerLog')->info("updateCustomerPassword: email not matching with sessionID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (trim(strtolower($userEmailDetails[0]->user_email)) == trim(strtolower($email))) {


            $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

            if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                $error = "Unknown error";
                if (!empty($customerDetails['error'])) {

                    $error = json_encode($customerDetails['error']);
                } elseif (!empty($customerDetails['errors'])) {
                    
                    $error = json_encode($customerDetails['errors']);
                }
                if (empty($error)) {

                    $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
                }
                $returnData = [
                    "error" => $error
                ];

                Log::channel('quizControllerLog')->info("updateCustomerPassword: Failed to get the customer details from Shopify(emailAddress:".$email."), 
                Error: (".$error.")");
            }

            if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
    
                $returnData = [
                    'error' => 'email not found in shopify',
                ];
                Log::channel('quizControllerLog')->info("updateCustomerPassword: email not found in Shopify(Email: ".$email.")");
                return $this->helper->returnData(600, "error", $returnData);
            }

            $customer = [];
            foreach ($customerDetails['customers'] as $cust) {
                if (trim(strtolower($cust['email'])) == strtolower($email)) {
                    $customer = $cust;
                    break;
                }
            }
        
            $customerId = !empty($customer['id']) ? trim($customer['id']) : "";
            $customerTag = !empty($customer['tags']) ? trim($customer['tags']) : "";  

            $params = array(
                'customer' => array(
                    'id' => $customerId,
                    'password' => $custPassword,
                    'password_confirmation' => $custPassword,
                    'tags' => $customerTag.", passwordSet"
                ),
            );

            //try a maximum of three times in case of an unexpected response
            $try = 1;
            $retry = false;
            do {
                $result = $this->api->rest('PUT', '/admin/api/' . $this->shopify_api_version . '/customers/' . $customerId . '.json', $params);
                if (!isset($result->body->customer->id)) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while ($retry && $try <= 3);

            //return error
            if (!isset($result['body']['customer']['id'])) {
                $returnData = [
                    'error' => 'No such customer',
                ];
                Log::channel('quizControllerLog')->info("updateCustomerPassword: Failed to update the customer(Email: ".$email.")");
                return $this->helper->returnData(600, "error", $returnData);
            }
            
            /* code changes for multipass login */
            $customer_login_data = array(
                "email" => $email
            );
            $multiPassToken = env('MULTI_PASS_TOKEN', true);
            $multipass = new ShopifyMultipass($multiPassToken);
            $token = $multipass->generate_token($customer_login_data);
            
            /* code changes for multipass login */

            //success
            $returnData = [
                'success' => 'Password successfully updated',
                'multi_pass_token' => ($token != '')?$token:"",
                'customer_id' => $result['body']['customer']['id']
            ];

            Log::channel('quizControllerLog')->info("------------------------");

            return $this->helper->returnData(200, "success", $returnData);
        }
    }

    /**
     * Add exit tag to the customer
     */
    public function addExitTag(Request $request) {

        Log::channel('quizControllerLog')->info("--------addExitTag---------");

        $email = !empty($request->input('email')) ? trim($request->input('email')) : "";
        $exitTag = !empty($request->input('tag')) ? trim($request->input('tag')) : "";
        $sessionID = !empty($request->input('sessionID')) ? trim($request->input('sessionID')) : "";
        $questionID = !empty($request->input('questionID')) ? trim($request->input('questionID')) : "";
        $questionNo = !empty($request->input('questionNo')) ? trim($request->input('questionNo')) : "";
        $responseID = !empty($request->input('responseID')) ? trim($request->input('responseID')) : "";

        Log::channel('quizControllerLog')->info("Request:".$request->input());

        if (empty($sessionID)) {
            $returnData = [
                'error' => 'Parameter missing - sessionID',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: Parameter missing - sessionID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: Parameter missing - email");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($exitTag)) {
            $returnData = [
                'error' => 'Parameter missing - tag',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: Parameter missing - tag");
            return $this->helper->returnData(600, "error", $returnData);
        }

        Log::channel('quizControllerLog')->info("ExitTag:".$exitTag);

        if (empty($questionID)) {
            $returnData = [
                'error' => 'Parameter missing - questionID',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: Parameter missing - questionID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($responseID)) {
            $returnData = [
                'error' => 'Parameter missing - responseID',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: Parameter missing - responseID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        $userEmailDetails = DB::select('
            select
                u.user_email
            from
                quiz_user as qr
            join
                users as u
            on
                u.id = qr.user_id
            where
                qr.session_id = ?', [$sessionID]);

        if (!isset($userEmailDetails[0]->user_email)) {
            $returnData = [
                'error' => 'email not matching with sessionID',
            ];
            Log::channel('quizControllerLog')->info("addExitTag: email not matching with sessionID");
            return $this->helper->returnData(600, "error", $returnData);
        }

        if (trim(strtolower($userEmailDetails[0]->user_email)) == strtolower($email)) {

            /** send an email for exit tag cases */
            if (!empty($email)) {

                $triggerEmailRequest = array(
                    "campaignId" => 882346,
                    "recipientEmail" => $email,
                );

                $eventTag = '';

                if ($exitTag == "exit-surgery") {
                    $eventTag = 'Quiz Surgery';
                } elseif ($exitTag == "exit-veganDiet") {
                    $eventTag = 'quizExit-Vegan';
                } else if ($exitTag == "exit-vegetarianDiet") {
                    $eventTag = 'quizExit-Vegetarian';
                }

                $eventRequest = array(
                    "email" => $email,
                    "eventName" => $eventTag,
                    // "campaignId" => 882346,
                    "dataFields" => array("userName" => ""),
                );
                // Trigger Event
                $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

                if (isset($eventResponse->code) && trim(strtolower($eventResponse->code)) == 'success') {
                    Log::channel('quizControllerLog')->info('Event-triggered successfully Please check mail : ' . $email);
                } else {
                    Log::channel('quizControllerLog')->info('Event not triggered');
                    Log::channel('quizControllerLog')->info("Response: ".json_encode($eventResponse));
                }

            }

            /*send an email for exit tag cases*/
            $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

            if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                $error = "Unknown error";
                if (!empty($customerDetails['error'])) {

                    $error = json_encode($customerDetails['error']);
                } elseif (!empty($customerDetails['errors'])) {
                    
                    $error = json_encode($customerDetails['errors']);
                }
                if (empty($error)) {

                    $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
                }
                $returnData = [
                    "error" => $error
                ];

                Log::channel('quizControllerLog')->info("addExitTag: Failed to get the customer details from Shopify(emailAddress:".$email."), 
                Error: (".$error.")");
            }

            if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
                $returnData = [
                    'error' => 'email not found',
                ];
                Log::channel('quizControllerLog')->info("addExitTag: Not found any email address(emailAddress:".$email.")");
                return $this->helper->returnData(600, "error", $returnData);
            }

            $customer = [];
            foreach ($customerDetails['customers'] as $cust) {
                if (trim(strtolower($cust['email'])) == strtolower($email)) {
                    $customer = $cust;
                    break;
                }
            }

            if (empty($customer)) {
                Log::channel('quizControllerLog')->info("addExitTag: Not found the searched email address in Shopify(emailAddress:".$email.")");
                $returnData = [
                    'error' => 'email not found',
                ];
                return $this->helper->returnData(600, "error", $returnData);
            }

            $customerId = !empty($customer['id']) ? trim($customer['id']) : "";
            $tags = !empty($customer['tags']) ? trim($customer['tags']) : "";

            $newTags = $tags . "," . $exitTag;

            $params = array(
                'customer' => array(
                    'id' => $customerId,
                    'tags' => $newTags,
                ),
            );
            Log::channel('quizControllerLog')->info("addExitTag: Customer update request - ".json_encode($params));
            $lastInsertedId = null;

            $lastInsertedId = DB::table('user_responses')->insertGetId(
                [
                    'session_id' => $sessionID,
                    'qn_id' => $questionID,
                    'qn_no' => $questionNo,
                    'response_id' => $responseID,
                    // 'response_text' => null,
                    'deleted' => 0,
                    'status' => 0,
                ]
            );

            if (empty($lastInsertedId)) {
                $returnData = [
                    'error' => 'Insertion Failed',
                ];
                Log::channel('quizControllerLog')->info("addExitTag: Insertion Failed(Email: ".$email.")");
                return $this->helper->returnData(600, "error", $returnData);
            }

            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            //try a maximum of three times in case of an unexpected response
            $try = 1;
            $retry = false;
            do {
                $result = $this->api->rest('PUT', '/admin/api/' . $this->shopify_api_version . '/customers/' . $customerId . '.json', $params);

                if (!isset($result['body']['customer']['id'])) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while ($retry && $try <= 3);

            //return error
            if (!isset($result['body']['customer']['id'])) {
                $returnData = [
                    'error' => 'No such customer',
                ];

                Log::channel('quizControllerLog')->info("addExitTag: Failed to update the customer in Shopify(Email:".$email.")");
                return $this->helper->returnData(600, "error", $returnData);
            }

            //success
            $returnData = [
                'success' => 'Successfully updated',
            ];

            Log::channel('quizControllerLog')->info("-------------------------");

            return $this->helper->returnData(200, "success", $returnData);
        }
    }

    /**
     * Get pack count
     */
    public function getPackCount(Request $request) {

        $postData = $request->input();

        if (empty($postData)) {
            $returnData = [
                'error' => 'Empty parameters',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $packRequest['ProductItems'] = array();

        foreach ($postData as $data) {
            $bulkID = isset($data['bulkID']) ? $data['bulkID'] : '';
            $qty = isset($data['qty']) ? $data['qty'] : '';

            if (empty($bulkID)) {
                $returnData = [
                    'error' => 'Empty parameter value - bulkID',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            if (empty($qty)) {
                $returnData = [
                    'error' => 'Empty parameter value - qty',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            if ($bulkID == 'vitamin-pack') {
                continue;
            }

            $tempPackRequest['BulkItem'] = $bulkID;
            $tempPackRequest['Quantity'] = $qty;
            array_push($packRequest['ProductItems'], $tempPackRequest);

        }

        if (empty($packRequest['ProductItems'])) {

            $returnData = [
                'error' => 'Empty ProductItems',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $returnASCII = $this->encode_items($packRequest);
        $packDetails = $this->callGetPack($returnASCII);

        if (strpos($packDetails, 'Packs') !== false) {
            $stripData = explode("Packs", $packDetails);
            $packData = $stripData[1];
            $packData = ltrim($packData, ":");
            $packData = rtrim($packData, "}");

            $returnData = [
                'Pack' => $packData,
            ];

            return $this->helper->returnData(200, "success", $returnData);

        } else {
            $returnData = [
                'error' => 'No Pack details',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

    }

    /**
     * Encode items
     */
    public function encode_items($array) {

        foreach ($array as $key => $value) {

            if (is_array($value)) {

                $array[$key] = $this->encode_items($value);
            } else {
                // $array[$key] = mb_convert_encoding($value, 'ASCII');
                $array[$key] = mb_convert_encoding($value, 'UTF-8');
            }
        }

        return $array;
    }

    /**
     * Retake quiz
     */
    public function retakeQuiz(Request $request) {

        $page = $request->input('page');
        $sessionID = $request->input('sessionID');
        $question = $request->input('question');
        $questionID = $request->input('questionID');
        $questionNo = $request->input('questionNo');
        $responses = $request->input('responses');
        $responseIds = $request->input('responseIDs');
        $responseKeys = $request->input('responseKeys');
        $email = $request->input('email');

        $retakeID = $request->input('retakeID');

        Log::channel('quizControllerLog')->info("---------retakeQuiz---------");

        if ($retakeID) {
            $recordDetails = $this->getQuizData($sessionID);

            if (empty($recordDetails)) {

                if (!empty($email)) {
                    $retakeUserID = DB::table('users')->select('id')->where([
                        'user_email' => $email,
                    ])->value('id');

                    if (empty($retakeUserID)) {
                        $returnData = [
                            'error' => 'Could not found existing User details',
                        ];

                        return $this->helper->returnData(600, "error", $returnData);
                    }

                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $retakeUserID,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);

                }

                $recordDetails = $this->getQuizData($retakeID);

                if (empty($recordDetails)) {
                    $returnData = [
                        'error' => 'Could not found existing Quiz details',
                    ];

                    return $this->helper->returnData(600, "error", $returnData);
                }

                $retakeQuestionData = $recordDetails['SurveyAnswers']['Questions'][$questionID];
                $retakeResponseData = $recordDetails['SurveyAnswers']['Questions'];

                foreach ($retakeResponseData as $key => $value) {

                    $answerIds = null;
                    if (isset($value['AnswerID'])) {
                        $answerIds = $value['AnswerID'];
                    }

                    $answerText = $value['AnswerText'];
                    $questNo = $value['QuestionNo'];
                    $questId = $value['QuestionId'];

                    if ($key < $questionNo) {
                        if (!empty($answerIds)) {
                            $lastInsertedId = null;
                            foreach ($answerIds as $answerId) {
                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questId,
                                        'qn_no' => $questNo,
                                        'response_id' => $answerId,
                                        // 'response_text' => null,
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );
                            }

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        } else {
                            $lastInsertedId = DB::table('user_responses')->insertGetId(
                                [
                                    'session_id' => $sessionID,
                                    'qn_id' => $questId,
                                    'qn_no' => $questNo,
                                    'response_id' => null,
                                    'response_text' => $answerText[0],
                                    'deleted' => 0,
                                    'status' => 0,
                                ]
                            );

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        }
                    }

                    if ($questionNo == $key) {

                        if (!empty($responseIds)) {
                            $lastInsertedId = null;
                            foreach ($responseIds as $responseId) {

                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questionID,
                                        'qn_no' => $questionNo,
                                        'response_id' => $responseId,
                                        // 'response_text' => null,
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );
                            }

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        } else {
                            $lastInsertedId = DB::table('user_responses')->insertGetId(
                                [
                                    'session_id' => $sessionID,
                                    'qn_id' => $questionID,
                                    'qn_no' => $questionNo,
                                    'response_id' => null,
                                    'response_text' => $responses[0],
                                    'deleted' => 0,
                                    'status' => 0,
                                ]
                            );

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        }

                    }
                }

            }
        }

        $questionTextArray = array('31', '48');

        if (empty($sessionID)) {

            // error
            $returnData = [
                'error' => 'Invalid Session',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if ($page == 'close') {
            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            $other = DB::table('question_responses')->select('id')->where([
                ['qn_id', '=', '12'],
                ['response', '=', 'Other'],
            ])->value('id');

            $userResponse = DB::select('
                select
                    qm.id,
                    qm.question,
                    qm.question_no,
                    qr.response,
                    qr.response_key,
                    ur.response_text
                from
                    user_responses as ur
                join
                    question_master as qm
                on
                    qm.id = ur.qn_id
                left join
                    question_responses as qr
                on
                    ur.response_id = qr.id
                where
                    session_id = ?', [$sessionID]);

            $dataResult = [];
            $details = [];

            if (!empty($userResponse)) {

                foreach ($userResponse as $response) {
                    $questionId = $response->id;
                    $questionNo = $response->question_no;
                    $questionText = $response->question;
                    $answerText = $response->response;
                    $answerKey = $response->response_key;
                    $responseText = $response->response_text;

                    if ($questionNo == '11' && $responseText != null) {
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                        $details[$questionNo]['AnswerText'][] = $other;
                        continue;
                    }

                    if (!isset($details[$questionNo])) {
                        $details[$questionNo] = [];
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                    }

                    if (in_array($questionNo, $questionTextArray)) {
                        $details[$questionNo]['AnswerText'][] = $answerText;
                    } else {
                        $details[$questionNo]['AnswerText'][] = empty($answerKey) ? $responseText : strval($answerKey);
                    }
                }
                $questionDetails = array_values($details);
                $dataResult['SurveyAnswers']['SessionId'] = $sessionID;
                $dataResult['SurveyAnswers']['Questions'] = $questionDetails;
            }

            $returnData = $dataResult;
            $jsonData = json_encode($returnData);
            Log::channel('quizControllerLog')->info($jsonData);
            // CALL to Recommendation API
            $recommendation = $this->callRecommendation($returnData);
            Log:info($recommendation);
            $recommendationObj = json_decode($recommendation);
            if (isset($recommendationObj->Result)) {
                $recommendationObj = $recommendationObj->Result;
            }

            if (isset($recommendationObj->Recommendation) && !empty($recommendationObj->Recommendation)) {

                $lastInsertedId = DB::table('recommendations')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'email' => isset($recommendationObj->Email) ? $recommendationObj->Email : '',
                        'first_name' => isset($recommendationObj->Firstname) ? $recommendationObj->Firstname : '',
                        'sex_profile' => $recommendationObj->SexProfile,
                        'sex' => $recommendationObj->Sex,
                        'age' => $recommendationObj->Age,
                        'zipcode' => $recommendationObj->ZipCode,
                        'latitude' => $recommendationObj->Latitude,
                        'longitude' => $recommendationObj->Longitude,
                        'city' => $recommendationObj->City,
                        'state' => $recommendationObj->State,
                        'allergens' => json_encode($recommendationObj->Allergens),
                        'deficiencies' => json_encode($recommendationObj->Deficiencies),
                        'recommendation' => json_encode($recommendationObj->Recommendation),
                        'buzz_words' => json_encode($recommendationObj->BuzzWordsTips),
                        'north37' => $recommendationObj->North37,
                        'prostate_interest' => $recommendationObj->ProstateInterest,
                        'prenatal_interest' => $recommendationObj->PrenatalInterest,
                        'postnatal_interest' => $recommendationObj->PostnatalInterest,
                        'healthy_interest' => $recommendationObj->HealthyMentrualCycleInterest,
                        'perimenopause_interest' => $recommendationObj->PerimenopauseInterest,
                        'menopause_interest' => $recommendationObj->MenopauseInterest,
                        'planning_pregnancy' => $recommendationObj->PlanningPregnancy,
                        'pregnant' => $recommendationObj->Pregnant,
                        'breastfeeding' => $recommendationObj->Breastfeeding,
                        'taking_medication' => $recommendationObj->TakingMedication,
                        'blood_clotting_issue' => $recommendationObj->BloodClottingIssue,

                    ]
                );

                if (empty($lastInsertedId)) {
                    $returnData = [
                        'error' => 'Insertion Failed',
                    ];

                    return $this->helper->returnData(600, "error", $returnData);
                }

                // For the tagging purpose
                $questionResponse = DB::select('
                    select
                        qm.id,
                        qm.question,
                        qr.response
                    from
                        user_responses as ur
                    join
                        question_master as qm
                    on
                        qm.id = ur.qn_id
                    left join
                        question_responses as qr
                    on
                        ur.response_id = qr.id
                    where
                        session_id = ? and ur.qn_id in (14,32)', [$sessionID]);

                if (!empty($questionResponse)) {
                    $tagQuestion = array();
                    foreach ($questionResponse as $qResponse) {
                        if (!isset($tagQuestion[$qResponse->id])) {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        } else {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        }
                    }
                }

                $reasonProduct = $recommendationObj->Recommendation->ReasonsForProduct;

                if (!empty($reasonProduct)) {

                    $reason = array();
                    $recommendationObj = json_decode($recommendation);
                    if (isset($recommendationObj->Result)) {
                        $recommendationObj = $recommendationObj->Result;
                    }
                    foreach ($reasonProduct as $value) {
                        $stripValue = preg_replace('/[^a-zA-Z0-9-;.,+"()%\/(&deg;) ]/', '', $value);
                        $reason[] = $stripValue;
                    }
                    $recommendationObj->Recommendation->ReasonsForProduct = $reason;
                }

                // Tag User
                $tagUser = dispatch(new TagUserJob($recommendation, $tagQuestion, $sessionID));

                // return $this->helper->returnData(200, "success", json_decode($recommendation) );
                return $this->helper->returnData(200, "success", $recommendationObj);

            } else {
                $returnData = [
                    'error' => 'Empty Recommendation',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

        }

        if (empty($questionID)) {
            // error

            $returnData = [
                'error' => 'Quiz not found',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        // To delete the already existing question and update the new record
        $deleteQuizDetails = DB::table('user_responses')
            ->where([
                ['session_id', '=', $sessionID],
                ['qn_id', '>=', $questionID],
            ])->delete();

        if (!empty($responseIds)) {
            $lastInsertedId = null;
            foreach ($responseIds as $responseId) {
                $lastInsertedId = DB::table('user_responses')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'qn_id' => $questionID,
                        'qn_no' => $questionNo,
                        'response_id' => $responseId,
                        // 'response_text' => null,
                        'deleted' => 0,
                        'status' => 0,
                    ]
                );
            }

            if (empty($lastInsertedId)) {
                $returnData = [
                    'error' => 'Insertion Failed',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            $returnData = [
                'stored' => true,
                'message' => 'success',
            ];

        } else {

            $lastInsertedId = DB::table('user_responses')->insertGetId(
                [
                    'session_id' => $sessionID,
                    'qn_id' => $questionID,
                    'qn_no' => $questionNo,
                    'response_id' => null,
                    'response_text' => $responses[0],
                    'deleted' => 0,
                    'status' => 0,
                ]
            );

            if (empty($lastInsertedId)) {
                $returnData = [
                    'error' => 'Insertion Failed',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            $returnData = [
                'stored' => true,
                'message' => 'success',
            ];

        }

        if ($page == 'email') {
            $email = isset($responses[0]) ? $responses[0] : '';

            if (empty($email)) {
                $returnData = [
                    'error' => 'Email is empty',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

            $user = DB::table('users')->where('user_email', '=', $email)->get();

            if (!isset($user[0]->user_email)) {
                // create
                $userName = '';
                $userId = DB::table('users')->insertGetId([
                    'user_email' => $email,
                    'status' => 1,
                ]);

                $quizId = DB::table('quiz_user')->select('id')->where([
                    'session_id' => $sessionID,
                ])->value('id');

                if ($quizId) {

                    $updateQuizUser = DB::table('quiz_user')->where(['id' => $quizId])
                        ->update(['user_id' => $userId]);

                } else {
                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $userId,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);
                }

                $userNameResponse = DB::table('user_responses')->where([
                    ['session_id', '=', $sessionID],
                    ['qn_id', '=', '2'],
                ])->get();

                if ($userNameResponse[0]) {
                    $userName = $userNameResponse[0]->response_text;
                }

                $customerCreationDetails = [];
                $customerCreationDetails['email'] = $email;
                $customerCreationDetails['userName'] = $userName;
                // for setting the partner value

                if (!empty($request->input('partnerValue'))) {
                    $customerCreationDetails['partnerValue'] = $request->input('partnerValue');
                }

                // Create customers in Shopify
                $result = dispatch(new CreateShopifyCustomerJob($customerCreationDetails));

            } else {
                $userId = $user[0]->id;
                $quizId = DB::table('quiz_user')->select('id')->where([
                    'session_id' => $sessionID,
                ])->value('id');

                if ($quizId) {

                    $updateQuizUser = DB::table('quiz_user')->where(['id' => $quizId])
                        ->update(['user_id' => $userId]);

                } else {
                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $userId,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);
                }

            }

            $returnData = [
                'stored' => true,
                'message' => 'success',
            ];

        }

        Log::channel('quizControllerLog')->info("----------------------------");

        return $this->helper->returnData(200, "success", $returnData);

    }

    /**
     * New retake quiz
     */
    public function newRetakeQuiz(Request $request) {

        $page = $request->input('page');
        $sessionID = $request->input('sessionID');
        $question = $request->input('question');
        $questionID = $request->input('questionID');
        $questionNo = $request->input('questionNo');
        $responses = $request->input('responses');
        $responseIds = $request->input('responseIDs');
        $responseKeys = $request->input('responseKeys');
        $email = $request->input('email');

        $retakeID = $request->input('retakeID');

        Log::channel('quizControllerLog')->info("----------newRetakeQuiz----------");

        if ($retakeID) {
            $recordDetails = $this->getQuizData($sessionID);
            if (empty($recordDetails)) {

                if (!empty($email)) {
                    $retakeUserID = DB::table('users')->select('id')->where([
                        'user_email' => $email,
                    ])->value('id');

                    if (empty($retakeUserID)) {
                        $returnData = [
                            'error' => 'Could not find existing User details',
                        ];

                        return $this->helper->returnData(600, "error", $returnData);
                    }

                    $quizId = DB::table('quiz_user')->insertGetId([
                        'user_id' => $retakeUserID,
                        'session_id' => $sessionID,
                        'deleted' => 0,
                        'status' => 0,
                    ]);

                }

                $recordDetails = $this->getQuizData($retakeID);
                if (empty($recordDetails)) {
                    $returnData = [
                        'error' => 'Could not find existing Quiz details',
                    ];

                    return $this->helper->returnData(600, "error", $returnData);
                }

                // $retakeQuestionData = $recordDetails['SurveyAnswers']['Questions'][$questionID];
                $retakeQuestionData = $recordDetails['SurveyAnswers']['Questions'][$questionNo];
                $retakeResponseData = $recordDetails['SurveyAnswers']['Questions'];

                foreach ($retakeResponseData as $key => $value) {

                    $answerIds = null;
                    if (isset($value['AnswerID'])) {
                        $answerIds = $value['AnswerID'];
                    }

                    $answerText = $value['AnswerText'];
                    $questNo = $value['QuestionNo'];
                    $questId = $value['QuestionId'];

                    if ($key != $questionNo) {
                        if (!empty($answerIds)) {
                            $lastInsertedId = null;
                            foreach ($answerIds as $answerId) {
                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questId,
                                        'qn_no' => $questNo,
                                        'response_id' => $answerId,
                                        // 'response_text' => null,
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );
                            }

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        } else {
                            $lastInsertedId = DB::table('user_responses')->insertGetId(
                                [
                                    'session_id' => $sessionID,
                                    'qn_id' => $questId,
                                    'qn_no' => $questNo,
                                    'response_id' => null,
                                    'response_text' => $answerText[0],
                                    'deleted' => 0,
                                    'status' => 0,
                                ]
                            );

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        }
                    }

                    if ($questionNo == $key) {

                        if (!empty($responseIds)) {
                            $lastInsertedId = null;
                            // $previousAnswer = true;
                            // $previousAnswerArray = $recordDetails['SurveyAnswers']['Questions'][$questionID]['AnswerID'];

                            // foreach($responseIds as $responseId){
                            //     if(!in_array($responseId, $previousAnswerArray)){
                            //         $previousAnswer = false;
                            //     }
                            // }

                            // if($previousAnswer == false){
                            //     $sessionID = $retakeID;
                            // }

                            foreach ($responseIds as $responseId) {

                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questionID,
                                        'qn_no' => $questionNo,
                                        'response_id' => $responseId,
                                        // 'response_text' => null,
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );
                            }

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        } else {
                            // if($responses[0] == $retakeQuestionData['AnswerText'][0])
                            // {
                            //     $sessionID = $retakeID;
                            // }

                            $lastInsertedId = DB::table('user_responses')->insertGetId(
                                [
                                    'session_id' => $sessionID,
                                    'qn_id' => $questionID,
                                    'qn_no' => $questionNo,
                                    'response_id' => null,
                                    'response_text' => $responses[0],
                                    'deleted' => 0,
                                    'status' => 0,
                                ]
                            );

                            if (empty($lastInsertedId)) {
                                $returnData = [
                                    'error' => 'Insertion Failed',
                                ];

                                return $this->helper->returnData(600, "error", $returnData);
                            }

                            $returnData = [
                                'stored' => true,
                                'message' => 'success',
                            ];

                        }

                    }
                }

            }
        }

        $questionTextArray = array('31', '48');

        if (empty($sessionID)) {

            // error
            $returnData = [
                'error' => 'Invalid Session',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if ($page == 'close') {
            // $sessionData = DB::table('quiz_user')->select('session_id')->where([
            //     ['session_id', '=', $sessionID]
            // ])->value('session_id');

            // if(empty($sessionData)){
            //     $sessionID = $retakeID;
            // }

            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            $other = DB::table('question_responses')->select('id')->where([
                ['qn_id', '=', '12'],
                ['response', '=', 'Other'],
            ])->value('id');

            $userResponse = DB::select('
                select
                    qm.id,
                    qm.question,
                    qm.question_no,
                    qr.response,
                    qr.response_key,
                    ur.response_text
                from
                    user_responses as ur
                join
                    question_master as qm
                on
                    qm.id = ur.qn_id
                left join
                    question_responses as qr
                on
                    ur.response_id = qr.id
                where
                    session_id = ?', [$sessionID]);

            $dataResult = [];
            $details = [];

            if (!empty($userResponse)) {

                foreach ($userResponse as $response) {
                    $questionId = $response->id;
                    $questionNo = $response->question_no;
                    $questionText = $response->question;
                    $answerText = $response->response;
                    $answerKey = $response->response_key;
                    $responseText = $response->response_text;

                    if ($questionNo == '11' && $responseText != null) {
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                        $details[$questionNo]['AnswerText'][] = $other;
                        continue;
                    }

                    if (!isset($details[$questionNo])) {
                        $details[$questionNo] = [];
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                    }

                    if (in_array($questionNo, $questionTextArray)) {
                        $details[$questionNo]['AnswerText'][] = $answerText;
                    } else {
                        $details[$questionNo]['AnswerText'][] = empty($answerKey) ? $responseText : strval($answerKey);
                    }
                }
                $questionDetails = array_values($details);
                $dataResult['SurveyAnswers']['SessionId'] = $sessionID;
                $dataResult['SurveyAnswers']['Questions'] = $questionDetails;
            }

            $returnData = $dataResult;
            $jsonData = json_encode($returnData);
            Log::channel('quizControllerLog')->info($jsonData);
            // CALL to Recommendation API
            $recommendation = $this->callRecommendation($returnData);
            Log:info($recommendation);
            $recommendationObj = json_decode($recommendation);
            if (isset($recommendationObj->Result)) {
                $recommendationObj = $recommendationObj->Result;
            }

            if (isset($recommendationObj->Recommendation) && !empty($recommendationObj->Recommendation)) {

                $lastInsertedId = DB::table('recommendations')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'email' => isset($recommendationObj->Email) ? $recommendationObj->Email : '',
                        'first_name' => isset($recommendationObj->Firstname) ? $recommendationObj->Firstname : '',
                        'sex_profile' => $recommendationObj->SexProfile,
                        'sex' => $recommendationObj->Sex,
                        'age' => $recommendationObj->Age,
                        'zipcode' => $recommendationObj->ZipCode,
                        'latitude' => $recommendationObj->Latitude,
                        'longitude' => $recommendationObj->Longitude,
                        'city' => $recommendationObj->City,
                        'state' => $recommendationObj->State,
                        'allergens' => json_encode($recommendationObj->Allergens),
                        'deficiencies' => json_encode($recommendationObj->Deficiencies),
                        'recommendation' => json_encode($recommendationObj->Recommendation),
                        'buzz_words' => json_encode($recommendationObj->BuzzWordsTips),
                        'north37' => $recommendationObj->North37,
                        'prostate_interest' => $recommendationObj->ProstateInterest,
                        'prenatal_interest' => $recommendationObj->PrenatalInterest,
                        'postnatal_interest' => $recommendationObj->PostnatalInterest,
                        'healthy_interest' => $recommendationObj->HealthyMentrualCycleInterest,
                        'perimenopause_interest' => $recommendationObj->PerimenopauseInterest,
                        'menopause_interest' => $recommendationObj->MenopauseInterest,
                        'planning_pregnancy' => $recommendationObj->PlanningPregnancy,
                        'pregnant' => $recommendationObj->Pregnant,
                        'breastfeeding' => $recommendationObj->Breastfeeding,
                        'taking_medication' => $recommendationObj->TakingMedication,
                        'blood_clotting_issue' => $recommendationObj->BloodClottingIssue,

                    ]
                );

                if (empty($lastInsertedId)) {
                    $returnData = [
                        'error' => 'Insertion Failed',
                    ];

                    return $this->helper->returnData(600, "error", $returnData);
                }

                // For the tagging purpose
                $questionResponse = DB::select('
                    select
                        qm.id,
                        qm.question,
                        qr.response
                    from
                        user_responses as ur
                    join
                        question_master as qm
                    on
                        qm.id = ur.qn_id
                    left join
                        question_responses as qr
                    on
                        ur.response_id = qr.id
                    where
                        session_id = ? and ur.qn_id in (14,32)', [$sessionID]);

                if (!empty($questionResponse)) {
                    $tagQuestion = array();
                    foreach ($questionResponse as $qResponse) {
                        if (!isset($tagQuestion[$qResponse->id])) {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        } else {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        }
                    }
                }

                // Tag User
                $tagUser = dispatch(new TagUserJob($recommendation, $tagQuestion, $sessionID));

                return $this->helper->returnData(200, "success", json_decode($recommendation));

            } else {
                $returnData = [
                    'error' => 'Empty Recommendation',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

        }
        Log::channel('quizControllerLog')->info("----------------------------------");
    }

    /**
     * Update retake quiz
     */
    public function updatedRetakeQuiz(Request $request) {

        $page = $request->input('page');
        $sessionID = $request->input('sessionID');
        $question = $request->input('question');
        $questionID = $request->input('questionID');
        $questionNo = $request->input('questionNo');
        $responses = $request->input('responses');
        $responseIds = $request->input('responseIDs');
        $responseKeys = $request->input('responseKeys');
        $email = $request->input('email');
        $removeQuestions = $request->input('removeQuestions');

        $retakeID = $request->input('retakeID');

        $questionTextArray = array('31', '48');

        Log::channel('quizControllerLog')->info("-----------updatedRetakeQuiz----------");

        if (empty($sessionID)) {

            // error
            $returnData = [
                'error' => 'Invalid Session',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if ($page == 'close') {
            //$removeQuestions = ['mood','energy','stress','mood-fair'];

            $removeQuestionNo = $this->getQuestionNumber($removeQuestions);

            $deleteQuizDetails = DB::table('user_responses') // To delete the question while changing the answers
                ->where('session_id', $sessionID)
                ->whereIn('qn_no', $removeQuestionNo)
                ->delete();

            $quizData = $this->getQuizData($sessionID);

            $quizResponse = $quizData['SurveyAnswers']['Questions'];
            $q28 = isset($quizResponse['28']) ? $quizResponse['28'] : '';
            $q29 = isset($quizResponse['29']) ? $quizResponse['29'] : '';

            // To remove the q30 question if q28 and q29 is null
            if (!empty($q28) && !empty($q29)) {
                $q28Answer = $q28['AnswerID'][0];
                $q29Answer = $q29['AnswerID'][0];

                if ($q28Answer == '388' && $q29Answer == '392') { // q28 & q29 is NONE

                    // delete 30 question
                    $deleteQuizDetails = DB::table('user_responses')
                        ->where([
                            ['session_id', '=', $sessionID],
                            ['qn_no', '=', '30'],
                        ])->delete();
                }
            }

            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            $other = DB::table('question_responses')->select('id')->where([
                ['qn_id', '=', '12'],
                ['response', '=', 'Other'],
            ])->value('id');

            $userResponse = DB::select('
                select
                    qm.id,
                    qm.question,
                    qm.question_no,
                    qr.response,
                    qr.response_key,
                    ur.response_text
                from
                    user_responses as ur
                join
                    question_master as qm
                on
                    qm.id = ur.qn_id
                left join
                    question_responses as qr
                on
                    ur.response_id = qr.id
                where
                    session_id = ?', [$sessionID]);

            $dataResult = [];
            $details = [];

            if (!empty($userResponse)) {

                foreach ($userResponse as $response) {
                    $questionId = $response->id;
                    $questionNo = $response->question_no;
                    $questionText = $response->question;
                    $answerText = $response->response;
                    $answerKey = $response->response_key;
                    $responseText = $response->response_text;

                    if ($questionNo == '11' && $responseText != null) {
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                        $details[$questionNo]['AnswerText'][] = $other;
                        continue;
                    }

                    if (!isset($details[$questionNo])) {
                        $details[$questionNo] = [];
                        $details[$questionNo]['QuestionNo'] = $questionNo;
                    }

                    if (in_array($questionNo, $questionTextArray)) {
                        $details[$questionNo]['AnswerText'][] = $answerText;
                    } else {
                        $details[$questionNo]['AnswerText'][] = empty($answerKey) ? $responseText : strval($answerKey);
                    }
                }
                $questionDetails = array_values($details);
                $dataResult['SurveyAnswers']['SessionId'] = $sessionID;
                $dataResult['SurveyAnswers']['Questions'] = $questionDetails;
            }

            $returnData = $dataResult;
            $jsonData = json_encode($returnData);
            Log::channel('quizControllerLog')->info($jsonData);
            // CALL to Recommendation API
            $recommendation = $this->callRecommendation($returnData);
            Log::channel('quizControllerLog')->info($recommendation);
            $recommendationObj = json_decode($recommendation);
            if (isset($recommendationObj->Result)) {
                $recommendationObj = $recommendationObj->Result;
            }

            if (isset($recommendationObj->Recommendation) && !empty($recommendationObj->Recommendation)) {

                $lastInsertedId = DB::table('recommendations')->insertGetId(
                    [
                        'session_id' => $sessionID,
                        'email' => isset($recommendationObj->Email) ? $recommendationObj->Email : '',
                        'first_name' => isset($recommendationObj->Firstname) ? $recommendationObj->Firstname : '',
                        'sex_profile' => $recommendationObj->SexProfile,
                        'sex' => $recommendationObj->Sex,
                        'age' => $recommendationObj->Age,
                        'zipcode' => $recommendationObj->ZipCode,
                        'latitude' => $recommendationObj->Latitude,
                        'longitude' => $recommendationObj->Longitude,
                        'city' => $recommendationObj->City,
                        'state' => $recommendationObj->State,
                        'allergens' => json_encode($recommendationObj->Allergens),
                        'deficiencies' => json_encode($recommendationObj->Deficiencies),
                        'recommendation' => json_encode($recommendationObj->Recommendation),
                        'buzz_words' => json_encode($recommendationObj->BuzzWordsTips),
                        'north37' => $recommendationObj->North37,
                        'prostate_interest' => $recommendationObj->ProstateInterest,
                        'prenatal_interest' => $recommendationObj->PrenatalInterest,
                        'postnatal_interest' => $recommendationObj->PostnatalInterest,
                        'healthy_interest' => $recommendationObj->HealthyMentrualCycleInterest,
                        'perimenopause_interest' => $recommendationObj->PerimenopauseInterest,
                        'menopause_interest' => $recommendationObj->MenopauseInterest,
                        'planning_pregnancy' => $recommendationObj->PlanningPregnancy,
                        'pregnant' => $recommendationObj->Pregnant,
                        'breastfeeding' => $recommendationObj->Breastfeeding,
                        'taking_medication' => $recommendationObj->TakingMedication,
                        'blood_clotting_issue' => $recommendationObj->BloodClottingIssue,

                    ]
                );

                if (empty($lastInsertedId)) {
                    $returnData = [
                        'error' => 'Insertion Failed',
                    ];

                    return $this->helper->returnData(600, "error", $returnData);
                }

                // For the tagging purpose
                $questionResponse = DB::select('
                    select
                        qm.id,
                        qm.question,
                        qr.response
                    from
                        user_responses as ur
                    join
                        question_master as qm
                    on
                        qm.id = ur.qn_id
                    left join
                        question_responses as qr
                    on
                        ur.response_id = qr.id
                    where
                        session_id = ? and ur.qn_id in (14,32)', [$sessionID]);

                $tagQuestion = array();
                if (!empty($questionResponse)) {
                    foreach ($questionResponse as $qResponse) {
                        if (!isset($tagQuestion[$qResponse->id])) {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        } else {
                            $tagQuestion[$qResponse->id][] = $qResponse->response;
                        }
                    }
                }

                // Tag User
                $tagUser = dispatch(new TagUserJob($recommendation, $tagQuestion, $sessionID));

                return $this->helper->returnData(200, "success", json_decode($recommendation));

            } else {
                $returnData = [
                    'error' => 'Empty Recommendation',
                ];

                return $this->helper->returnData(600, "error", $returnData);
            }

        } else {
            if ($retakeID) {
                $recordDetails = $this->getQuizData($sessionID);

                if (empty($recordDetails)) {
                    if (!empty($email)) {
                        $retakeUserID = DB::table('users')->select('id')->where([
                            'user_email' => $email,
                        ])->value('id');

                        if (empty($retakeUserID)) {
                            $returnData = [
                                'error' => 'Could not find existing User details',
                            ];

                            return $this->helper->returnData(600, "error", $returnData);
                        }

                        $quizId = DB::table('quiz_user')->insertGetId([
                            'user_id' => $retakeUserID,
                            'session_id' => $sessionID,
                            'deleted' => 0,
                            'status' => 0,
                        ]);

                    }

                    $recordDetails = $this->getQuizData($retakeID);
                    if (empty($recordDetails)) {
                        $returnData = [
                            'error' => 'Could not find existing Quiz details',
                        ];

                        return $this->helper->returnData(600, "error", $returnData);
                    }

                    // $retakeQuestionData = $recordDetails['SurveyAnswers']['Questions'][$questionID];
                    $retakeQuestionData = $recordDetails['SurveyAnswers']['Questions'][$questionNo];
                    $retakeResponseData = $recordDetails['SurveyAnswers']['Questions'];

                    foreach ($retakeResponseData as $key => $value) {

                        $answerIds = null;
                        if (isset($value['AnswerID'])) {
                            $answerIds = $value['AnswerID'];
                        }

                        $answerText = $value['AnswerText'];
                        $questNo = $value['QuestionNo'];
                        $questId = $value['QuestionId'];

                        if ($key != $questionNo) {
                            if (!empty($answerIds)) {
                                $lastInsertedId = null;
                                foreach ($answerIds as $answerId) {
                                    $lastInsertedId = DB::table('user_responses')->insertGetId(
                                        [
                                            'session_id' => $sessionID,
                                            'qn_id' => $questId,
                                            'qn_no' => $questNo,
                                            'response_id' => $answerId,
                                            // 'response_text' => null,
                                            'deleted' => 0,
                                            'status' => 0,
                                        ]
                                    );
                                }

                                if (empty($lastInsertedId)) {
                                    $returnData = [
                                        'error' => 'Insertion Failed',
                                    ];

                                    return $this->helper->returnData(600, "error", $returnData);
                                }

                                $returnData = [
                                    'stored' => true,
                                    'message' => 'success',
                                ];

                            } else {
                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questId,
                                        'qn_no' => $questNo,
                                        'response_id' => null,
                                        'response_text' => $answerText[0],
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );

                                if (empty($lastInsertedId)) {
                                    $returnData = [
                                        'error' => 'Insertion Failed',
                                    ];

                                    return $this->helper->returnData(600, "error", $returnData);
                                }

                                $returnData = [
                                    'stored' => true,
                                    'message' => 'success',
                                ];

                            }
                        }

                        if ($questionNo == $key) {

                            if (!empty($responseIds)) {
                                $lastInsertedId = null;
                                foreach ($responseIds as $responseId) {
                                    $lastInsertedId = DB::table('user_responses')->insertGetId(
                                        [
                                            'session_id' => $sessionID,
                                            'qn_id' => $questionID,
                                            'qn_no' => $questionNo,
                                            'response_id' => $responseId,
                                            // 'response_text' => null,
                                            'deleted' => 0,
                                            'status' => 0,
                                        ]
                                    );
                                }

                                if (empty($lastInsertedId)) {
                                    $returnData = [
                                        'error' => 'Insertion Failed',
                                    ];

                                    return $this->helper->returnData(600, "error", $returnData);
                                }

                                $returnData = [
                                    'stored' => true,
                                    'message' => 'success',
                                ];

                            } else {
                                $lastInsertedId = DB::table('user_responses')->insertGetId(
                                    [
                                        'session_id' => $sessionID,
                                        'qn_id' => $questionID,
                                        'qn_no' => $questionNo,
                                        'response_id' => null,
                                        'response_text' => $responses[0],
                                        'deleted' => 0,
                                        'status' => 0,
                                    ]
                                );

                                if (empty($lastInsertedId)) {
                                    $returnData = [
                                        'error' => 'Insertion Failed',
                                    ];

                                    return $this->helper->returnData(600, "error", $returnData);
                                }

                                $returnData = [
                                    'stored' => true,
                                    'message' => 'success',
                                ];

                            }

                        }
                    }

                    return $this->helper->returnData(200, "success", $returnData);

                } else {

                    $retakeResponseData = $recordDetails['SurveyAnswers']['Questions'];
                    $sessionID = $recordDetails['SessionId'];

                    $deleteQuizDetails = DB::table('user_responses')
                        ->where([
                            ['session_id', '=', $sessionID],
                            ['qn_no', '=', $questionNo],
                        ])->delete();

                    if (!empty($responseIds)) {
                        $lastInsertedId = null;
                        foreach ($responseIds as $responseId) {
                            $lastInsertedId = DB::table('user_responses')->insertGetId(
                                [
                                    'session_id' => $sessionID,
                                    'qn_id' => $questionID,
                                    'qn_no' => $questionNo,
                                    'response_id' => $responseId,
                                    // 'response_text' => null,
                                    'deleted' => 0,
                                    'status' => 0,
                                ]
                            );
                        }

                        if (empty($lastInsertedId)) {
                            $returnData = [
                                'error' => 'Insertion Failed',
                            ];

                            return $this->helper->returnData(600, "error", $returnData);
                        }

                        $returnData = [
                            'stored' => true,
                            'message' => 'success',
                        ];

                        return $this->helper->returnData(200, "success", $returnData);

                    } else {
                        $lastInsertedId = DB::table('user_responses')->insertGetId(
                            [
                                'session_id' => $sessionID,
                                'qn_id' => $questionID,
                                'qn_no' => $questionNo,
                                'response_id' => null,
                                'response_text' => $responses[0],
                                'deleted' => 0,
                                'status' => 0,
                            ]
                        );

                        if (empty($lastInsertedId)) {
                            $returnData = [
                                'error' => 'Insertion Failed',
                            ];

                            return $this->helper->returnData(600, "error", $returnData);
                        }

                        $returnData = [
                            'stored' => true,
                            'message' => 'success',
                        ];

                        return $this->helper->returnData(200, "success", $returnData);

                    }
                }
            }
        }

        Log::channel('quizControllerLog')->info("------------------------------------");
    }

    /**
     * Get quiz data
     */
    public function getQuizData($sessionID) {

        if (empty($sessionID)) {
            return null;
        }

        $dataResult = array();

        $results = DB::select('
            select
                qm.id,
                qm.question_no,
                qm.question,
                IFNULL(qr.response,ur.response_text) as response,
                qr.id as response_id,
                qm.page
            from
                user_responses as ur
            join
                question_master as qm
            on
                qm.id = ur.qn_id
            left join
                question_responses as qr
            on
                ur.response_id = qr.id
            where
                session_id = ?', [$sessionID]);

        if (empty($results)) {
            return null;
        }

        foreach ($results as $result) {

            $questionId = $result->id;
            $questionNo = $result->question_no;
            $questionText = $result->question;
            $answerText = $result->response;
            $questionHandle = $result->page;
            $answerID = $result->response_id;

            $temp = [];

            if (!isset($details[$questionNo])) {

                $details[$questionNo] = [];
                $details[$questionNo]['QuestionNo'] = $questionNo;
                $details[$questionNo]['QuestionId'] = $questionId;
                $details[$questionNo]['QuestionText'] = $questionText;
                $details[$questionNo]['QuestionHandle'] = $questionHandle;
            }

            $details[$questionNo]['AnswerText'][] = $answerText;

            if (!empty($answerID)) {
                $details[$questionNo]['AnswerID'][] = $answerID;
            }

        }

        $questionDetails['Questions'] = $details;
        $dataResult['SessionId'] = $sessionID;
        $dataResult['SurveyAnswers'] = $questionDetails;

        return $dataResult;
    }

    /**
     * Get question answer
     */
    public function getQuestionNumber($handles) {

        $results = DB::table('question_master')
            ->select('question_no as qNo', 'page as page')
            ->whereIn('page', $handles)
            ->get();

        $questionNo = [];
        foreach ($results as $result) {

            $page = $result->page;
            $qNo = $result->qNo;

            $questionNo[$page] = $qNo;
        }
        return $questionNo;
    }

    /**
     * Create iterable event
     */
    public function createEvent($data) {

        $path = $this->iterableUrl . 'events/track';
        $method = 'POST';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Trigger email via iterable
     */
    public function triggerEmailTarget($data) {

        $path = $this->iterableUrl . 'email/target';
        $method = 'POST';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }


    /**
     * Get customer activation status
     */
    public function getActivationStatus($email) {

        Log::channel('quizControllerLog')->info("---------getActivationStatus-----------");

        $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('quizControllerLog')->info("getActivationStatus: Failed to get the customer details from Shopify(emailAddress:".$email."), 
            Error: (".$error.")");
        }

        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            Log::channel('quizControllerLog')->info("getActivationStatus: No such customer in Shopify(Email:".$email.")");
            return null;
        }

        $customer = [];
        foreach ($customerDetails['customers'] as $cust) {
            if (trim(strtolower($cust['email'])) == strtolower($email)) {
                $customer = $cust;
                break;
            }
        }

        if (empty($customer)) {
            Log::channel('quizControllerLog')->info("getActivationStatus: Not found the searched email address in Shopify(Email:".$email.")");
            return null;
        }

        $status = !empty($customer['state']) ? trim($customer['state']) : "";

        return $status;

        Log::channel('quizControllerLog')->info("---------------------------------------");
    }

    /**
     * CURL call
     */
    public function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "content-type: application/json",
            "api_key: " . $this->iterableApiKey,
        ));

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        $response = curl_exec($curl);
        return $response;
    }

    /**
     * Send password recovery email
     */
    function sendPasswordRecoveryEmail($customerEmail) {

        $storeFrontToken   = Config::get('shopify.storeFrontTokenQuiz');
        $shopDomain = Config::get('shopify.shopDomainQuiz');

        $path = 'https://'.$shopDomain.'/api/2020-04/graphql.json';
        $method = 'POST';

        $data = "{\"query\":\"mutation sendPasswordRecoverEmail {\\n  customerRecover(email: \\\"$customerEmail\\\") {\\n    customerUserErrors { code field message }\\n  }\\n}\",\"variables\":null,\"operationName\":\"sendPasswordRecoverEmail\"}";

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "content-type: application/json",
            "x-shopify-storefront-access-token: ".$storeFrontToken
        ));

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        $response = curl_exec($curl);
        return $response;
    }

     /**
     * Function to send custom password reset mail
     */
    public function sendCustomPasswordResetEmail(Request $request) {

        $email = !empty($request->input('email')) ? trim($request->input('email')) : "";

        Log::channel('quizControllerLog')->info("---------sendCustomPasswordResetEmail---------");
        if (empty($email)) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];

            Log::channel('quizControllerLog')->info('Invalid endpoint request or the email address is empty in the request');
            return $this->helper->returnData(600, "error", $returnData );
        }

        // $api = new ExtentedShopifyAPI();

        // $password   = Config::get('shopify.passwordQuiz');
        // $shopDomain = Config::get('shopify.shopDomainQuiz');

        // $api->setShop($shopDomain);
        // $api->setAccessToken($password);


        $customerDetails = $this->customerObj->getCustomerDetailBySearch($email);

        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

            $error = "Unknown error";
            if (!empty($customerDetails['error'])) {

                $error = json_encode($customerDetails['error']);
            } elseif (!empty($customerDetails['errors'])) {
                
                $error = json_encode($customerDetails['errors']);
            }
            if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $email . ")";
            }
            $returnData = [
                "error" => $error
            ];

            Log::channel('quizControllerLog')->info("sendCustomPasswordResetEmail: Failed to get the customer details from Shopify
            (emailAddress:".$email."), Error: (".$error.")");
        }

        if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {
            $returnData = [
                'error' => 'email not found',
            ];
            Log::channel('quizControllerLog')->info("sendCustomPasswordResetEmail: Not found any email address(Email:".$email.")");
            return $this->helper->returnData(600, "error", $returnData);
        }

        $requestedEmailShopifyStatus = 0;
        $customerStatus = null;
        foreach ($customerDetails['customers'] as $customer) {
            if (trim(strtolower($cust['email'])) == trim(strtolower($email))) {
                $requestedEmailShopifyStatus = 1;
                $customerStatus = $customer['state'];
                break;
            }
        }

        if ($requestedEmailShopifyStatus == 0) {

            $returnData = [
                'error' => 'email address not found in the Shopify search result ('.$email.')'
            ];

            Log::channel('quizControllerLog')->info('email address not found in the Shopify search result ('.$email.')');
            return $this->helper->returnData(600, "error", $returnData );
        }

        if (strtolower(trim($customerStatus)) != 'enabled') {

            $returnData = [
                'error' => 'Shopify account is not enabled ('.$email.')'
            ];

            Log::channel('quizControllerLog')->info('Shopify account is not enabled ('.$email.')');
            return $this->helper->returnData(600, "error", $returnData );

        } else {

            $databaseStatus = $this->dbObj->checkEmailStatusInDb($email);

            if (!isset($databaseStatus) || empty($databaseStatus)) {

                $returnData = [
                    "message" => "email address not found in the database (".$email.")"
                ];

                Log::channel('quizControllerLog')->info("email address not found in the database (".$email.")");
                return $this->helper->returnData(600, "error", $returnData );
            }
            $registrationAuth = null;
            if (!isset($databaseStatus[0]->registration_auth) || empty($databaseStatus[0]->registration_auth)) {

                $generatedRegistrationAuth = $this->generateRandomPasswordToken(32);
                $authUpdateStatus = $this->dbObj->updateAuthTokenInDB($email, $generatedRegistrationAuth);
                if ($authUpdateStatus != 1) {

                    $returnData = [
                        "message" => "failed to update the generated auth token in the database (".$email.")"
                    ];

                    Log::channel('quizControllerLog')->info("failed to update the generated auth token in database (".$email.")");
                    return $this->helper->returnData(600, "error", $returnData );
                }
                $registrationAuth = $generatedRegistrationAuth;
            } else {

                $registrationAuth = $databaseStatus[0]->registration_auth;
            }

            $passwordResetURL = env('MIDDLEWARE_URL',true).'/pages/password-reset?registration_auth='.$registrationAuth.'&email='.$email.'';

            $iterableMailRequest = array(

                "campaignId" => intval(env('ITERABLE_CUSTOMER_PASSWORD_RESET_CAMPAIGN_ID',true)),
                "recipientEmail" => $email,
                "dataFields" => array(
                    "customUrl" => $passwordResetURL,
                )
            );

            $mailTriggerResponse = $this->iterableObj->sendPasswordRestMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {

                $returnData = [
                    "mailStatus" => true,
                    "message" => "custom password reset mail triggered successfully (".$email.")"
                ];

                Log::channel('quizControllerLog')->info("custom password reset mail triggered successfully (".$email.")");
                return $this->helper->returnData(200, "success", $returnData );

            } else {

                $error = $mailTriggerResponse->code;

                if (empty($error)) {

                    $returnData = [
                        'error' => 'failed to send the custom password reset mail ('.$email.')'
                    ];
                } else {

                    $returnData = [
                        'error' => $error.' ('.$email.')'
                    ];
                }
                Log::channel('quizControllerLog')->info('failed to send the custom password reset mail ('.$email.')');
                return $this->helper->returnData(600, "error", $returnData );
            }
        }

        Log::channel('quizControllerLog')->info("----------------------------------------");
    }
}

class ArrayValue implements JsonSerializable
{
    public function __construct(array $array)
    {
        $this->array = $array;
    }

    public function jsonSerialize()
    {
        return $this->array;
    }
}
