<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Models\IterableApi;
use App\Models\Shopify\Order;
use App\Models\Shopify\Product;
use Carbon\Carbon;
use Log;

class ShipmentDeliveryTaggingCron extends Controller
{

    public $iterableObj;
    public $newAllTags;
    public $newOrderFlag;
    public $productObj;
    public $daysUntilFulfillment;
    public $productVariantDetails;

    public $daysUntilFulfillmentThreeList;

    public function __construct() {

        $this->setPaths();
        $this->orderObj = new Order('shipmentDeliveryTagCron');
        $this->productObj = new Product('shipmentDeliveryTagCron');
        $this->iterableObj = new IterableApi();
        $this->helperObj = new Helper();
        $this->daysUntilFulfillmentThreeList = [];
        $this->productVariantDetails = [];
        date_default_timezone_set('America/Los_Angeles');
    }

    public function setPaths() {

        $this->csvDirectory = dirname(__FILE__) . "/../../../public/";
    }

    public function getExcludedHolidays() {

        $excludedDates = [];
        $csvDetails = $this->helperObj->getReadCsvDetails($this->csvDirectory . 'usps-holidays.csv');
        if (!empty($csvDetails)) {
            foreach ($csvDetails as $date) {
                $excludedDates[] = $date['usps_holidays'];
            }
        }
        return $excludedDates;
    }

    /**
     * Process fulfilment order tagging
     */
    public function processTagging() {

        $this->getShopifyProducts();

        $excludedDates = $this->getExcludedHolidays();

        $cronStartedDateTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        Log::channel('shipmentDeliveryTagging')->info("Cron Started-------" . $cronStartedDateTime);

        $page = 1;
        $orderNumber = 1;
        $limit = 250;
        $orderCount = 0;
        // Get latest 60 days orders
        $orderSearchDate = new Carbon(date('Y-m-d'), 'America/Los_Angeles');
        $orderSearchDate = $orderSearchDate->subDays(60)->toDateString();
        $responseDetails = '';

        do {
            if (isset($responseDetails['link']) && !empty($responseDetails['link'])) {

                $nextPage = $responseDetails['link'];
            } else {
                
                $nextPage = '';
            }
            sleep(1);
            $responseDetails = $this->orderObj->getShopifyOrders($limit, $nextPage, $orderSearchDate);
            //$responseDetails = $this->orderObj->getShopifyOrders($limit, $nextPage);
            echo "Page - " . $page . PHP_EOL;
            if (isset($responseDetails['body']) && !empty($responseDetails['body'])) {
                $orderCount = count($responseDetails['body']);

                $orders = $responseDetails['body'];
                foreach ($orders as $order) {

                    echo $orderNumber . " ) Order - " . $order['id'] . PHP_EOL;

                    $orderId = $order['id'];
                    $this->newAllTags = [];
                    $this->newOrderFlag = 0;
                    $fulfillmentDetails = '';
                    $fulfillmentDetails = $order['fulfillments'];
                    $orderCancelledAt = !empty($order['cancelled_at']) ? trim($order['cancelled_at']) : '';
                    $orderRefundStatus = !empty($order['financial_status']) ? trim($order['financial_status']) : '';

                    $orderTags = $order['tags'];
                    Log::channel('shipmentDeliveryTagging')->info('Existing Tags - ' . $orderId . ' - ' . $order['tags']);
                    $currentOrderTags = explode(',', $orderTags);
                    $currentOrderTags = array_map('trim', $currentOrderTags);
                    $this->newAllTags = $currentOrderTags;

                    //If the order is cancelled, then no need for further tagging
                    if (!empty($orderCancelledAt)) {
                        echo 'SKIPPED- cancelled' . PHP_EOL;
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order is cancelled - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }

                    //If the order is refunded, then no need for further tagging
                    if ($orderRefundStatus == 'refunded') {
                        echo 'SKIPPED- refunded' . PHP_EOL;
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order is refunded - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }

                    //If the order is already delivered and tagged then no need for further tagging.
                    if (in_array('delivered', $currentOrderTags)) {
                        echo 'SKIPPED- delivered' . PHP_EOL;
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order already delivered - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }

                    if (in_array('exception:undelivered', $currentOrderTags)) {
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order shipping-status attempted_delivery(undelivered) - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }

                    if (in_array('exception:returned', $currentOrderTags)) {
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order shipping-status failure(returned) - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }
                    if (in_array('VIP', $currentOrderTags)) {
                        echo 'SKIPPED- VIP' . PHP_EOL;
                        Log::channel('shipmentDeliveryTagging')->info("SKIPPED - Order VIP tagged order - " . $orderId . ' Tags - ' . implode(',', $currentOrderTags));
                        continue;
                    }
                    //create deliveryDays:X tags
                    $this->createDeliveryDaysTag($order, $currentOrderTags);

                    //create daysUntilFulfillment:X tags
                    $this->createFulfillmentTag($order, $currentOrderTags);

                    $this->createBusinessFulfillmentTag($order, $currentOrderTags, $excludedDates);

                    //create transitDays:X tags
                    if (!empty($fulfillmentDetails)) {
                        Log::channel('shipmentDeliveryTagging')->info('----------------*' . $fulfillmentDetails[0]['shipment_status']);
                        $this->createTransitDaysTag($order, $currentOrderTags);
                        $this->createReturnAndUndeliveredTags($order, $currentOrderTags);

                    }

                    if (!empty($this->newAllTags) && $this->newOrderFlag > 0) {
                        if (in_array('businessDaysUntilFulfillment:4', $this->newAllTags) &&
                            !in_array('businessDaysUntilFulfillment:4', $currentOrderTags)) {

                            $this->newAllTags[] = 'delayCouponApplied';
                            $daysUntilThreeOrder = [];
                            $daysUntilThreeOrder['id'] = $orderId;
                            $daysUntilThreeOrder['order'] = $order['name'];
                            $daysUntilThreeOrder['email'] = $order['email'];
                            $daysUntilThreeOrder['first_name'] = $order['shipping_address']['first_name'];
                            $daysUntilThreeOrder['last_name'] = $order['shipping_address']['last_name'];

                            $orderCreatedAtDate = new Carbon($order['created_at'], 'America/Los_Angeles');
                            $orderCreatedAtDate = $orderCreatedAtDate->toDayDateTimeString();
                            $daysUntilThreeOrder['created_at'] = $orderCreatedAtDate;

                            $this->daysUntilFulfillmentThreeList[] = $daysUntilThreeOrder;
                        }

                        try {
                            Log::channel('shipmentDeliveryTagging')->info('Updating order - ' . $orderId);
                            $request = [];
                            $request['order']['id'] = $orderId;
                            $request['order']['tags'] = implode(',', $this->newAllTags);
                            $updateResponse = $this->orderObj->updateOrder($request, $orderId);

                            $retry = false;
                            $try = 1;
                            do {
                                $updateResponse = $this->orderObj->updateOrder($request, $orderId);
                                if (!isset($updateResponse['body']['order'])) {
                                    sleep(1);
                                    $retry = true;
                                    $try++;
                                }
                            } while($retry && $try <=3);

                            if (isset($updateResponse['errors'])) {
                                Log::channel('shipmentDeliveryTagging')->info("Error on updating the order - " . $orderId . ' Error - ' . json_encode($updateResponse['errors']));
                            } elseif (isset($updateResponse['order']) && !empty($updateResponse['order'])) {
                                Log::channel('shipmentDeliveryTagging')->info("Successfully updated the order - " . $orderId . ' Tags - ' . implode(',', $this->newAllTags));

                                if ($this->daysUntilFulfillment >= 2 && !in_array('delivered', $this->newAllTags)) {
                                    $this->createDaysUntilFulfillmentEvent($order);
                                }

                            } else {
                                Log::channel('shipmentDeliveryTagging')->info("Failed to update the order - " . $orderId . ' Error - ' . json_encode($updateResponse['errors']));

                            }

                        } catch (Exception $e) {
                            Log::channel('shipmentDeliveryTagging')->info("Error on updating the order - " . $orderId . ' - ' . $e->getMessage());
                        }

                    }

                    $orderNumber++;
                }
            }
            $page++;

        } while ($orderCount == $limit);

        //For sending the report email
        $this->sendReportForDaysUntilFulfillment();
    }

    /**
     * Send fulfilment tagging report
     */
    public function sendReportForDaysUntilFulfillment() {

        $commonContent = '<p>Please see the orders tagged during this execution with businessDaysUntilFulfillment:4.
         This means they have exceeded 96 business hours of being unfulfilled since the order was placed.</p>';

        $tableContent = '';
        if (!empty($this->daysUntilFulfillmentThreeList)) {
            $tableContent .= "
            <table>
                <tr>
                    <td>#</td>
                    <td>Order</td>
                    <td>Customer</td>
                    <td>Email</td>
                    <td>Date</td>
                </tr>
            ";

            $i = 1;
            foreach ($this->daysUntilFulfillmentThreeList as $record) {
                $tableContent .= "
                <tr>
                    <td>" . $i . "</td>
                    <td><a href=".env("SHOPIFY_ORDER_ADMIN_URL", true) . $record['id'] . "'>" . $record['order'] . "</a></td>
                    <td>" . $record['first_name'] . " " . $record['last_name'] . "</td>
                    <td>" . $record['email'] . "</td>
                    <td>" . $record['created_at'] . "</td>
                </tr>
                ";
                $i++;
            }
            $tableContent .= '</table>';
        } else {
            $commonContent .= '<p><b>Congrats - All Orders have been fulfilled within 96 business hours.</b></p>';
        }

        $reportSendingTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $subject = "Nurish-Dev Fulfillment Report - " . $reportSendingTime;

        $recipientEmails = 'testing@p80w.com';
        //$recipientEmails = 'mikegordon79@gmail.com';

        $iterableMailRequest = array(
            "campaignId" => env("SHIPMENT_TAGGING_CRON_CAMPAIGN_ID", true),
            "recipientEmail" => $recipientEmails,
            "dataFields" => array(
                "recordDetails" => $tableContent,
                "subject" => $subject,
                "commonContent" => $commonContent,
            ),
        );

        $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

        if (strtolower(trim($mailTriggerResponse->code)) == 'success') {
            Log::channel('shipmentDeliveryTagging')->info("Successfully send the mail (Email:".$recipientEmails.")");

        } else {
            Log::channel('shipmentDeliveryTagging')->info("Failed to send the mail (Email:".$recipientEmails.")");
            Log::channel('shipmentDeliveryTagging')->info("Response: ".json_encode($mailTriggerResponse));
        }
    }

    /**
     * Create return and undelivered tags
     */
    public function createReturnAndUndeliveredTags($orderDetails, $currentTags) {

        $fulfillmentDetails = $orderDetails['fulfillments'];

        if (!empty($fulfillmentDetails)) {
            $shipmentStatus = $fulfillmentDetails[0]['shipment_status'];

            $fulfillmentUpdatedDate = $fulfillmentDetails[0]['updated_at'];
            $fulfillmentUpdatedDate = new Carbon($fulfillmentUpdatedDate, 'America/Los_Angeles');

            $orderTags = $orderDetails['tags'];

            if ($shipmentStatus == 'attempted_delivery' || $shipmentStatus == 'failure') {

                if (strpos($orderTags, 'exceptionDT') !== false) {
                    //If tags exist, then modify the tagging
                    if (!empty($currentTags)) {
                        foreach ($currentTags as $tag) {
                            if (strpos($tag, 'exceptionDT') !== false) {
                                $key = array_search($tag, $this->newAllTags);
                                unset($this->newAllTags[$key]);
                                $this->newAllTags = array_values($this->newAllTags);
                            }
                        }
                        $this->newAllTags[] = 'exceptionDT:' . $fulfillmentUpdatedDate;

                    }
                } else {
                    if ($shipmentStatus == 'attempted_delivery') {

                        Log::channel('shipmentDeliveryTagging')->info("Order shipment Status - " . $orderDetails['id'] .
                            ' - ' . $shipmentStatus);

                        $this->newAllTags[] = 'exception:undelivered';
                        $this->newAllTags[] = 'exceptionDT:' . $fulfillmentUpdatedDate;

                    }

                    if ($shipmentStatus == 'failure') {
                        Log::channel('shipmentDeliveryTagging')->info("Order shipment Status - " . $orderDetails['id'] .
                            ' - ' . $shipmentStatus);
                        $this->newAllTags[] = 'exception:returned';
                        $this->newAllTags[] = 'exceptionDT:' . $fulfillmentUpdatedDate;
                    }
                }

            }

        }

    }

    /**
     * Create delivery day tags
     */
    public function createDeliveryDaysTag($orderDetails, $currentTags) {

        $orderCreatedAt = new Carbon($orderDetails['created_at'], 'America/Los_Angeles');
        //$orderCreatedAt = $orderCreatedAt->tz('utc');

        $currentDateTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        //$currentDateTime = $currentDateTime->tz('utc');

        $fulfillmentDetails = $orderDetails['fulfillments'];

        if (!empty($fulfillmentDetails)) {
            $shipmentStatus = $fulfillmentDetails[0]['shipment_status'];
            if ($shipmentStatus == 'delivered') {
                $currentDateTime = $orderDetails['updated_at'];
                $currentDateTime = new Carbon($currentDateTime, 'America/Los_Angeles');
                // $currentDateTime = $currentDateTime->tz('utc');
                $this->newAllTags[] = 'delivered';
                $this->newAllTags[] = 'deliveryDateTime:' . $currentDateTime;
            }
            if ($shipmentStatus == 'attempted_delivery' || $shipmentStatus == 'failure') {
                $currentDateTime = $fulfillmentDetails[0]['updated_at'];
                $currentDateTime = new Carbon($currentDateTime, 'America/Los_Angeles');
            }
        }

        $days = $orderCreatedAt->diffInDays($currentDateTime);
        $orderTags = $orderDetails['tags'];

        if ($days > 0) {
            $this->newOrderFlag++;

            if (strpos($orderTags, 'deliveryDays') !== false) {
                //If tags exist, then modify the tagging

                if (!empty($currentTags)) {
                    foreach ($currentTags as $tag) {
                        if (strpos($tag, 'deliveryDays') !== false) {
                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }
                    $this->newAllTags[] = 'deliveryDays:' . $days;

                }
            } else {
                $this->newAllTags[] = 'deliveryDays:' . $days;
            }

        }

    }

    public function createFulfillmentTag($orderDetails, $currentTags) {

        $orderCreatedAt = new Carbon($orderDetails['created_at'], 'America/Los_Angeles');
        // $orderCreatedAt = $orderCreatedAt->tz('utc');

        $currentDateTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        // $currentDateTime = $currentDateTime->tz('utc');

        $fulfillmentDetails = $orderDetails['fulfillments'];

        if (!empty($fulfillmentDetails)) {
            $currentDateTime = $fulfillmentDetails[0]['created_at'];
            $currentDateTime = new Carbon($currentDateTime, 'America/Los_Angeles');
            // $currentDateTime = $currentDateTime->tz('utc');
        }

        $this->daysUntilFulfillment = 0;
        $days = $orderCreatedAt->diffInDays($currentDateTime);
        $orderTags = $orderDetails['tags'];
        if ($days > 0) {
            $this->newOrderFlag++;
            $this->daysUntilFulfillment = $days;
            $key = '';

            if (strpos($orderTags, 'daysUntilFulfillment') !== false) {
                //If tags exist, then modify the tagging
                if (!empty($currentTags)) {
                    foreach ($currentTags as $tag) {
                        if (strpos($tag, 'daysUntilFulfillment') !== false) {
                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }
                    $this->newAllTags[] = 'daysUntilFulfillment:' . $days;

                }
            } else {

                $this->newAllTags[] = 'daysUntilFulfillment:' . $days;
            }

            $key = '';

            if (strpos($orderTags, 'DT-UntilFulfillment') !== false) {
                //If tags exist, then modify the tagging
                if (!empty($currentTags)) {
                    foreach ($currentTags as $keyTag => $tag) {

                        if (strpos($tag, 'DT-UntilFulfillment') !== false) {
                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }

                    $this->newAllTags[] = 'DT-UntilFulfillment:' . $currentDateTime;
                }
            } else {

                $this->newAllTags[] = 'DT-UntilFulfillment:' . $currentDateTime;

            }

        }

    }

    /**
     * Create business fulfilment tag
     */
    public function createBusinessFulfillmentTag($orderDetails, $currentTags, $excludedDates) {

        // $cronExecutionDate =  new Carbon('2021-07-05', 'America/Los_Angeles');
        $cronExecutionDate = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        $day = $cronExecutionDate->dayName;
        $cronExecutionDate = $cronExecutionDate->format('Y-m-d');
        $weekEnds = ['saturday', 'sunday'];

        $dayOfWeek = strtolower($day);

        $orderCreatedAt = new Carbon($orderDetails['created_at'], 'America/Los_Angeles');

        // $orderCreatedAt = $orderCreatedAt->tz('utc');

        $currentDateTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        // $currentDateTime = $currentDateTime->tz('utc');

        $fulfillmentDetails = $orderDetails['fulfillments'];

        if (!empty($fulfillmentDetails)) {
            $currentDateTime = $fulfillmentDetails[0]['created_at'];
            $currentDateTime = new Carbon($currentDateTime, 'America/Los_Angeles');
            // $currentDateTime = $currentDateTime->tz('utc');
        }

        $this->businessDaysUntilFulfillment = 0;
        $days = $orderCreatedAt->diffInDays($currentDateTime);
        $orderTags = $orderDetails['tags'];

        if ($days > 0 && !in_array($cronExecutionDate, $excludedDates) && !in_array($dayOfWeek, $weekEnds)) {
            $this->newOrderFlag++;
            $this->businessDaysUntilFulfillment = $days;
            $key = '';

            if (strpos($orderTags, 'businessDaysUntilFulfillment') !== false) {
                //If tags exist, then modify the tagging
                if (!empty($currentTags)) {
                    foreach ($currentTags as $tag) {
                        if (strpos($tag, 'businessDaysUntilFulfillment') !== false) {
                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }
                    $this->newAllTags[] = 'businessDaysUntilFulfillment:' . $days;

                }
            } else {

                $this->newAllTags[] = 'businessDaysUntilFulfillment:' . $days;

            }
        }
    }

    /**
     * Create transit tag
     */
    public function createTransitDaysTag($order, $currentTags) {

        $fulfillmentDetails = $order['fulfillments'];
        $orderTags = $order['tags'];

        $shipmentStatus = $fulfillmentDetails[0]['shipment_status'];

        $fulfillmentCreateDate = $fulfillmentDetails[0]['created_at'];
        $fulfillmentCreateDate = new Carbon($fulfillmentCreateDate, 'America/Los_Angeles');
        // $fulfillmentCreateDate = $fulfillmentCreateDate->tz('utc');

        $currentDateTime = new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        // $currentDateTime = $currentDateTime->tz('utc');

        if ($shipmentStatus == 'delivered' || $shipmentStatus == 'out_for_delivery') {
            $currentDateTime = $fulfillmentDetails[0]['updated_at'];
            $currentDateTime = new Carbon($currentDateTime, 'America/Los_Angeles');
            // $currentDateTime = $currentDateTime->tz('utc');
        }

        if ($shipmentStatus == 'delivered' && !in_array('delivered', $this->newAllTags)) {
            $this->newAllTags[] = 'delivered';
        }

        $days = $fulfillmentCreateDate->diffInDays($currentDateTime);

        if ($days > 0 && ($shipmentStatus == 'in_transit' || $shipmentStatus == 'delivered'
            || $shipmentStatus == 'out_for_delivery')) {
            $this->newOrderFlag++;
            $key = '';

            if (strpos($orderTags, 'transitDays') !== false) {
                //If tags exist, then modify the tagging
                if (!empty($currentTags)) {
                    foreach ($currentTags as $tag) {
                        if (strpos($tag, 'transitDays') !== false) {

                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }
                    $this->newAllTags[] = 'transitDays:' . $days;

                }
            } else {
                $this->newAllTags[] = 'transitDays:' . $days;
            }

            $key = '';
            if (strpos($orderTags, 'transitDateTime') !== false) {
                //If tags exist, then modify the tagging
                if (!empty($currentTags)) {
                    foreach ($currentTags as $tag) {
                        if (strpos($tag, 'transitDateTime') !== false && $shipmentStatus != 'delivered') {

                            $key = array_search($tag, $this->newAllTags);
                            unset($this->newAllTags[$key]);
                            $this->newAllTags = array_values($this->newAllTags);
                        }
                    }
                    $this->newAllTags[] = 'transitDateTime:' . $currentDateTime;

                }
            } else {
                if ($shipmentStatus != 'delivered') {
                    $this->newAllTags[] = 'transitDateTime:' . $currentDateTime;
                }

            }

        }

    }


    /**
     * Create fulfilment iterable event
     */
    public function createDaysUntilFulfillmentEvent($order) {

        if (!empty($order)) {
            $email = $order['email'];
            $eventDataDetails = [];
            $lineItemProperties = isset($order['line_items'][0]['properties']) ? $order['line_items'][0]['properties'] : '';
            $eventDataDetails['order']['email'] = $email;
            $eventDataDetails['order']['total_price'] = $order['total_price'];
            $eventDataDetails['order']['subtotal_price'] = $order['subtotal_price'];
            $eventDataDetails['order']['first_name'] = $order['shipping_address']['first_name'];

            if (!empty($lineItemProperties)) {
                $lineItemDetails = [];
                foreach ($lineItemProperties as $key => $value) {

                    if (strpos(strtolower($value['name']), 'vitamin') !== false) {

                        $explodeValue = explode('|', $value['value']);
                        $variantDetails = '';
                        $variantValue = 0;

                        // print_r($explodeValue);die('--------');

                        if (isset($explodeValue[3]) && !empty($explodeValue[3])) {

                            $variantById = trim($explodeValue[3]);
                            if (isset($this->productVariantDetails[$variantById])) {
                                $variantDetails = $this->productVariantDetails[$variantById];
                            }
                            $variantValue = isset($variantDetails['price']) ? $variantDetails['price'] : 0;
                        }

                        $lineItemDetails[] = [
                            "name" => isset($explodeValue[0]) ? trim($explodeValue[0]) : '',
                            "value" => isset($explodeValue[1]) ? trim($explodeValue[1]) : '',
                            "quantity" => isset($explodeValue[2]) ? trim($explodeValue[2]) : '',
                            "variant_id" => isset($explodeValue[3]) ? trim($explodeValue[3]) : '',
                            "sku" => isset($explodeValue[4]) ? trim($explodeValue[4]) : '',
                            "price" => trim($variantValue),
                        ];

                    }

                }
                $eventDataDetails['order']['line_items'] = $lineItemDetails;
                $eventDataDetails['order']['daysUntilFulfillment'] = $this->daysUntilFulfillment;

                $this->createEvent($email, $eventDataDetails);
            }

        }

    }

    /**
     * Create iterable event
     */
    public function createEvent($email, $data) {

        $eventName = 'daysUntilFulfillment';
        $eventRequest = '';
        if (!empty($data) && !empty($email)) {

            $eventRequest = array(
                "email" => strtolower($email),
                "eventName" => $eventName,
                "dataFields" => $data,
            );

            $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));
            if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
                Log::channel('shipmentDeliveryTagging')->info(
                    'Event created successfully : ' . $email .
                    '| EventName : ' . $eventName
                );

            } else {

                Log::channel('shipmentDeliveryTagging')->info(
                    'Failed to create event : ' . $email .
                    ' | EventName : ' . $eventName .
                    ' | EventDetails : ' . json_encode($eventRequest) .
                    ' | EventResponse : ' . json_encode($eventResponse)
                );
            }
        } else {
            Log::channel('shipmentDeliveryTagging')->info(
                'Invalid data : ' . $email .
                ' | EventName : ' . $eventName .
                ' | EventResponse : ' . json_encode($eventResponse) .
                ' | EventDetails : ' . json_encode($eventRequest)
            );
        }

    }

    /**
     * Get Shopify products
     */
    public function getShopifyProducts() {

        $limit = 250;
        do {
            if (isset($responseDetails['link']) && !empty($responseDetails['link'])) {
                $nextPage = $responseDetails['link'];
            } else {
                $nextPage = '';
            }
            $responseDetails = $this->productObj->getShopifyAllProducts($limit, $nextPage);

            if (isset($responseDetails['body']) && !empty($responseDetails['body'])) {
                $productCount = count($responseDetails['body']);
                $products = $responseDetails['body'];

                foreach ($products as $product) {

                    foreach ($product['variants'] as $variant) {
                        $this->productVariantDetails[$variant['id']]['sku'] = $variant['sku'];
                        $this->productVariantDetails[$variant['id']]['price'] = $variant['price'];
                    }

                }

            }

        } while ($productCount == $limit);

    }

}
