<?php

namespace App\Http\Controllers;

use Log;
use App\Users;
use Illuminate\Http\Request;
use App\Jobs\ShipmentTrackingUpdateJob;
use Config;

class ShipmentTrackingWebhookController extends Controller
{

    private $shopifyWebhookSecret;

    public function __construct() {

        $this->shopifyWebhookSecret = env('SHOPIFY_WEBHOOK_SECRET', true);
    }

    /**
     * Process webhook request
     */
    public function processWebhook() {
        
        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_shopify_webhook($requestData, $hmac_header);
        $webhookStatus = var_export($verified, true);
        
        if ($webhookStatus) {

            $receivedData = json_decode($requestData);
            
            Log::channel('shipmentWebhookIterable')->info("Shopify order fulfillment webhook validation success");
            
            Log::channel('shipmentWebhookIterable')->info("Request: ".$requestData);

            if (empty($receivedData->email)) {
                Log::channel('shipmentWebhookIterable')->info("Error in the iterable user profile tacking update - Missing required
                parameters in the webhook response or the response is not correct");
                
            } else {
                Log::channel('shipmentWebhookIterable')->info("Data transferred to ShipmentTrackingUpdateJob");
                $queueResponse = dispatch(new ShipmentTrackingUpdateJob($receivedData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {
            Log::channel('shipmentWebhookIterable')->info("Shopify order fulfillment webhook validation failed (Request: ".$requestData.")");
        }
    }
    
    public function verify_shopify_webhook($data, $hmac_header) {
        
        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shopifyWebhookSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }

}
