<?php

namespace App\Http\Controllers;

use Log;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use App\Models\dataPostInternal;
use App\Models\Shopify\Product;
use App\Models\Shopify\Order;
use App\Helper\Helper;
use Illuminate\Http\Request;

class ShopifyOrderProcessController extends Controller {

    private $rechargeObj;
    private $iterableObj;
    private $helperObj;
    private $productObj;
    private $orderObj;
    private $internalDataPostObj;

    public function __construct() {

        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj = new IterableApi();
        $this->productObj = new Product();
        $this->orderObj = new Order();
        $this->internalDataPostObj = new dataPostInternal();
        date_default_timezone_set('America/Los_Angeles');
    }

    /**
     * Process order refund and cancel
     */
    public function processOrderCancel(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);

        if (empty($receivedData->orderId) && empty($receivedData->token)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $shopifyOrderId = !empty($receivedData->orderId) ? trim($receivedData->orderId) : '';
        $rechargeToken = !empty($receivedData->token) ? trim($receivedData->token) : '';
        $requestedSource = !empty($receivedData->requestSource) ? trim(strtolower($receivedData->requestSource)) : '';
        $orderCreationToolStatus = !empty($receivedData->orderCreationToolStatus) ? 
        trim($receivedData->orderCreationToolStatus) : '';

        $customOrderCreationStatus = 0;
        if ($requestedSource == 'app' && $orderCreationToolStatus == 'true') {

            $customOrderCreationStatus = 1;
        }

        if (empty($shopifyOrderId)) {

            $returnData = [
                'error' => 'Required parameter orderId is missing in the endpoint request',
            ];
            Log::channel('orderCancelRefundEndpointLog')->info("Required parameter orderId is missing 
            in the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }
        if (empty($rechargeToken) && $customOrderCreationStatus == 0) {

            $returnData = [
                'error' => 'Required parameter token is missing in the endpoint request',
            ];
            Log::channel('orderCancelRefundEndpointLog')->info("Required parameter recharge token is missing 
            in the endpoint request, OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        if ($customOrderCreationStatus == 0) {

            // Get charge details from recharge by shopify order ID
            $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderId);
            if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError  = "Unknown error";
                if (!empty($chargeDetails->error)) {

                    $rechargeError = json_encode($chargeDetails->error);
                } else if (!empty($chargeDetails->errors)) {

                    $rechargeError = json_encode($chargeDetails->errors);
                } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                    $rechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderId;
                }
                $returnData = [
                    'error' => $rechargeError
                ];

                Log::channel('orderCancelRefundEndpointLog')->info("Failed to get the charge details from 
                the recharge(OrderId:".$shopifyOrderId."), Error: (".$rechargeError.")");
                return $this->returnData(600, "error", $returnData);
                
            }

            $chargeId = $chargeDetails->charges[0]->id;
            //$subscriptionId = $chargeDetails->charges[0]->line_items[0]->subscription_id;
            $productVariantId = $chargeDetails->charges[0]->line_items[0]->shopify_variant_id;
            $rechargeChargeAmount = $chargeDetails->charges[0]->total_price;
            $rechargeHashToken = trim($chargeDetails->charges[0]->customer_hash);

            if ($rechargeHashToken != $rechargeToken) {

                $returnData = [
                    'error' => 'Failed to authenticate the user'
                ];
                Log::channel('orderCancelRefundEndpointLog')->info("Failed to authenticate the user
                (The recharge hash tokens are mismatched), OrderId: ".$shopifyOrderId);
                return $this->returnData(600, "error", $returnData);
            }
        }


        // Get order details from Shopify
        $shopifyOrderDetails = $this->orderObj->getOrderDetailsById($shopifyOrderId);
        if (!empty($shopifyOrderDetails->error) || !empty($shopifyOrderDetails->errors) ||
            empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderDetails->error)) {

                $shopifyError = json_encode($shopifyOrderDetails->error);
            } else if (!empty($shopifyOrderDetails->errors)) {

                $shopifyError = json_encode($shopifyOrderDetails->errors);
            } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

                $shopifyError = 'Failed to get the shopify order details by using the given orderId: '.$shopifyOrderId;
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('orderCancelRefundEndpointLog')->info("Failed to get the order details 
            from Shopify(OrderId:".$shopifyOrderId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $emailAddress = !empty($shopifyOrderDetails['order']['email']) ? 
        trim($shopifyOrderDetails['order']['email']) : '';
        $orderTags = !empty($shopifyOrderDetails['order']['tags']) ? 
        trim($shopifyOrderDetails['order']['tags']) : '';
        $orderCreatedAt = !empty($shopifyOrderDetails['order']['created_at']) ?
        trim($shopifyOrderDetails['order']['created_at']) : '';

        $orderFinancialStatus = !empty($shopifyOrderDetails['order']['financial_status']) ? 
        trim(strtolower($shopifyOrderDetails['order']['financial_status'])) : '';
        $orderFulfillmentStatus = !empty($shopifyOrderDetails['order']['fulfillment_status']) ? 
        trim($shopifyOrderDetails['order']['fulfillment_status']) : '';
        $orderCancelledStatus = !empty($shopifyOrderDetails['order']['cancelled_at']) ? 
        trim($shopifyOrderDetails['order']['cancelled_at']) : '';

        $lineItemId = !empty($shopifyOrderDetails['order']['line_items'][0]['id']) ?
        $shopifyOrderDetails['order']['line_items'][0]['id'] : '';
        $productQuantity = !empty($shopifyOrderDetails['order']['line_items'][0]['quantity']) ?
        $shopifyOrderDetails['order']['line_items'][0]['quantity'] : '';

        if (!empty($orderTags)) {
            $checkOrderTagStatus = $this->checkOrderTagWithExtracted($orderTags);
        }
        // Check order tags
        if ($checkOrderTagStatus) {

            $returnData = [
                'error' => 'Skipped: order tagged with extracted'
            ];
            Log::channel('orderCancelRefundEndpointLog')->info("Skipped: order tagged with extracted, 
            OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        // Check the order is canceled or not
        if (!empty($orderCancelledStatus)) {

            $returnData = [
                'error' => 'Skipped: The Shopify order is canceled',
            ];
            Log::channel('orderCancelRefundEndpointLog')->info("Skipped: The Shopify order is canceled, 
            OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        // Check order status
        if ($orderFinancialStatus != 'paid' || !empty($orderFulfillmentStatus)) {

            $errorMessage = "Unknown error";
            if (!empty($orderFulfillmentStatus)) {

                $errorMessage = 'Skipped: The Shopify order fulfillment status is '.$orderFulfillmentStatus.'';
            } else {

                $errorMessage = 'Skipped: The Shopify order financial status is '.$orderFinancialStatus.'';
            }
            $returnData = [
                'error' => $errorMessage
            ];
            Log::channel('orderCancelRefundEndpointLog')->info($errorMessage.", OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        // Check order is older than 7 days
        $orderDateDifference = '';
        $currentDate = date('Y-m-d');
        if (!empty($orderCreatedAt) && !empty($currentDate)) {
            $orderDateDifference = $this->getOrderDateDifference($orderCreatedAt, $currentDate);
        }
        if ($orderDateDifference > 7) {

            $returnData = [
                'error' => 'Skipped: The Shopify order is older than 7 days'
            ];
            Log::channel('orderCancelRefundEndpointLog')->info("Skipped: The Shopify order is older than 7 days, 
            OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        if ($customOrderCreationStatus == 0) {

            // Get shopify order transactional details
            $transactionalDetails = $this->orderObj->getOrderTransactionalDetails($shopifyOrderId);
            if (!empty($transactionalDetails->error) || !empty($transactionalDetails->errors) ||
                empty($transactionalDetails) || empty($transactionalDetails['transactions'])) {

                $shTransactionalError  = "Unknown error";
                if (!empty($transactionalDetails->error)) {

                    $shTransactionalError = json_encode($transactionalDetails->error);
                } else if (!empty($transactionalDetails->errors)) {

                    $shTransactionalError = json_encode($transactionalDetails->errors);
                } else if (empty($transactionalDetails) || empty($transactionalDetails['transactions'])) {

                    $shTransactionalError = 'Failed to get the shopify transaction details by using the given orderId: '.$shopifyOrderId;
                }
                $returnData = [
                    'error' => $shTransactionalError
                ];

                Log::channel('orderCancelRefundEndpointLog')->info("Failed to get the Shopify order 
                transactional details(OrderId:".$shopifyOrderId."), Error: (".$shTransactionalError.")");
                return $this->returnData(600, "error", $returnData);
                
            }
            $transactionalId = '';
            $transactionalGateWay = '';
            foreach ($transactionalDetails['transactions'] as $transKey => $transValue) {

                if ($transValue['kind'] == 'sale') {
                    $transactionalId = $transValue['id'];
                    $transactionalGateWay = $transValue['gateway'];
                }
            }

            // Refund shopify order via shopify API
            $shopifyRefundRequest = [];
            $shopifyRefundRequest = array(
                "refund" => array(
                    "currency" => "USD",
                    "notify" => true,
                    "shipping" => array(
                        "full_refund" => true
                    ),
                    "refund_line_items" => array(
                        array(
                            "line_item_id" => $lineItemId,
                            "quantity" => $productQuantity,
                            "line_item" => array(
                                "variant_id" => $productVariantId
                            )
                        )
                    ),
                    "transactions" => array(
                        array(
                            "parent_id" => $transactionalId,
                            "amount" => $rechargeChargeAmount,
                            "kind" => "refund",
                            "gateway" => $transactionalGateWay
                        )
                    )
                )
            );

            $refundResponse = $this->orderObj->refundOrder($shopifyOrderId, $shopifyRefundRequest);
            if (!empty($refundResponse->error) || !empty($refundResponse->errors) ||
                empty($refundResponse)) {

                $shopifyRefundError  = "Unknown error";
                if (!empty($refundResponse->error)) {

                    $shopifyRefundError = json_encode($refundResponse->error);
                } else if (!empty($refundResponse->errors)) {

                    $shopifyRefundError = json_encode($refundResponse->errors);
                } else if (empty($refundResponse)) {

                    $shopifyRefundError = 'The refunded response from Shopify is empty';
                }

                $returnData = [
                    'error' => $shopifyRefundError
                ];

                Log::channel('orderCancelRefundEndpointLog')->info("Failed to refund the order from Shopify
                (OrderId:".$shopifyOrderId."), Error: (".$shopifyRefundError.")");
                return $this->returnData(600, "error", $returnData);
            }

            // Refund recharge charge
            $refundAmountRequest = array(
                "amount" => $rechargeChargeAmount
            );
            $chargeRefundedDetails = $this->rechargeObj->refundRechargeChargeById($chargeId, json_encode($refundAmountRequest));
            if (!empty($chargeRefundedDetails->error) || !empty($chargeRefundedDetails->errors) ||
                empty($chargeRefundedDetails)) {

                $rechargeRefundError  = "Unknown error";
                if (!empty($chargeRefundedDetails->error)) {

                    $rechargeRefundError = json_encode($chargeRefundedDetails->error);
                } else if (!empty($chargeRefundedDetails->errors)) {

                    $rechargeRefundError = json_encode($chargeRefundedDetails->errors);
                } else if (empty($chargeRefundedDetails)) {

                    $rechargeRefundError = 'The refunded response from recharge is empty';
                }

                $returnData = [
                    'error' => $rechargeRefundError
                ];

                Log::channel('orderCancelRefundEndpointLog')->info("Failed to refund the charge from recharge
                (OrderId:".$shopifyOrderId."), Error: (".$rechargeRefundError.")");
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Cancel shopify order
        $orderCancelledDetails = $this->orderObj->cancelOrder($shopifyOrderId);
        if (!empty($orderCancelledDetails->error) || !empty($orderCancelledDetails->errors) ||
            empty($orderCancelledDetails)) {

            $orderCancelError  = "Unknown error";
            if (!empty($orderCancelledDetails->error)) {

                $orderCancelError = json_encode($orderCancelledDetails->error);
            } else if (!empty($orderCancelledDetails->errors)) {

                $orderCancelError = json_encode($orderCancelledDetails->errors);
            } else if (empty($orderCancelledDetails)) {

                $orderCancelError = 'The order canceled response from Shopify is empty';
            }

            $returnData = [
                'error' => $orderCancelError
            ];

            Log::channel('orderCancelRefundEndpointLog')->info("Failed to cancel the order from Shopify
            (OrderId:".$shopifyOrderId."), Error: (".$orderCancelError.")");
            return $this->returnData(600, "error", $returnData);
        }

        // Update Shopify order tags
        $existingOrderTags = [];
        if (!empty($orderTags)) {
            $existingOrderTags = explode(",", $orderTags);
            $existingOrderTags = array_map('trim', $existingOrderTags);
        }

        $updatedShopifyTags = '';
        $requestedSourceTag = '';
        $shopifyTagsToUpdate = [];
        $shopifyTagsToUpdate = $existingOrderTags;
        $shopifyTagsToUpdate[] = 'auto-cancelled-refunded';
        if ($requestedSource == 'faq') {
            $requestedSourceTag = 'cancel-by-FAQ';
        } else if ($requestedSource == 'app') {
            $requestedSourceTag = 'cancel-by-CS';
        }
        if (!empty($requestedSourceTag)) {
            $shopifyTagsToUpdate[] = $requestedSourceTag;
        }
        $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

        $orderUpdateRequest = [];
        $orderUpdateRequest = array (
            "order" => array (
                "id" => $shopifyOrderId,
                "tags" => $updatedShopifyTags
            )
        );

        // Update order tags to Shopify
        $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
        if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
            empty($shopifyOrderUpdateResponse)) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderUpdateResponse['error'])) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
            } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
            } else if (empty($shopifyOrderUpdateResponse)) {

                $shopifyError = 'Failed to update the Shopify order';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('orderCancelRefundEndpointLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
            json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $returnData = [
            'message' => 'Successfully refunded and canceled the Shopify order'
        ];

        Log::channel('orderCancelRefundEndpointLog')->info("Successfully refunded and canceled the Shopify order
        (OrderId:".$shopifyOrderId.", Email: ".$emailAddress.")");
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Process order rerouted
     */
    public function processOrderReroute(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);

        if (empty($receivedData->orderId) && empty($receivedData->token)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $shopifyOrderId = !empty($receivedData->orderId) ? trim($receivedData->orderId) : '';
        $rechargeToken = !empty($receivedData->token) ? trim($receivedData->token) : '';
        $allFutureOrderStatus = !empty($receivedData->allFutureOrders) ? trim($receivedData->allFutureOrders) : 0;
        $comments = !empty($receivedData->comments) ? trim($receivedData->comments) : '';
        $requestedSource = !empty($receivedData->requestSource) ? trim(strtolower($receivedData->requestSource)) : '';
        $orderCreationToolStatus = !empty($receivedData->orderCreationToolStatus) ? 
        trim(strtolower($receivedData->orderCreationToolStatus)) : '';

        $customOrderCreationStatus = 0;
        if ($requestedSource == 'app' && $orderCreationToolStatus == 'true') {

            $customOrderCreationStatus = 1;
        }

        if (empty($shopifyOrderId)) {

            $returnData = [
                'error' => 'Required parameter orderId is missing in the endpoint request',
            ];
            Log::channel('orderCancelRerouteEndpointLog')->info("Required parameter orderId is missing 
            in the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }
        if (empty($rechargeToken) && $customOrderCreationStatus == 0) {

            $returnData = [
                'error' => 'Required parameter token is missing in the endpoint request',
            ];
            Log::channel('orderCancelRerouteEndpointLog')->info("Required parameter recharge token is missing 
            in the endpoint request, OrderId: ".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        $orderNumberPrefix = (int)env('ORDER_NUMBER_PREFIX', true);
        $orderPrefixCount = (int)strlen((string)$orderNumberPrefix);
        $prefixFromTheInputOrderId = substr($shopifyOrderId, 0, $orderPrefixCount);
        $prefixFromTheInputOrderId = (int)$prefixFromTheInputOrderId;
        $shopifyTopId = '';

        if ($orderNumberPrefix == $prefixFromTheInputOrderId) {

            // Get Shopify order details by order name
            $shopifyOrderDetails = $this->orderObj->getOrderDetailsByOrderName($shopifyOrderId);
            if (!empty($shopifyOrderDetails->error) || !empty($shopifyOrderDetails->errors) ||
                empty($shopifyOrderDetails) || empty($shopifyOrderDetails['orders'])) {

                $shopifyError  = "Unknown error";
                if (!empty($shopifyOrderDetails->error)) {

                    $shopifyError = json_encode($shopifyOrderDetails->error);
                } else if (!empty($shopifyOrderDetails->errors)) {

                    $shopifyError = json_encode($shopifyOrderDetails->errors);
                } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['orders'])) {

                    $shopifyError = 'Failed to get the shopify order details by using the given orderId: '.$shopifyOrderId;
                }
                $returnData = [
                    'error' => $shopifyError
                ];

                Log::channel('orderCancelRerouteEndpointLog')->info("Failed to get the order details 
                from Shopify(OrderId:".$shopifyOrderId."), Error: (".$shopifyError.")");
                return $this->returnData(600, "error", $returnData);
            }

            $shopifyTopId = !empty($shopifyOrderDetails['orders'][0]['id']) ? trim($shopifyOrderDetails['orders'][0]['id']) : '';
        }

        if (empty($shopifyTopId)) {

            $shopifyTopId = $shopifyOrderId;
        }

        if ($customOrderCreationStatus == 0) {

            // Get charge details from recharge by shopify order ID
            $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyTopId);
            if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError  = "Unknown error";
                if (!empty($chargeDetails->error)) {

                    $rechargeError = json_encode($chargeDetails->error);
                } else if (!empty($chargeDetails->errors)) {

                    $rechargeError = json_encode($chargeDetails->errors);
                } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                    $rechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyTopId;
                }
                $returnData = [
                    'error' => $rechargeError
                ];

                Log::channel('orderCancelRerouteEndpointLog')->info("Failed to get the charge details from 
                the recharge(OrderId:".$shopifyTopId."), Error: (".$rechargeError.")");
                return $this->returnData(600, "error", $returnData);
                
            }

            $rechargeHashToken = trim($chargeDetails->charges[0]->customer_hash);

            if ($rechargeHashToken != $rechargeToken) {

                $returnData = [
                    'error' => 'Failed to authenticate the user'
                ];
                Log::channel('orderCancelRerouteEndpointLog')->info("Failed to authenticate the user
                (The recharge hash tokens are mismatched), OrderId: ".$shopifyTopId);
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Get order details from Shopify
        $shopifyOrderDetails = $this->orderObj->getOrderDetailsById($shopifyTopId);
        if (!empty($shopifyOrderDetails->error) || !empty($shopifyOrderDetails->errors) ||
            empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderDetails->error)) {

                $shopifyError = json_encode($shopifyOrderDetails->error);
            } else if (!empty($shopifyOrderDetails->errors)) {

                $shopifyError = json_encode($shopifyOrderDetails->errors);
            } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

                $shopifyError = 'Failed to get the shopify order details by using the given orderId: '.$shopifyTopId;
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('orderCancelRerouteEndpointLog')->info("Failed to get the order details 
            from Shopify(OrderId:".$shopifyTopId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $emailAddress = !empty($shopifyOrderDetails['order']['email']) ? 
        trim($shopifyOrderDetails['order']['email']) : '';
        $orderTags = !empty($shopifyOrderDetails['order']['tags']) ? 
        trim($shopifyOrderDetails['order']['tags']) : '';
        $orderCreatedAt = !empty($shopifyOrderDetails['order']['created_at']) ?
        trim($shopifyOrderDetails['order']['created_at']) : '';

        $orderFinancialStatus = !empty($shopifyOrderDetails['order']['financial_status']) ? 
        trim(strtolower($shopifyOrderDetails['order']['financial_status'])) : '';
        $orderFulfillmentStatus = !empty($shopifyOrderDetails['order']['fulfillment_status']) ? 
        trim($shopifyOrderDetails['order']['fulfillment_status']) : '';
        $orderCancelledStatus = !empty($shopifyOrderDetails['order']['cancelled_at']) ? 
        trim($shopifyOrderDetails['order']['cancelled_at']) : '';

        $orderNumber = !empty($shopifyOrderDetails['order']['name']) ? 
        trim($shopifyOrderDetails['order']['name']) : '';
        $customerFirstName = !empty($shopifyOrderDetails['order']['customer']['first_name']) ? 
        trim($shopifyOrderDetails['order']['customer']['first_name']) : '';

        $checkOrderTagStatus = '';
        if (!empty($orderTags)) {
            $checkOrderTagStatus = $this->checkOrderTagWithExtracted($orderTags);
        }
        // Check order tags
        if (!$checkOrderTagStatus) {

            // $returnData = [
            //     'error' => 'Skipped: order not tagged with extracted'
            // ];
            // Log::channel('orderCancelRerouteEndpointLog')->info("Skipped: order not tagged with extracted, 
            // OrderId: ".$shopifyTopId);
            // return $this->returnData(600, "error", $returnData);
            $orderDateDifference = '';
            $requestedSourceValue = '';
            $currentDate = date('Y-m-d');
            if (!empty($orderCreatedAt) && !empty($currentDate)) {
                $orderDateDifference = $this->getOrderDateDifference($orderCreatedAt, $currentDate);
            }

            $requestedSourceValue = !empty($requestedSource) ? trim($requestedSource) : '';
            if ($orderDateDifference > 2) {
 
                $returnData = [
                    'error' => 'Skipped: The Shopify order is older than 24 hours'
                ];
                Log::channel('orderCancelRerouteEndpointLog')->info("Skipped: The Shopify order is older than 24 hours, 
                OrderId: ".$shopifyTopId);
                return $this->returnData(600, "error", $returnData);
            }

            $laravelRequestObj = new \Illuminate\Http\Request();
            $laravelRequestObj->setMethod('POST');
            $endpointRequestParameters = [
                "token"   => $rechargeToken,
                "orderId" => $shopifyTopId,
                "requestSource" => $requestedSourceValue,
                "orderCreationToolStatus" => $orderCreationToolStatus
            ];
            $encodedEndpointRequest = json_encode($endpointRequestParameters);
            $laravelRequestObj->request->add([
                "data" => $encodedEndpointRequest
            ]);
            $orderCancelAndRefundResponse = $this->processOrderCancel($laravelRequestObj);
            $endpointResponseStatus = !empty($orderCancelAndRefundResponse->original['status']) ? 
            trim($orderCancelAndRefundResponse->original['status']) : '';
            $endpointResponseError = !empty($orderCancelAndRefundResponse->original['body']['error']) ? 
            trim($orderCancelAndRefundResponse->original['body']['error']) : '';

            if (empty($endpointResponseStatus) || $endpointResponseStatus != '200') {

                $errorMessage = !empty($endpointResponseError) ? $endpointResponseError : 
                "Failed to refund and cancel the Shopify order";
                $returnData = [
                    'message' => $errorMessage
                ];
        
                Log::channel('orderCancelRerouteEndpointLog')->info($errorMessage.
                "(OrderId: ".$shopifyTopId.", Email: ".$emailAddress.")");
                return $this->returnData(600, "error", $returnData);
            }
            $returnData = [
                'message' => 'Successfully refunded and canceled the Shopify order'
            ];
    
            Log::channel('orderCancelRerouteEndpointLog')->info("Successfully refunded and canceled the Shopify order
            (OrderId: ".$shopifyTopId.", Email: ".$emailAddress.")");
            return $this->returnData(200, "success", $returnData);
        }

        // Check the order is canceled or not
        if (!empty($orderCancelledStatus)) {

            $returnData = [
                'error' => 'Skipped: The Shopify order is canceled',
            ];
            Log::channel('orderCancelRerouteEndpointLog')->info("Skipped: The Shopify order is canceled, 
            OrderId: ".$shopifyTopId);
            return $this->returnData(600, "error", $returnData);
        }

        // Check order status
        if ($orderFinancialStatus != 'paid' || !empty($orderFulfillmentStatus)) {

            $errorMessage = "Unknown error";
            if (!empty($orderFulfillmentStatus)) {

                $errorMessage = 'Skipped: The Shopify order fulfillment status is '.$orderFulfillmentStatus.'';
            } else {

                $errorMessage = 'Skipped: The Shopify order financial status is '.$orderFinancialStatus.'';
            }
            $returnData = [
                'error' => $errorMessage
            ];
            Log::channel('orderCancelRerouteEndpointLog')->info($errorMessage.", OrderId: ".$shopifyTopId);
            return $this->returnData(600, "error", $returnData);
        }

         // Check order is older than 7 days
         $orderDateDifference = '';
         $currentDate = date('Y-m-d');
         if (!empty($orderCreatedAt) && !empty($currentDate)) {
             $orderDateDifference = $this->getOrderDateDifference($orderCreatedAt, $currentDate);
         }
         if ($orderDateDifference > 7) {
 
             $returnData = [
                 'error' => 'Skipped: The Shopify order is older than 7 days'
             ];
             Log::channel('orderCancelRerouteEndpointLog')->info("Skipped: The Shopify order is older than 7 days, 
             OrderId: ".$shopifyTopId);
             return $this->returnData(600, "error", $returnData);
         }

         $requestToInternalEndpoint = [];
         $requestToInternalEndpoint = array(
             "OrderNumber" => $orderNumber,
             "AllFutureOrders" => $allFutureOrderStatus,
             "Source" => "nurish.com",
             "Comments" => $comments
         );
         
         $internalEndpointSubmitResponse = $this->internalDataPostObj->postRerouteOrderDetailsToNurish(
             json_encode($requestToInternalEndpoint));
        
        if (!empty($internalEndpointSubmitResponse)) {

            $message = 'Failed';
            $returnLogData = [
                'request' => json_encode($requestToInternalEndpoint),
                'error' => json_encode($internalEndpointSubmitResponse)
            ];
            $returnData = [
                'error' => 'Error occured while posting the request to the internal API'
            ];
            
            Log::channel('orderCancelRerouteEndpointLog')->info(json_encode($returnLogData));
            return $this->returnData(600, 'error', $returnData);
        }

        // Update Shopify order tags
        $existingOrderTags = [];
        if (!empty($orderTags)) {
            $existingOrderTags = explode(",", $orderTags);
            $existingOrderTags = array_map('trim', $existingOrderTags);
        }

        $updatedShopifyTags = '';
        $shopifyTagsToUpdate = [];
        $shopifyTagsToUpdate = $existingOrderTags;
        $shopifyTagsToUpdate[] = 'cancel-request-sent-to-room';
        $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

        $orderUpdateRequest = [];
        $orderUpdateRequest = array (
            "order" => array (
                "id" => $shopifyTopId,
                "tags" => $updatedShopifyTags
            )
        );

        // Update order tags to Shopify
        $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyTopId);
        if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
            empty($shopifyOrderUpdateResponse)) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderUpdateResponse['error'])) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
            } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
            } else if (empty($shopifyOrderUpdateResponse)) {

                $shopifyError = 'Failed to update the Shopify order';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('orderCancelRerouteEndpointLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
            json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        if ($requestedSource == 'faq') {
            // Create iterable event
            $iterableEventCreateRequest = [];

            $iterableEventCreateRequest = array(
                "email" => strtolower($emailAddress), 
                "eventName" => "order-cancellation-request",
                "dataFields" => array(
                    "email" => $emailAddress,
                    "orderID" => $orderNumber,
                    "firstName" => $customerFirstName
                )
            );

            $eventResponse = $this->iterableObj->createIterableEvent(json_encode($iterableEventCreateRequest));

            if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {

                Log::channel('orderCancelRerouteEndpointLog')->info("Iterable event: - Successfully created the
                event for Email(".strtolower($emailAddress).")");
            } else {
                Log::channel('orderCancelRerouteEndpointLog')->info("Iterable event: - Failed to create the event 
                for Email (".strtolower($emailAddress).")");
            }
        }

        $returnData = [
            'message' => 'Successfully posted the given request to the internal API'
        ];

        Log::channel('orderCancelRerouteEndpointLog')->info("Successfully posted the given request in the API
        (OrderId:".$shopifyTopId.", Email: ".$emailAddress.")");
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Check Shopify order tags
     */
    public function checkOrderTagWithExtracted($orderTags) {

        $explodeTags = explode(",", $orderTags);
        $explodeTags = array_map('trim', $explodeTags);
        $explodeTags = array_map('strtolower', $explodeTags);
        if (in_array("extracted", $explodeTags)) {
            return true;
        }
        return false;
    }

    /**
     * Calculate the order date difference
     */
    public function getOrderDateDifference($orderCreatedDate, $currentDate) {

        $orderCreatedDate = strtotime($orderCreatedDate);
        $currentDate = strtotime($currentDate);
        $difference = $currentDate - $orderCreatedDate;
        return round($difference / 86400);
    }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error

        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
        }
    }

    /**
     * Get order extracted tag status
     */
    public function getOrderExtractedTagStatus(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);

        if (empty($receivedData->orderId) && empty($receivedData->hash)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $shopifyOrderID = !empty($receivedData->orderId) ? trim($receivedData->orderId) : '';
        $customerHash = !empty($receivedData->hash) ? trim($receivedData->hash) : '';

        if (empty($shopifyOrderID) || empty($customerHash)) {

            if (empty($shopifyOrderID)) {

                $returnData = [
                    'error' => 'Required parameter orderId is missing in the endpoint request',
                ];
                Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Required parameter orderId is missing 
                in the endpoint request, request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }

            if (empty($customerHash)) {

                $returnData = [
                    'error' => 'Required parameter customer hash is missing in the endpoint request',
                ];
                Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Required parameter customer hash is missing 
                in the endpoint request, request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Get charge details from recharge by shopify order ID
        $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderID);
        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
            empty($chargeDetails) || empty($chargeDetails->charges)) {

            $rechargeError  = "Unknown error";
            if (!empty($chargeDetails->error)) {

                $rechargeError = json_encode($chargeDetails->error);
            } else if (!empty($chargeDetails->errors)) {

                $rechargeError = json_encode($chargeDetails->errors);
            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderID;
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Failed to get the charge details from 
            the recharge(OrderId:".$shopifyOrderID."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $rechargeHashToken = trim($chargeDetails->charges[0]->customer_hash);

        if ($rechargeHashToken != $customerHash) {

            $returnData = [
                'error' => 'Found hash mismatch and failed to authenticate'
            ];
            Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Found hash mismatch and failed to authenticate
            (The recharge hash tokens are mismatched), OrderId: ".$shopifyOrderID);
            return $this->returnData(600, "error", $returnData);
        }

        // Get order details from Shopify
        $shopifyOrderDetails = $this->orderObj->getOrderDetailsById($shopifyOrderID);
        if (!empty($shopifyOrderDetails->error) || !empty($shopifyOrderDetails->errors) ||
            empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderDetails->error)) {

                $shopifyError = json_encode($shopifyOrderDetails->error);
            } else if (!empty($shopifyOrderDetails->errors)) {

                $shopifyError = json_encode($shopifyOrderDetails->errors);
            } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

                $shopifyError = 'Failed to get the shopify order details by using the given orderId: '.$shopifyOrderID;
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Failed to get the order details 
            from Shopify(OrderId:".$shopifyOrderID."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $emailAddress = !empty($shopifyOrderDetails['order']['email']) ? 
        trim($shopifyOrderDetails['order']['email']) : '';
        $orderTags = !empty($shopifyOrderDetails['order']['tags']) ? 
        trim($shopifyOrderDetails['order']['tags']) : '';

        $extractedOrderTagStatus = 'false';
        if (!empty($orderTags)) {
            $explodeTags = explode(",", $orderTags);
            $explodeTags = array_map('trim', $explodeTags);
            $explodeTags = array_map('strtolower', $explodeTags);
            if (in_array("extracted", $explodeTags)) {

                $extractedOrderTagStatus = 'true';
            } else {
                $extractedOrderTagStatus = 'false';
            }
        }

        $returnData = [
            'OrderExtractedTagStatus' => $extractedOrderTagStatus
        ];

        Log::channel('orderExtractedTagStatusCheckEndpointLog')->info("Extracted order tag status - ".
        $extractedOrderTagStatus." (OrderId:".$shopifyOrderID.", Email: ".$emailAddress.")");
        return $this->returnData(200, "success", $returnData);
    }
}
