<?php

namespace App\Http\Controllers;


use Carbon\Carbon;
use Log;
use App\Models\RechargeApi;
use App\Models\IterableApi;

class SubscriptionDiscountCron extends Controller
{

    private $rechargeObj;
    private $iterableObj;
    private $processedCharges;

    public function __construct()
    {          
        $this->processedCharges = [];
        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj = new IterableApi();    
        date_default_timezone_set('America/Los_Angeles');

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/discountAppliedChargeReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }
        
        $this->reportFileShortname = "discount-success".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'chargeDiscount' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->reportFileShortname,
                "mode"     => "",
                "columns"  => array(
                    "Email",
                    "SubscriptionID",
                    "ChargeID",                    
                    "ScheduledAt",                   
                    "DiscountCode",
                    "Status"
                )
            )        
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }
    

    public function processSubscriptions() {

        $startDate = new Carbon(date('Y-m-d'), 'America/Los_Angeles');
        $startDate = $startDate->subDays("1");
        $startDate = $startDate->format('Y-m-d');

        $endDate =  new Carbon(date('Y-m-d'), 'America/Los_Angeles');       
        $endDate = $endDate->addDays("90");  
        $endDate = $endDate->format('Y-m-d');
       
        // $subscriptionData = $this->getAllRechargeSubscriptions();
        $allCharges = $this->getUpcomingCharges($startDate, $endDate);

        if(isset($allCharges) && !empty($allCharges)){
            $subscriptionCharges = $allCharges;
            foreach($subscriptionCharges as $charge){

                $lineItemProperties = $charge->line_items[0]->properties;
                foreach ($lineItemProperties as $property) {                               
                    

                    $propertyName = strtolower(trim($property->name));
                    $propertyValue = strtolower(trim($property->value));

                    if (strpos($propertyName, 'future_month_free') !== false && $propertyValue == '3') {
                        
                        $this->processCharges($charge);
                    }
                }
            }
        }

        $cronStartedAt =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        Log::channel('subscriptionDiscountCron')->info("Cron Started-------".$cronStartedAt);

        
                     
        // for sending the report email 
        $this->sendReportForDaysUntilFulfillment();
    }

    public function processCharges($charge) {

        $successStatusCount = 0;
        $queuedStatusCount = 0;
        if(!empty($charge->line_items[0]->subscription_id)){
            $subscriptionId = $charge->line_items[0]->subscription_id;
            $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);
    
            if(isset($subscriptionChargeDetails->charges) && !empty($subscriptionChargeDetails->charges)){

                foreach ($subscriptionChargeDetails->charges as $subKey => $subValue) {

                    if (strtolower(trim($subValue->status)) == 'success') {
                        $successStatusCount++; 
                    }
                    if (strtolower(trim($subValue->status)) == 'queued') {
                        $queuedStatusCount++; 
                    }
                }
                if(
                    //(count($subscriptionChargeDetails->charges) > 1)
                    ($successStatusCount == 2 && $queuedStatusCount == 1)  && (empty($charge->discount_codes))
                ){
                    $this->applyDiscountToCharge($charge);                    
                }
            }
        }
    }

    public function applyDiscountToCharge($charge) {

        $fileName    = $this->logFiles['chargeDiscount']['filename'];
        $filePointer = fopen($fileName, "a");
        $discountCode = '3rdMonthFree';
        $chargeId = $charge->id;
        $email = $charge->email;
        $firstName = $charge->first_name;
        $subscriptionId = $charge->line_items[0]->subscription_id;
        
        
        $scheduledAtDate =  new Carbon($charge->scheduled_at,'America/Los_Angeles');                            
        $scheduledAtDate = $scheduledAtDate->toDayDateTimeString();

        $discountRequest = json_encode(array(
            "discount_code" => $discountCode
        ));
        $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($chargeId,$discountRequest);
        if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                            empty($discountApplyResponse)) {
            $error  = "Unknown error";
            if (!empty($discountApplyResponse->error)) {

                $error = $discountApplyResponse->error;

            } else if (!empty($discountApplyResponse->errors)) {

                $error = "Failed to apply the discount to the charge";
            }

            Log::channel('subscriptionDiscountCron')->info(
                "Failed to apply the discount to the charge: ".
                " | Subscription -> ".$subscriptionId.
                " | chargeID: ".$chargeId.
                " | DiscountCode: ".$discountCode.
                " | Error -> ".json_encode($error)
            );
            $failedCharge =array(
                trim($email),
                $subscriptionId,
                $chargeId,
                $scheduledAtDate,                
                $discountCode,
                'Failed to apply discount code - '.json_encode($error)
            );
            fputcsv($filePointer, $failedCharge);
            $this->processedCharges[] = $failedCharge;

        }else{
            Log::channel('subscriptionDiscountCron')->info(
                "Successfully changes the charge date of subscription and applied the discount.".
                " | Subscription -> ".$subscriptionId.
                " | chargeID: ".$chargeId.
                " | DiscountCode: ".$discountCode
            );

            $successCharge = array(
                trim($email),
                $subscriptionId,
                $chargeId,
                $scheduledAtDate,                
                $discountCode,
                'Success'
            );
            fputcsv($filePointer, $successCharge);


            $this->processedCharges[] = $successCharge;
        }
    }

   
    public function sendReportForDaysUntilFulfillment() {

        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();        
        $subject = "Nurish-Dev Discount Report - ".$reportSendingTime;
        
        //$recipientEmails = 'testing@p80w.com';
        $recipientEmails = 'mikegordon79@gmail.com';
        
        if (count($this->processedCharges) == 0) {
            Log::channel('subscriptionDiscountCron')->info("No charges met the specified conditions");
            $commonContent = '<p>No discount codes were applied to orders during this execution</p>';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_DISCOUNT_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => '',
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } elseif( count($this->processedCharges) > 30) {

            $fileUrl = url('/').'/discountAppliedChargeReport/'.$this->reportFileDirectory.'/'.$this->reportFileShortname;
            $commonContent = '<p>Please see the report for subscriptions with discount codes applied to orders.</p>';
            $recordDetails = 'Please <a href="'.$fileUrl.'">click here</a> to download the file.';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_DISCOUNT_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
            
        } elseif (count($this->processedCharges) > 0 && count($this->processedCharges) <= 30) {
            $commonContent = '<p>Please see the report for subscriptions with discount codes applied to orders.</p>';
            
            $recordDetails = '';
            if(!empty($this->processedCharges)){           
                $recordDetails .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>Email</td>
                        <td>SubscriptionID</td> 
                        <td>ChargeID</td> 
                        <td>ScheduledAt</td>
                        <td>DiscountCode</td>   
                        <td>Status</td>                       
                    </tr>            
                ";

                $i = 1;
                foreach($this->processedCharges as $record){
                    
                    $orderScheduledAtDate =  new Carbon($record[3],'America/Los_Angeles');                            
                    $orderScheduledAtDate = $orderScheduledAtDate->toDayDateTimeString();
                            
                    $recordDetails .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record[0]."</td> 
                        <td>".$record[1]."</td>  
                        <td>".$record[2]."</td>
                        <td>".$orderScheduledAtDate."</td>
                        <td>".$record[4]."</td>
                        <td>".$record[5]."</td> 
                    </tr>
                    ";
                    $i++;            
                }
                $recordDetails .= '</table>';
            }
            
            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_DISCOUNT_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        }
        
        if(!empty($iterableMailRequest)){
            
            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                
                Log::channel('subscriptionDiscountCron')->info("mail send.........................");
    
            } else {
                
                Log::channel('subscriptionDiscountCron')->info('mail not send......................');
                Log::channel('subscriptionDiscountCron')->info(json_encode($mailTriggerResponse));
                $error = $mailTriggerResponse->code;
    
                if (empty($error)) {
    
                    $returnData = [
                        'error' => 'failed to send mail(' . $recipientEmails . ')',
                    ];
                } else {
    
                    $returnData = [
                        'error' => $error . ' (' . $recipientEmails . ')',
                    ];
                }
                
                Log::channel('subscriptionDiscountCron')->info($returnData['error']);
                 
            }
        }
    }

    /**
     * Function to get all the upcoming charges
    */

    public function getUpcomingCharges($startDate, $endDate) {
        
        $page   = 1;
        $limit  = 100;       
        $subscriptionCharges = [];        
        
        do{
            $result = $this->rechargeObj->getChargesByMaxMinDates($startDate, $endDate, $page, $limit);          
            
            if(isset($result->charges)){
                $count = count($result->charges);        
                foreach($result->charges as $charge){
                    
                    $subscriptionCharges[] = $charge;
                }
            } else {
                $count = 0;
            }
            $page++; 
        } while ($count == $limit);
        
        return $subscriptionCharges;
    }
}
