<?php

namespace App\Http\Middleware;

use Closure;
use DB;
use App\Http\ResponseHelper;
use Config;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;

class TokenAuthorizer
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */

    private $helper;

    public function handle($request, Closure $next) {

        $this->helper = new ResponseHelper();

        $options = new Options();
        $shApiVersion = env("SHOPIFY_API_VERSION", true);
        $storeDomain = Config::get('shopify.shopDomainQuiz');
        $accessToken = Config::get('shopify.passwordQuiz');
        $options->setVersion($shApiVersion);

        // Create the client and session
        $api = new BasicShopifyAPI($options);
        $api->setSession(new Session($storeDomain, $accessToken));

        $email     = $request->input('email');
        $token     = $request->input('token');

        if(empty($email)){
            $returnData = [
                'error' => 'Illegal Request'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }

        if(empty($token)){
            $returnData = [
                'error' => 'Illegal Request'
            ];
            
            return $this->helper->returnData(600, "error", $returnData );
        }

        //check customer in db
        $authToken = DB::table('users')->select('auth_token')->where('user_email', $email)->value('auth_token');

        if(empty($authToken)) {
        
            $query = '{
                customers(first: 1, query: "'.$email.'") {
                    edges {
                        node {
                            id,
                            email,
                            metafield(key:"authToken", namespace:"global"){
                                value
                            }
                        }       
                    }
                }
            }';

            $variables = [];
            $results = $api->graph($query, $variables = []);

            $customerEdge =  isset($result->body->customers->edges) 
                ? $result->body->customers->edges : '';
            
            if(empty($customerEdge)) {
                $returnData = [
                    'error' => 'Illegal Request'
                ];

                return $this->helper->returnData(600, "error", $returnData );
            }
            
            $customerNode = $customerEdge[0]->node;
            $authToken = isset($customerNode->metafield->value) ? $customerNode->metafield->value : '';
            
        }

        if(empty($authToken)) {
            $returnData = [
                'error' => 'Illegal Request'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }

        $hasHToken = md5($authToken);

        if($token == $hasHToken) {
            return $next($request);
        } else { 
            $returnData = [
                'error' => 'Illegal Request'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }
    }
}
