<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Config;
// use App\Models\Shopify\ExtentedShopifyAPI;
// use OhMyBrew\BasicShopifyAPI;
use DB;
use Log;


class GenerateUserTokenJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $webhook;
    public $randString;
    public $shopify_api_version;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhook, $randString) {
        
        $this->webhook = $webhook;
        $this->randString = $randString;
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle() {

        $webhookDetails = json_decode($this->webhook);
        //$api = new BasicShopifyAPI();
        //$api = new ExtentedShopifyAPI();

        $password   = Config::get('shopify.passwordWebhook');
        $shopDomain = Config::get('shopify.shopDomainWebhook');

        $options = new Options();
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $api = new BasicShopifyAPI($options);
        $api->setSession(new Session($shopDomain, $password));

        $email = $webhookDetails->email;
        $customerId  = $webhookDetails->id;

        $authToken = $this->randString;

        Log::channel('generateUserTokenLog')->info('AuthToken 1 = '.$authToken. " Email = ".$email);
        $metaFieldArray = array();

        $temp['key']        = 'authToken';
        $temp['value']      = $authToken;
        $temp['type']       = 'single_line_text_field';
        $temp['namespace']  = 'global';
        
        $metaFieldArray[] = $temp;
        //array_push($metaFieldArray, $temp);

        $user = DB::table('users')->where('user_email', '=', $email)->get();
        
        if (!isset($user[0]->user_email)) {
            // create user in DB 
            $userId = DB::table('users')->insertGetId([
                'user_email' => $email,
                'status' => 1,
                'auth_token' => $authToken
            ]);
            Log::channel('generateUserTokenLog')->info('generateUserTokenLog')->info('AuthToken 2 = '.$authToken. " Email = ".$email);
        } else {

            $userId = DB::table('users')->where('user_email', $email)->update(['auth_token' => $authToken]);  
        }

        $method = 'GET';
        $url    = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'/metafields.json';
        $params = null;
        $retry = false;
        $try = 1;
        do {
            Log::channel('generateUserTokenLog')->info($try.' try-get customer metafields ');
            $result = $api->rest($method, $url, $params); 
            Log::channel('generateUserTokenLog')->info(json_encode($result));

	        if (!isset($result['body']['metafields'])) {
                $retry = true;
                $try++;
            }
        } while($retry && $try <=3);

        if (empty($result['errors'])) {

            $metafields = $result['body']['metafields'];

            if (!empty($metafields)) {

                $updated = false;
                foreach ($metafields as $metafield) {
    
                    $metaId        = $metafield->id;
                    $metaKey       = $metafield->key;
                    $metaNameSpace = $metafield->namespace;
    
                    if ($metaKey == 'authToken' && $metaNameSpace == 'global') {
    
                        // update Metafield
                        $updateRequest = [];
                        $updateRequest['metafield']['id']         = $metaId;
                        $updateRequest['metafield']['value']      = $authToken;
                        $updateRequest['metafield']['type']       = "single_line_text_field";
    
                        $params = $updateRequest;
                
                        $method = 'PUT';
                        $url    = '/admin/api/'.$this->shopify_api_version.'/metafields/'.$metaId.'.json';
                       // sleep(1);
			     $retry = false;
       			 $try = 1;
     	 		 do {
                    Log::channel('generateUserTokenLog')->info($try.' try-update metafields ');

                    $result = $api->rest($method, $url, $params); 
    			    if (!isset($result['body']['metafield'])) {
                		$retry = true;
                		$try++;
        	   	    }
        		} while ($retry && $try <=3);

                        if (!empty($result['body']['metafield'])) {
                            // Updated successfully
                            Log::channel('generateUserTokenLog')->info('AuthToken 6 = '.$authToken. " Email = ".$email);

                            $updated = true;
                            Log::channel('generateUserTokenLog')->info("Updated metafield successfully for the user ".$customerId." with value ".$authToken);
                            
                        } else {
                            // not updated
                            $updated = false;
                            Log::channel('generateUserTokenLog')->info("Failed to update metafield for the user ".$customerId." with value ".$authToken);
                           
                        }
                
                    }
    
                }
    
                if ($updated == false) {

                    Log::channel('generateUserTokenLog')->info('AuthToken 4 = '.$authToken. " Email = ".$email);
    
                    $this->createCustomerMetafield($metaFieldArray, $customerId, $authToken, $api, $email);
                }
            } else {

                Log::channel('generateUserTokenLog')->info('AuthToken 3 = '.$authToken. " Email = ".$email);
                $this->createCustomerMetafield($metaFieldArray, $customerId, $authToken, $api, $email);
        
            }
        }
    }

    /**
     * Create customer metafield
     */
    public function createCustomerMetafield($metafield, $customerId, $authToken, $api, $email) {

        Log::channel('generateUserTokenLog')->info('AuthToken 5 = '.$authToken. " Email = ".$email);
        
        $params = array(
            'customer' => array(
                'id' => $customerId,
                'metafields' => $metafield
            )
        );

        $method = 'PUT';
        $url = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json';
        //sleep(1);
	    $retry = false;
        $try = 1;
        do {
            Log::channel('generateUserTokenLog')->info($try.' try-update customer metafields ');

       	    $result = $api->rest($method, $url, $params);
            if (!isset($result['body']['customer']['id'])) {
                $retry = true;
                $try++;
            }
        } while($retry && $try <=3);

        if (!empty($result['body']['customer'])) {
            // Updated customer successfully by adding new metafield
            $updated = true;
            Log::channel('generateUserTokenLog')->info("Updated customer successfully by adding new metafield ".$customerId." with value ".$authToken);
        } else {
            // not updated
            $updated = false;
            Log::channel('generateUserTokenLog')->info("Failed to update customer metafield for the user ".$customerId." with value ".$authToken);
        }
    }
}
