<?php

namespace App\Jobs;

use Log;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use App\Models\IterableApi;
use App\Models\RechargeApi;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Config;
// use App\Models\Shopify\ExtentedShopifyAPI;
// use OhMyBrew\BasicShopifyAPI;
use DB;

class OrderTaggingJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $webhook;
    public $shopify_api_version;
    public $rechargeObj;
    public $iterableObj;
    public $api;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhook) {
        
        $this->webhook = $webhook;
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle() {

        //$api = new BasicShopifyAPI();
        // $api = new ExtentedShopifyAPI();
        $this->rechargeObj = new RechargeApi();
        $this->iterableObj = new IterableApi();

        $password   = Config::get('shopify.passwordOrderTag');
        $shopDomain = Config::get('shopify.shopDomainOrderTag');

        // $api->setShop($shopDomain);
        // $api->setAccessToken($password);


        $options = new Options();
        $options->setVersion($this->shopify_api_version);

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session($shopDomain, $password));


        $webhookData = json_decode($this->webhook);
        // $webhookData = json_decode($webhookData);
        $lineItems    = $webhookData->line_items;
        $orderId      = $webhookData->id;
        $orderName    = $webhookData->name;
        $orderTags    = $webhookData->tags;
        $orderEmail   = $webhookData->email;
        $customerId   = $webhookData->customer->id;
        $customerTags = $webhookData->customer->tags;
        
        $newOrderTag = [];
        $newCustomerTag = [];
        $oTagFlag = 0;
        $giftFirstOrderStatus = 0;
        $giftOrderStatus = 0;
        $giftNotificationDate = '';
        $giftSendersFirstName = '';
        $giftSendersLastName = '';
        $giftPackName = '';
        $giftRecipientEmail = '';
        $chargeIntervalFrequency = '';
        $giftOrderSuccessCount = 0;
        $giftOrderQueuedCount = 0;

        if(!empty($lineItems)){

            foreach($lineItems as $lineitem){
                
                $lineItemProperties = $lineitem->properties;
                if(!empty($lineItemProperties)){

                    foreach($lineItemProperties as $lineItemProperty){

                        $name  = trim($lineItemProperty->name);
                        $value = trim($lineItemProperty->value);
                        $value = str_replace(',', '-', $value);
                        
                        $oTag  = '';
                        

                        // Set Special Pack tag
                        if($name == 'special_pack_type'){  
                            
                            $newOrderTag[]    = 'special pack:'.$value;
                            $newCustomerTag[] = 'special pack:'.$value;
                        }

                        // Set Free Override Shipping tag
                        if($name == 'override_free_shipping' && $value == 'true'){
                            
                            $newOrderTag[] = 'free-shipping-override';
                            
                        }

                        // Set Partner tag
                        if($name == 'partner'){
                            
                            $oTag = 'partner:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'partner:'.$value;
                            
                        }
                        // Set campaign tag
                        if($name == 'campaign'){
                            
                            $oTag = 'campaign:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'campaign:'.$value;
                            
                        }
                        // Set source tag
                        if($name == 'source'){
                           
                            $oTag = 'source:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'source:'.$value;
                           
                        }
                        // Set medium tag
                        if($name == 'medium'){
                           
                            $oTag = 'medium:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'medium:'.$value;
                           
                        }
                        // Set content tag
                        if($name == 'content'){
                           
                            $oTag = 'content:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'content:'.$value;
                           
                        }
                        // Set term tag
                        if($name == 'term'){
                           
                            $oTag = 'term:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'term:'.$value;
                           
                        }

                        // Set gift pack order tags
                        if ($name == 'giftee_email') {

                            $giftRecipientEmail = trim($lineItemProperty->value);
                            $giftOrderStatus = 1;
                            $oTag = 'gift';
                            if (strlen($oTag) > 40) {
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            $newOrderTag[]    =  $oTag;

                            // Get order details from recharge by shopify order ID
                            $orderDetails = $this->rechargeObj->getRechargeOrderByShopifyOrderId($orderId);
                            if (!empty($orderDetails->error) || !empty($orderDetails->errors) ||
                                empty($orderDetails) || empty($orderDetails->orders)) {

                                $rechargeError  = "Unknown error";
                                if (!empty($orderDetails->error)) {

                                    $rechargeError = json_encode($orderDetails->error);
                                } else if (!empty($orderDetails->errors)) {

                                    $rechargeError = json_encode($orderDetails->errors);
                                } else if (empty($orderDetails) || empty($orderDetails->orders)) {

                                    $rechargeError = 'Failed to get the order details by using the given orderId: '.$orderId;
                                }
                                $returnData = [
                                    'error' => $rechargeError
                                ];

                                Log::channel('orderTag')->info("Failed to get the order details from 
                                the recharge(OrderId:".$orderId."), Error: (".$rechargeError.")");
                                //return $this->returnData(600, "error", $returnData);
                            }
                            $subscriptionAddressId = !empty($orderDetails->orders[0]->address_id) ? 
                            $orderDetails->orders[0]->address_id : '';

                            // Get order by subscription addressId
                            if (!empty($subscriptionAddressId)) {
                                $rechargeOrders = $this->rechargeObj->getRechargeOrdersByAddressId($subscriptionAddressId);
                                if (!empty($rechargeOrders->error) || !empty($rechargeOrders->errors) ||
                                    empty($rechargeOrders) || empty($rechargeOrders->orders)) {

                                    $rechargeError  = "Unknown error";
                                    if (!empty($rechargeOrders->error)) {

                                        $rechargeError = json_encode($rechargeOrders->error);
                                    } else if (!empty($rechargeOrders->errors)) {

                                        $rechargeError = json_encode($rechargeOrders->errors);
                                    } else if (empty($rechargeOrders) || empty($rechargeOrders->charges)) {

                                        $rechargeError = 'Failed to get the recharge orders by using the addressId: '.$subscriptionAddressId;
                                    }
                                    $returnData = [
                                        'error' => $rechargeError
                                    ];

                                    Log::channel('orderTag')->info("Failed to get the order details from 
                                    the recharge(OrderId:".$orderId.",addressId:".$subscriptionAddressId."), Error: (".$rechargeError.")");
                                    //return $this->returnData(600, "error", $returnData);
                                }
                            } else {
                                Log::channel('orderTag')->info("The addressId in the charge is empty(OrderId:".$orderId.")");
                            }

                            $queuedOrderId = '';
                            if (!empty($rechargeOrders->orders)) {

                                foreach ($rechargeOrders->orders as $orderKey => $orderValue) {

                                    $orderStatus = !empty($orderValue->status) ? strtolower(trim($orderValue->status)) : '';
                                    if ($orderStatus == 'queued') {
                                        $queuedOrderId = $orderValue->id;
                                        $giftOrderQueuedCount++;
                                    } else if ($orderStatus == 'success') {
                                        $giftOrderSuccessCount++;
                                    }
                                }

                                if (!empty($queuedOrderId)) {

                                    $secondOrderFutureDate = date('Y-m-d', strtotime('+99 years'));
                                    $payload = array(
                                        "scheduled_at" => $secondOrderFutureDate
                                    );
                                    $queuedOrderUpdateResponse = $this->rechargeObj->updateOrderScheduledDate(
                                        $queuedOrderId, json_encode($payload));

                                    if (!empty($queuedOrderUpdateResponse->error) || !empty($queuedOrderUpdateResponse->errors) ||
                                    empty($queuedOrderUpdateResponse)) {

                                        $rechargeError  = "Unknown error";
                                        if (!empty($queuedOrderUpdateResponse->error)) {

                                            $rechargeError = json_encode($queuedOrderUpdateResponse->error);
                                        } else if (!empty($queuedOrderUpdateResponse->errors)) {

                                            $rechargeError = json_encode($queuedOrderUpdateResponse->errors);
                                        } else if (empty($queuedOrderUpdateResponse)) {

                                            $rechargeError = 'Failed to update the order scheduled_at date';
                                        }

                                        $returnData = [
                                            'error' => $rechargeError
                                        ];

                                        Log::channel('orderTag')->info("Failed to update the order scheduled_at 
                                        date(orderId: ".$queuedOrderId.", Error:".$rechargeError.")");
                                        //return $this->returnData(600, "error", $returnData, $request->input('callback'));
                                    }
                                }
                            }
                        }

                        // Set gift notification date order tags
                        if ($name == 'gift_notification_date') {
                            $giftNotificationDate = trim($lineItemProperty->value);
                        }

                        // Set gift renewal tag to the customer and order
                        if ($name == 'giftToSub') {

                            $newOrderTag[]    = 'gift-renewed';
                            $newCustomerTag[] = 'gift-renewed';
                        }

                        // For giftee-order-notification event
                        if ($name == 'gift_senders_first_name') {
                            $giftSendersFirstName = trim($lineItemProperty->value);
                        }

                        if ($name == 'gift_senders_last_name') {
                            $giftSendersLastName = trim($lineItemProperty->value);
                        }

                        if ($name == 'gift_pack_name') {
                            $giftPackName = trim($lineItemProperty->value);
                        }

                        if ($name == 'charge_interval_frequency') {
                            $chargeIntervalFrequency = trim($lineItemProperty->value);
                        }

                    }
                }
            }

            // Check the order is a first gift order
            if (!empty($chargeIntervalFrequency) && !empty($giftOrderSuccessCount)) {

                $totalOrderCount = (int)$chargeIntervalFrequency / 28;
                $currentQueuedOrderCount = (int)$totalOrderCount - $giftOrderSuccessCount;

                if ((($totalOrderCount - 1) == $currentQueuedOrderCount) ||
                ($giftOrderQueuedCount == 0 && $giftOrderSuccessCount == 1)) {

                    $giftFirstOrderStatus = 1;
                    $newOrderTag[]    =  'unclaimed';
                }
            }

            if ($giftFirstOrderStatus == 1 && !empty($giftNotificationDate)) {
                $newOrderTag[]    =  'giftNotificationDate:'.$giftNotificationDate;
            }
            
            // Creating giftee-order-notification event
            if ($giftOrderStatus == 1 && $giftFirstOrderStatus == 0) {

                //$iterableModelObj = new IterableApi();
                $eventCreateRequest = [];

                $eventCreateRequest = array(
                    "email" => strtolower($giftRecipientEmail), 
                    "eventName" => "giftee-order-notification",
                    "dataFields" => array(
                        "gift_senders_first_name" => $giftSendersFirstName,
                        "gift_senders_last_name" => $giftSendersLastName,
                        "gift_pack_name" => $giftPackName,
                        "gift_pack_order_number" => $orderName
                    )
                );

                $eventCreatedResponse = $this->iterableObj->createIterableEvent(json_encode($eventCreateRequest));

                if (isset($eventCreatedResponse->code) || trim(strtolower($eventCreatedResponse->code)) == 'success') {

                    Log::channel('orderTag')->info("giftee-order-notification - Successfully created the event for Email (".
                    strtolower($giftRecipientEmail).")");
                } else {

                    Log::channel('orderTag')->info("giftee-order-notification - Failed to create the event for Email (".
                    strtolower($giftRecipientEmail).")");
                }
            }

            if ($oTagFlag > 0) {
                $newOrderTag[]    =  'utm_trimmed';
            }
            
            $newOrderTag    = implode(',',$newOrderTag);
            $newCustomerTag = implode(',',$newCustomerTag);

            $newTag = $orderTags.", ".$newOrderTag;
            $params = array(
                'order' => array(
                    'id' => $orderId,
                    'tags' => $newTag
                )
            );
            
            $method = 'PUT';
            $url = '/admin/api/'.$this->shopify_api_version.'/orders/'.$orderId.'.json';
           
            $retry = false;
            $try = 1;
            do {
                $result = $this->api->rest($method, $url, $params);
                if (!isset($result['body']['order'])) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while($retry && $try <=3);

            if (empty($result['errors']) && !empty($result['body']['order'])) { // success

                Log::channel('orderTag')->info("Successfully updated the order tags - ".$orderId);
                Log::channel('orderTag')->info(
                    "Order Id = ".$orderId.
                    " => OrderTag => ".$orderTags.
                    " => New tags => ". $newTag
                );
                if (!empty($newCustomerTag)){

                    $this->seTCustomerTag($api, $customerId, $customerTags, $newCustomerTag, $orderId);
                    $this->setIterableTag($orderEmail, $customerTags, $newCustomerTag);
		        }
               
            } else {

                Log::channel('orderTag')->info("Failed to update the order tags - ".$orderId);
                Log::channel('orderTag')->info(json_encode($result));
            }

        }

    }
    
    /**
     * Set customer tag
     */
    function seTCustomerTag($api, $customerId, $customerTags, $newCustomerTag, $orderId) {

        $newTag = $customerTags.", ".$newCustomerTag;
 
        $params = array(
            'customer' => array(
                'id' => $customerId,
                'tags' => $newTag
            )
        );

        $method = 'PUT';
        $url = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json';
        
        $retry = false;
	    $try = 1;
        do {
            $result = $this->api->rest($method, $url, $params);
            if (!isset($result['body']['customer']['id'])) {
                sleep(1);
                $retry = true;
                $try++;
            }
        } while($retry && $try <=3);
        
        Log::channel('orderTag')->info(json_encode($result));
        if (empty($result['errors']) && !empty($result['body']['customer'])) { // success

            Log::channel('orderTag')->info("Successfully updated the customer tags - ".$customerId." for the order ".$orderId);
            Log::channel('orderTag')->info(
                "Order Id = ".$orderId.
                " => Customer Id => ".$customerId.
                " => Customer tag => ".$customerTags.
                " => New tags => ". $newTag
            );
        } else {

            Log::channel('orderTag')->info("Failed to update the Customer tags - ".$customerId." for the order ".$orderId);
            Log::channel('orderTag')->info(json_encode($result));
        }
    }

    /**
     * Set iterable tag
     */
    function setIterableTag($email, $customerTags, $newCustomerTag) {

        //$iterableModel = new IterableApi();
        $userData = $this->iterableObj->getIterableSubscription($email);

        if (!empty($userData)) {

            Log::channel('orderTag')->info(json_encode($userData));
        }
 
        $newTag = $customerTags.", ".$newCustomerTag; 
        $iterableRequest = array(
            "email" => $email,
            "dataFields" => array(
                "tags" => $newTag
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($iterableRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

            Log::channel('orderTag')->info("Successfully updated the customer tags for the customer - ".$email);
        } else {

            Log::channel('orderTag')->info("Failed to update customer tags for the customer - ".$email);
            Log::channel('orderTag')->info(json_encode($userIterableUpdateResponse));
        }
    }
    
}
