<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use App\Models\IterableApi;
use Config;
use Log;

class ShipmentTrackingUpdateJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $webhookData;
    public $shopify_api_version;
    public $iterableObj;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhookDetails) {

        $this->webhookData = $webhookDetails;
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle() {

        $this->iterableObj = new IterableApi();

        $fulfillments = [];
        $customerEmail = '';
        $phoneNumber = '';

        if (isset($this->webhookData->fulfillments) && !empty($this->webhookData->fulfillments)) {
            $fulfillments  = $this->webhookData->fulfillments;
        }
        if (isset($this->webhookData->email) && !empty($this->webhookData->email)) {
            $customerEmail  = trim($this->webhookData->email);
        }
        if (isset($this->webhookData->shipping_address->phone) && !empty($this->webhookData->shipping_address->phone)) {
            $phoneNumber  = trim($this->webhookData->shipping_address->phone);
        }

        if ($phoneNumber == '' && isset($this->webhookData->customer->default_address->phone) &&
            !empty($this->webhookData->customer->phone->default_address->phone)) {
            $phoneNumber  = trim($this->webhookData->customer->phone->default_address->phone);
        }

        $shipmentTrackingURL = '';

        if (!empty($fulfillments)) {
            $shipmentTrackingURL = trim($fulfillments[0]->tracking_url);
        }


        if (!empty($customerEmail) && !empty($shipmentTrackingURL)) {

            $userIterableUpdateRequest = array(
                "email" => $customerEmail,
                "dataFields" => array(
                    "shipmentTrackingURL" => $shipmentTrackingURL,
                    "phoneNumber" => $phoneNumber
                ),
            );

            $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

            if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

                Log::channel('shipmentWebhookIterable')->info("Successfully updated tracking URL in iterable (Email:".$customerEmail.", 
                Tracking URL:".$shipmentTrackingURL.", PhoneNumber:".$phoneNumber.")");
            } else {
                Log::channel('shipmentWebhookIterable')->info("Failed to update tracking URL in iterable (Email:".$customerEmail.", 
                Tracking URL:".$shipmentTrackingURL.", PhoneNumber:".$phoneNumber.")");
            }

        } else {
            Log::channel('shipmentWebhookIterable')->info("Failed to update tracking URL in iterable(Missing parameters: Tracking URL, 
            PhoneNumber),(Email:".$customerEmail);
        }

    }

}
