<?php

namespace App\Models\Shopify;
use Illuminate\Database\Eloquent\Model;
/**
 * Class Product
 * @package model\shopify
 */
// $GLOBALS = array(
//     'apiRequestTime' => 0
// );
class Customer extends Model{

    private  $shopifyObj;
    public $shop_domain;
    private $shopify_api_version;
    private $token;
    private $api_key;
    private $secret;
    private $last_response_headers = null;
    protected $global;
    protected $apiCount;

    function __construct($type = NULL) {

        $this->shop_domain   = env('SHOPIFY_DOMAIN', true);
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);

        $type = !empty($type) ? trim($type) : "";
        if (empty($type)) {
            $type = "NULL";
        }
        $this->apiAllocation($type);
        $this->apiCount     = 0;
        global $GLOBALS;
        $this->global = array(
            'apiRequestTime' => 0
        );
    }

    /**
     * Shopify API allocation
     */
    public function apiAllocation($type) {

        switch ($type) {

            case (trim($type) == 'gift'):
                $this->api_key  = env('SHOPIFY_API_KEY_GIFT', true);
                $this->token    = env('SHOPIFY_PASSWORD_GIFT', true);
                $this->secret = env('SHOPIFY_SECRET_GIFT', true);
                break;

            case (trim($type) == 'getAllSubscriptions'):
                $this->api_key  = env('SHOPIFY_API_KEY_GET_ALL_SUBSCRIPTIONS', true);
                $this->token    = env('SHOPIFY_PASSWORD_GET_ALL_SUBSCRIPTIONS', true);
                $this->secret = env('SHOPIFY_SECRET_GET_ALL_SUBSCRIPTIONS', true);
                break;

            case (trim($type) == 'cronJobs'):
                $this->api_key  = env('SHOPIFY_API_KEY_CRON', true);
                $this->token    = env('SHOPIFY_PASSWORD_CRON', true);
                $this->secret = env('SHOPIFY_SECRET_CRON', true);
                break;

            default:
                $this->api_key  = env('SHOPIFY_API_KEY', true);
                $this->token    = env('SHOPIFY_PASSWORD', true);
                $this->secret = env('SHOPIFY_SECRET', true);
        }
    }

    /**
     * Create metafield in a customer
     */
    public function createCustomerMetaField($userID = null, $request = null){

        if ($userID && $request){

            $method = 'PUT';
            $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$userID.'.json';
            $params = $request;
            $response = $this->call($method, $path, $params);
            if ($response){

                return $response;
            }
        }
        return null;
    }

    /**
     * Get all metafields of a customer
     */
    public function getCustomerMetafields($userID = null) {

        if ($userID) {

            $method = 'GET';
            $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$userID.'/metafields.json';
            $params = '';
            $response = $this->call($method, $path, $params);
            if ($response) {
                return $response;
            }
        }
        return null;
    }

     /**
     * Update metafield of a customer
     */
    public function updateCustomerMetafields($userID = null, $metafieldID = null, $request = null) {

        if ($userID && $metafieldID && $request) {
            $method = 'PUT';
            $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$userID.'/metafields/'.$metafieldID.'.json';
            $params = $request;
            $response = $this->call($method, $path, $params);
            if ($response) {
                return $response;
            }
        }
        return null;
    }

    /**
     * Create a Shopify customer
     */
    public function createShopifyCustomer($customerCreateRequest, $apiType = NULL) {

        if (!empty($apiType)) {
            $this->apiAllocation($apiType);
        }
        $method = 'POST';
        $path   = '/admin/api/'.$this->shopify_api_version.'/customers.json';
        $params = $customerCreateRequest;
        $response = $this->call($method, $path, $params);
        if ($response) {
            return $response;
        }
        return null;
    }

    /**
     * Get details of a customer
     */
    public function getCustomerDetails($customerId = null) {

        $method = 'GET';
        $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json';
        $params = '';
        $response = $this->call($method, $path, $params);
        if ($response) {
            return $response;
        }
        return null;
    }

    /**
     * Create customer activation link
     */
    public function createAccountActivationLink($customerId, $apiType = NULL) {

        if (!empty($apiType)) {
            $this->apiAllocation($apiType);
        }
        $method = 'POST';
        $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'/account_activation_url.json';
        $response = $this->call($method, $path);
        if ($response) {
            return $response;
        }
        return null;
    }

    /**
     * Getting customerDetail using email id
     */
    function getCustomerDetailBySearch($email = null, $apiType = NULL) {

        if (!empty($apiType)) {
            $this->apiAllocation($apiType);
        }
        if ($email){
            $method = 'GET';
            $path   = '/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$email;
            $params = '';
            $customer = $this->call($method,$path);
            if ($customer){

                return $customer;
            }
        }
        return null;
    }

    /**
     * Update Shopify customer
     */
    public function updateShopifyCustomer($customerId, $request) {

        $method = 'PUT';
        $path   = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json';
        $response = $this->call($method, $path, $request);
        if ($response) {
            return $response;
        }
        return null;
    }

    public function callPaging($method, $path, $params=array(), $repeated = false)
    {
        $duration = 1;
        $apiLimit = 2;
        $errorMessage = '';
        if($this->global['apiRequestTime'] == 0)
        {
            $this->global['apiRequestTime'] = strtotime("now");
        }
        else{
            $timeDifference = strtotime("now")- $this->global['apiRequestTime'];
            if ($timeDifference< $duration)  {
                if ( $this->apiCount >= $apiLimit ) {
                    $sleepTime = (int)ceil($duration-$timeDifference);
                    sleep($sleepTime);  // this step will wait for next api request.
                    $this->global['apiRequestTime'] = strtotime("now");
                    $this->apiCount = 0;       // next cycle starts
                }
            }else {
                $this->global['apiRequestTime'] = strtotime("now");
                $this->apiCount = 0;
            }
        }
        $this->apiCount++;

        $baseurl = "https://{$this->shop_domain}/";

        if ($repeated) {
            $url = $path;
        } else {
            $url = $baseurl.ltrim($path, '/');
        }

        $query = in_array($method, array('GET','DELETE')) ? $params : array();
        $payload = in_array($method, array('POST','PUT')) ? json_encode($params) : array();
        $request_headers = in_array($method, array('POST','PUT')) ? array("Content-Type: application/json; charset=utf-8", 'Expect:') : array();

        // add auth headers
        $request_headers[] = 'X-Shopify-Access-Token: ' . $this->token;
        //sleep(1);
        $response = $this->curlHttpApiRequestPaging($method, $url, $query, $payload, $request_headers);
        list($message_headers, $message_body) = preg_split("/\r\n\r\n|\n\n|\r\r/", $response, 2);
        $this->last_response_headers = $this->curlParseHeaders($message_headers);

        $pageLink = !empty($this->last_response_headers['link']) ? $this->last_response_headers['link'] : "";

        if (!empty($pageLink)) {

            if (strpos($pageLink, 'rel="previous"') && strpos($pageLink, 'rel="next"')) {

                $all_urls = explode(", ", $pageLink);
                $nextUrl = $all_urls[1];
            }
            elseif (strpos($pageLink, 'rel="next"')){

                $nextUrl = $pageLink;
            } else {

                $nextUrl = '';
            }

            $nextUrl = explode(";", $nextUrl);
            $nextUrl = $nextUrl[0];
            $nextUrl = rtrim($nextUrl,">");
            $nextUrl = ltrim($nextUrl,"<");
        }

        $message_body = json_decode($message_body, true);
        if(!empty($message_body['errors']) || !empty($message_body['error'])) {
            $errorMessage = !empty($message_body['errors']) ? $message_body['errors']
            : $message_body['error'];
        }
        $message_body = (is_array($message_body) and (count($message_body) > 0)) ? array_shift($message_body) : $message_body;
        $response = array(
            "link" => !empty($nextUrl) ? $nextUrl : '',
            "body" => $message_body,
            "error" => $errorMessage
        );
        return $response;
    }

    private function curlHttpApiRequestPaging($method, $url, $query='', $payload='', $request_headers=array())
    {
        $url = $this->curlAppendQuery($url, $query);
        $ch = curl_init($url);
        $this->curlSetopts($ch, $method, $payload, $request_headers);
        $response = curl_exec($ch);
        $errno = curl_errno($ch);
        $error = curl_error($ch);
        curl_close($ch);

        if ($errno){
            if($this->apiCount <= 3){
             sleep(2);
             $this->apiCount++;
             self::curlHttpApiRequestPaging($method, $url, $query, $payload, $request_headers);
            }
            else{
                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => $error,
                );
                echo json_encode($response);
                die();
            }
        }
        return $response;
    }

    private function curlAppendQuery($url, $query)
    {
        if (empty($query)) return $url;
        if (is_array($query)) return "$url?".http_build_query($query);
        else return "$url?$query";
    }

    private function curlParseHeaders($message_headers)
    {
        $header_lines = preg_split("/\r\n|\n|\r/", $message_headers);
        $headers = array();
        $httpStatus = explode(' ', trim(array_shift($header_lines)), 3);
        if (count($httpStatus) == 2) {
            list($headers['http_status_code'], $headers['http_status_message']) = explode(' ', trim(array_shift($header_lines)), 3);
        } else {
            list(, $headers['http_status_code'], $headers['http_status_message']) = explode(' ', trim(array_shift($header_lines)), 3);
        }
        foreach ($header_lines as $header_line)
        {
            list($name, $value) = explode(':', $header_line, 2);
            $name = strtolower($name);
            $headers[$name] = trim($value);
        }

        return $headers;
    }

    private function curlSetopts($ch, $method, $payload, $request_headers)
    {
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_USERAGENT, 'ohShopify-php-api-client');
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 100);
        curl_setopt($ch, CURLOPT_TIMEOUT, 100);

        curl_setopt ($ch, CURLOPT_CUSTOMREQUEST, $method);
        if (!empty($request_headers)) curl_setopt($ch, CURLOPT_HTTPHEADER, $request_headers);

        if ($method != 'GET' && !empty($payload))
        {
            if (is_array($payload)) $payload = http_build_query($payload);
            curl_setopt ($ch, CURLOPT_POSTFIELDS, $payload);
        }
    }

     function call($method, $path, $params=array())
    {
        /****************api limit control**********************************/
        $duration = 1;
        $apiLimit = 2;
        if($this->global['apiRequestTime'] == 0)
        {
            $this->global['apiRequestTime'] = strtotime("now");
        }
        else{
            $timeDifference = strtotime("now")- $this->global['apiRequestTime'];
            if ($timeDifference< $duration)  {
                if ( $this->apiCount >= $apiLimit ) {
                    $sleepTime = (int)ceil($duration-$timeDifference);
                    sleep($sleepTime);  // this step will wait for next api request.
                    $this->global['apiRequestTime'] = strtotime("now");
                    $this->apiCount = 0;       // next cycle starts
                }
            }else {
                $this->global['apiRequestTime'] = strtotime("now");
                $this->apiCount = 0;
            }
        }
        $this->apiCount++;

           /**************************************************/
        $baseurl = "https://{$this->shop_domain}/";

        $url = $baseurl.ltrim($path, '/');
        $query = in_array($method, array('GET','DELETE')) ? $params : array();
        $payload = in_array($method, array('POST','PUT')) ? json_encode($params) : array();
        $request_headers = in_array($method, array('POST','PUT')) ? array("Content-Type: application/json; charset=utf-8", 'Expect:') : array();

        // add auth headers
        $request_headers[] = 'X-Shopify-Access-Token: ' . $this->token;

        $response = $this->curlHttpApiRequest($method, $url, $query, $payload, $request_headers);
        $response = json_decode($response, true);

        return $response;
    }

    private function curlHttpApiRequest($method, $url, $query='', $payload='', $request_headers=array())
    {

        $url = $this->curlAppendQuery($url, $query);
        $ch = curl_init($url);
        $this->curlSetopts($ch, $method, $payload, $request_headers);
        $response = curl_exec($ch);
        $errno = curl_errno($ch);
        $error = curl_error($ch);
        curl_close($ch);

       // if ($errno) throw new ShopifyCurlException($error, $errno);
       if ($errno){
           if($this->apiCount <= 3){
            sleep(2);
            $this->apiCount++;
            self::curlHttpApiRequest($method, $url, $query, $payload, $request_headers);
           } else {
               
                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => $error,
                );
                echo json_encode($response);
                die();
           }
       }
        list($message_headers, $message_body) = preg_split("/\r\n\r\n|\n\n|\r\r/", $response, 2);
        $this->last_response_headers = $this->curlParseHeaders($message_headers);

        return $message_body;
    }

}
