<?php

namespace App\Models\Shopify;
use Illuminate\Database\Eloquent\Model;
/**
 * Class Order
 * @package model\shopify
 */
// $GLOBALS = array(
//     'apiRequestTime' => 0
// );
class OrderCron extends Model{

    private  $shopifyObj ;
    public $shop_domain;
    private $shopify_api_version;
    private $token;
    private $api_key;
    private $secret;
    private $last_response_headers = null;
    protected $global;
    protected $apiCount;

    function __construct($type = NULL) {

        $this->api_key  = env('SHOPIFY_API_KEY_ORDER_TAGGING', true);
        $this->token    = env('SHOPIFY_PASSWORD_ORDER_TAGGING', true);
        $this->secret = env('SHOPIFY_SECRET_ORDER_TAGGING', true);
        $this->shop_domain   = env('SHOPIFY_DOMAIN', true);
        $this->shopify_api_version = env('SHOPIFY_API_VERSION_ORDER_TAGGING', true);
        $this->apiCount     = 0;
        global $GLOBALS;
        $this->global = array(
            'apiRequestTime' => 0
        );
    }

    /**
     * Update Shopify order
     */
    function updateOrder($orderDetails,$shopifyOrderId) {

        $method = 'PUT';
        $path   = '/admin/api/'.$this->shopify_api_version.'/orders/'.$shopifyOrderId.'.json';
        $params = $orderDetails;

        $response = $this->call($method, $path,$params);
        if($response){
            return $response;
        }
        return null;
    }

    /**
    * Getting shopify orders based on page no. and limit in api call
    */
    function getShopifyOrders($limit = NULL, $url = NULL, $created_at = NULL) {

        $method = 'GET';
        // $this->token = env('SHOPIFY_PASSWORD2', true);

        $path   = '/admin/api/'.$this->shopify_api_version.'/orders.json?limit='.$limit.'&status=any';
        if($created_at != NULL){
            $path.='&created_at_min='.$created_at;
        }

        if ($url != NULL) {
            $response = $this->callPaging($method, $url,[], true);
        } else {
            $response = $this->callPaging($method, $path);
        }
        if($response){
            return  $response;
        }
        return null;
    }


    public function callPaging($method, $path, $params=array(), $repeated = false) {

        $duration = 1;
        $apiLimit = 2;
        $errorMessage = '';
        if($this->global['apiRequestTime'] == 0)
        {
            $this->global['apiRequestTime'] = strtotime("now");
        }
        else{
            $timeDifference = strtotime("now")- $this->global['apiRequestTime'];
            if ($timeDifference< $duration)  {
                if ( $this->apiCount >= $apiLimit ) {
                    $sleepTime = (int)ceil($duration-$timeDifference);
                    sleep($sleepTime);  // this step will wait for next api request.
                    $this->global['apiRequestTime'] = strtotime("now");
                    $this->apiCount = 0;       // next cycle starts
                }
            }else {
                $this->global['apiRequestTime'] = strtotime("now");
                $this->apiCount = 0;
            }
        }
        $this->apiCount++;

        $baseurl = "https://{$this->shop_domain}/";

        if ($repeated) {
            $url = $path;
        } else {
            $url = $baseurl.ltrim($path, '/');
        }

        $query = in_array($method, array('GET','DELETE')) ? $params : array();
        $payload = in_array($method, array('POST','PUT')) ? json_encode($params) : array();
        $request_headers = in_array($method, array('POST','PUT')) ? array("Content-Type: application/json; charset=utf-8", 'Expect:') : array();

        // add auth headers
        $request_headers[] = 'X-Shopify-Access-Token: ' . $this->token;
        //sleep(1);
        $response = $this->curlHttpApiRequestPaging($method, $url, $query, $payload, $request_headers);
        list($message_headers, $message_body) = preg_split("/\r\n\r\n|\n\n|\r\r/", $response, 2);
        $this->last_response_headers = $this->curlParseHeaders($message_headers);

        $pageLink = !empty($this->last_response_headers['link']) ? $this->last_response_headers['link'] : "";

        if (!empty($pageLink)) {

            if (strpos($pageLink, 'rel="previous"') && strpos($pageLink, 'rel="next"')) {

                $all_urls = explode(", ", $pageLink);
                $nextUrl = $all_urls[1];
            }
            elseif (strpos($pageLink, 'rel="next"')){

                $nextUrl = $pageLink;
            } else {

                $nextUrl = '';
            }

            $nextUrl = explode(";", $nextUrl);
            $nextUrl = $nextUrl[0];
            $nextUrl = rtrim($nextUrl,">");
            $nextUrl = ltrim($nextUrl,"<");
        }

        $message_body = json_decode($message_body, true);
        if(!empty($message_body['errors']) || !empty($message_body['error'])) {
            $errorMessage = !empty($message_body['errors']) ? $message_body['errors']
            : $message_body['error'];
        }
        $message_body = (is_array($message_body) and (count($message_body) > 0)) ? array_shift($message_body) : $message_body;
        $response = array(
            "link" => !empty($nextUrl) ? $nextUrl : '',
            "body" => $message_body,
            "error" => $errorMessage
        );
        return $response;
    }

    private function curlHttpApiRequestPaging($method, $url, $query='', $payload='', $request_headers=array())
    {
        $url = $this->curlAppendQuery($url, $query);
        $ch = curl_init($url);
        $this->curlSetopts($ch, $method, $payload, $request_headers);
        $response = curl_exec($ch);
        $errno = curl_errno($ch);
        $error = curl_error($ch);
        curl_close($ch);

        if ($errno){
            if($this->apiCount <= 3){
             sleep(2);
             $this->apiCount++;
             self::curlHttpApiRequestPaging($method, $url, $query, $payload, $request_headers);
            }
            else{
                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => $error,
                );
                echo json_encode($response);
                die();
            }
        }
        return $response;
    }

    private function curlAppendQuery($url, $query)
    {
        if (empty($query)) return $url;
        if (is_array($query)) return "$url?".http_build_query($query);
        else return "$url?$query";
    }

    private function curlParseHeaders($message_headers)
    {
        $header_lines = preg_split("/\r\n|\n|\r/", $message_headers);
        $headers = array();
        $httpStatus = explode(' ', trim(array_shift($header_lines)), 3);
        if (count($httpStatus) == 2) {
            list($headers['http_status_code'], $headers['http_status_message']) = explode(' ', trim(array_shift($header_lines)), 3);
        } else {
            list(, $headers['http_status_code'], $headers['http_status_message']) = explode(' ', trim(array_shift($header_lines)), 3);
        }
        foreach ($header_lines as $header_line)
        {
            list($name, $value) = explode(':', $header_line, 2);
            $name = strtolower($name);
            $headers[$name] = trim($value);
        }

        return $headers;
    }

    private function curlSetopts($ch, $method, $payload, $request_headers)
    {
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_USERAGENT, 'ohShopify-php-api-client');
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 100);
        curl_setopt($ch, CURLOPT_TIMEOUT, 100);

        curl_setopt ($ch, CURLOPT_CUSTOMREQUEST, $method);
        if (!empty($request_headers)) curl_setopt($ch, CURLOPT_HTTPHEADER, $request_headers);

        if ($method != 'GET' && !empty($payload))
        {
            if (is_array($payload)) $payload = http_build_query($payload);
            curl_setopt ($ch, CURLOPT_POSTFIELDS, $payload);
        }
    }

     function call($method, $path, $params=array())
    {
        /****************api limit control**********************************/
        $duration = 1;
        $apiLimit = 2;
        if($this->global['apiRequestTime'] == 0)
        {
            $this->global['apiRequestTime'] = strtotime("now");
        }
        else{
            $timeDifference = strtotime("now")- $this->global['apiRequestTime'];
            if ($timeDifference< $duration)  {
                if ( $this->apiCount >= $apiLimit ) {
                    $sleepTime = (int)ceil($duration-$timeDifference);
                    sleep($sleepTime);  // this step will wait for next api request.
                    $this->global['apiRequestTime'] = strtotime("now");
                    $this->apiCount = 0;       // next cycle starts
                }
            }else {
                $this->global['apiRequestTime'] = strtotime("now");
                $this->apiCount = 0;
            }
        }
        $this->apiCount++;

           /**************************************************/
        $baseurl = "https://{$this->shop_domain}/";

        $url = $baseurl.ltrim($path, '/');
        $query = in_array($method, array('GET','DELETE')) ? $params : array();
        $payload = in_array($method, array('POST','PUT')) ? json_encode($params) : array();
        $request_headers = in_array($method, array('POST','PUT')) ? array("Content-Type: application/json; charset=utf-8", 'Expect:') : array();

        // add auth headers
        $request_headers[] = 'X-Shopify-Access-Token: ' . $this->token;

        $response = $this->curlHttpApiRequest($method, $url, $query, $payload, $request_headers);
        $response = json_decode($response, true);

        return $response;
    }

    private function curlHttpApiRequest($method, $url, $query='', $payload='', $request_headers=array())
    {

        $url = $this->curlAppendQuery($url, $query);
        $ch = curl_init($url);
        $this->curlSetopts($ch, $method, $payload, $request_headers);
        $response = curl_exec($ch);
        $errno = curl_errno($ch);
        $error = curl_error($ch);
        curl_close($ch);

       // if ($errno) throw new ShopifyCurlException($error, $errno);
       if ($errno){
           if($this->apiCount <= 3){
            sleep(2);
            $this->apiCount++;
            self::curlHttpApiRequest($method, $url, $query, $payload, $request_headers);
           }
           else{
                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => $error,
                );
                echo json_encode($response);
                die();
           }
       }
        list($message_headers, $message_body) = preg_split("/\r\n\r\n|\n\n|\r\r/", $response, 2);
        $this->last_response_headers = $this->curlParseHeaders($message_headers);

        return $message_body;
    }

}
