<?php

use Monolog\Handler\NullHandler;
use Monolog\Handler\StreamHandler;
use Monolog\Handler\SyslogUdpHandler;

return [

    /*
    |--------------------------------------------------------------------------
    | Default Log Channel
    |--------------------------------------------------------------------------
    |
    | This option defines the default log channel that gets used when writing
    | messages to the logs. The name specified in this option should match
    | one of the channels defined in the "channels" configuration array.
    |
    */

    'default' => env('LOG_CHANNEL', 'stack'),

    /*
    |--------------------------------------------------------------------------
    | Deprecations Log Channel
    |--------------------------------------------------------------------------
    |
    | This option controls the log channel that should be used to log warnings
    | regarding deprecated PHP and library features. This allows you to get
    | your application ready for upcoming major versions of dependencies.
    |
    */

    'deprecations' => env('LOG_DEPRECATIONS_CHANNEL', 'null'),

    /*
    |--------------------------------------------------------------------------
    | Log Channels
    |--------------------------------------------------------------------------
    |
    | Here you may configure the log channels for your application. Out of
    | the box, Laravel uses the Monolog PHP logging library. This gives
    | you a variety of powerful log handlers / formatters to utilize.
    |
    | Available Drivers: "single", "daily", "slack", "syslog",
    |                    "errorlog", "monolog",
    |                    "custom", "stack"
    |
    */

    'channels' => [
        'stack' => [
            'driver' => 'stack',
            'channels' => ['single'],
            'ignore_exceptions' => false,
        ],

        'single' => [
            'driver' => 'single',
            'path' => storage_path('logs/laravel.log'),
            'level' => env('LOG_LEVEL', 'debug'),
        ],

        'daily' => [
            'driver' => 'daily',
            'path' => storage_path('logs/laravel.log'),
            'level' => env('LOG_LEVEL', 'debug'),
            'days' => 14,
        ],

        'slack' => [
            'driver' => 'slack',
            'url' => env('LOG_SLACK_WEBHOOK_URL'),
            'username' => 'Laravel Log',
            'emoji' => ':boom:',
            'level' => env('LOG_LEVEL', 'critical'),
        ],

        'email' => [
            'driver' => 'daily',
            'path' => storage_path('logs/email.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
	    ],
        
        'cron' => [
             'driver' => 'daily',
             'path' => storage_path('logs/cron.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
        
        'abandonedCart' => [
            'driver' => 'daily',
            'path' => storage_path('logs/abandonedCart.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'quizControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/quizControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'commonControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/commonControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'customerControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/customerControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionRefundControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionRefundControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'createShopifyCustomerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/createShopifyCustomerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'generateUserTokenLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/generateUserTokenLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'recommendationToIterableLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/recommendationToIterableLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        
        'shipmentDeliveryTagging' => [
            'driver' => 'daily',
            'path' => storage_path('logs/shipmentDeliveryTagging.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionDiscountCron' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionDiscountCron.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionMetaUpdate' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionMetaUpdate.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderCancelRefundEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderCancelRefundEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderCancelRerouteEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderCancelRerouteEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderExtractedTagStatusCheckEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderExtractedTagStatusCheckEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'giftPackLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftPackLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'giftCronLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftCronLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'gifteeCheckoutLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/gifteeCheckoutLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'ePcApiCallLogging' => [
            'driver' => 'daily',
            'path' => storage_path('logs/ePcApiCallLogging.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'quizCartOffLoadLogging' => [
            'driver' => 'daily',
            'path' => storage_path('logs/quizCartOffLoadLogging.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionsAndOrderHandlingLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionsAndOrderHandlingLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'giftRecipientPasswordRestStatusLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftRecipientPasswordRestStatusLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderStatusNotificationLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderStatusNotificationLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'automatedOrderCancelAndRefundLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/automatedOrderCancelAndRefundLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'upcomingChargeValidityCheckingLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/upcomingChargeValidityCheckingLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'newsLetterEmailLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/newsLetterEmailLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
         'ohMyBrew' => [
             'driver' => 'daily',
             'path' => storage_path('logs/ohMyBrew.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
         'orderTag' => [
             'driver' => 'daily',
             'path' => storage_path('logs/orderTagging.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
         'shipmentWebhookIterable' => [
            'driver' => 'daily',
            'path' => storage_path('logs/shipmentWebhookIterable.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'gorgiasCronLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/gorgiasCronLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'papertrail' => [
            'driver' => 'monolog',
            'level' => env('LOG_LEVEL', 'debug'),
            'handler' => env('LOG_PAPERTRAIL_HANDLER', SyslogUdpHandler::class),
            'handler_with' => [
                'host' => env('PAPERTRAIL_URL'),
                'port' => env('PAPERTRAIL_PORT'),
                'connectionString' => 'tls://'.env('PAPERTRAIL_URL').':'.env('PAPERTRAIL_PORT'),
            ],
        ],

        'stderr' => [
            'driver' => 'monolog',
            'level' => env('LOG_LEVEL', 'debug'),
            'handler' => StreamHandler::class,
            'formatter' => env('LOG_STDERR_FORMATTER'),
            'with' => [
                'stream' => 'php://stderr',
            ],
        ],

        'syslog' => [
            'driver' => 'syslog',
            'level' => env('LOG_LEVEL', 'debug'),
        ],

        'errorlog' => [
            'driver' => 'errorlog',
            'level' => env('LOG_LEVEL', 'debug'),
        ],

        'null' => [
            'driver' => 'monolog',
            'handler' => NullHandler::class,
        ],

        'emergency' => [
            'path' => storage_path('logs/laravel.log'),
        ],
    ],

];
